@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,27 +9,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#ifndef EXTENSIONSYSTEM_PLUGINMANAGER_H
-#define EXTENSIONSYSTEM_PLUGINMANAGER_H
+#pragma once
 
 #include "extensionsystem_global.h"
 #include <aggregation/aggregate.h>
@@ -43,7 +37,6 @@ class QSettings;
 QT_END_NAMESPACE
 
 namespace ExtensionSystem {
-class PluginCollection;
 class IPlugin;
 class PluginSpec;
 
@@ -89,6 +82,8 @@ public:
         }
         return results;
     }
+
+    // This is useful for soft dependencies using pure interfaces.
     template <typename T> static T *getObject()
     {
         QReadLocker lock(listLock());
@@ -119,11 +114,13 @@ public:
     static void loadPlugins();
     static QStringList pluginPaths();
     static void setPluginPaths(const QStringList &paths);
-    static QList<PluginSpec *> plugins();
-    static QHash<QString, PluginCollection *> pluginCollections();
-    static void setFileExtension(const QString &extension);
-    static QString fileExtension();
+    static QString pluginIID();
+    static void setPluginIID(const QString &iid);
+    static const QList<PluginSpec *> plugins();
+    static QHash<QString, QList<PluginSpec *>> pluginCollections();
     static bool hasError();
+    static QSet<PluginSpec *> pluginsRequiringPlugin(PluginSpec *spec);
+    static QSet<PluginSpec *> pluginsRequiredByPlugin(PluginSpec *spec);
 
     // Settings
     static void setSettings(QSettings *settings);
@@ -145,28 +142,27 @@ public:
     static QString serializedArguments();
 
     static bool testRunRequested();
-    static QString testDataDirectory();
 
     static void profilingReport(const char *what, const PluginSpec *spec = 0);
 
     static QString platformName();
 
+    static bool isInitializationDone();
+
+    void remoteArguments(const QString &serializedArguments, QObject *socket);
+    void shutdown();
+
+    QString systemInformation() const;
+
 signals:
     void objectAdded(QObject *obj);
     void aboutToRemoveObject(QObject *obj);
 
     void pluginsChanged();
     void initializationDone();
+    void testsFinished(int failedTests);
 
-public slots:
-    void remoteArguments(const QString &serializedArguments, QObject *socket);
-    void shutdown();
-
-private slots:
-    void startTests();
     friend class Internal::PluginManagerPrivate;
 };
 
 } // namespace ExtensionSystem
-
-#endif // EXTENSIONSYSTEM_PLUGINMANAGER_H
