@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,21 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -107,7 +103,7 @@ public:
 
     const Value *value() const { return m_value; }
 
-    virtual bool processProperty(const QString &name, const Value *value)
+    virtual bool processProperty(const QString &name, const Value *value, const PropertyInfo &)
     {
         return process(name, value);
     }
@@ -136,40 +132,37 @@ public:
 } // end of anonymous namespace
 
 namespace QmlJS {
-namespace Internal {
-class MetaFunction: public FunctionValue
+
+MetaFunction::MetaFunction(const FakeMetaMethod &method, ValueOwner *valueOwner)
+    : FunctionValue(valueOwner), m_method(method)
 {
-    FakeMetaMethod m_method;
+}
 
-public:
-    MetaFunction(const FakeMetaMethod &method, ValueOwner *valueOwner)
-        : FunctionValue(valueOwner), m_method(method)
-    {
-    }
+int MetaFunction::namedArgumentCount() const
+{
+    return m_method.parameterNames().size();
+}
 
-    virtual int namedArgumentCount() const
-    {
-        return m_method.parameterNames().size();
-    }
+QString MetaFunction::argumentName(int index) const
+{
+    if (index < m_method.parameterNames().size())
+        return m_method.parameterNames().at(index);
 
-    virtual QString argumentName(int index) const
-    {
-        if (index < m_method.parameterNames().size())
-            return m_method.parameterNames().at(index);
+    return FunctionValue::argumentName(index);
+}
 
-        return FunctionValue::argumentName(index);
-    }
-
-    virtual bool isVariadic() const
-    {
-        return false;
-    }
-    const MetaFunction *asMetaFunction() const
-    {
-        return this;
-    }
-};
-} // namespace Internal
+bool MetaFunction::isVariadic() const
+{
+    return false;
+}
+const MetaFunction *MetaFunction::asMetaFunction() const
+{
+    return this;
+}
+const FakeMetaMethod &MetaFunction::fakeMetaMethod() const
+{
+    return m_method;
+}
 
 FakeMetaObjectWithOrigin::FakeMetaObjectWithOrigin(FakeMetaObject::ConstPtr fakeMetaObject, const QString &originId)
     : fakeMetaObject(fakeMetaObject)
@@ -186,6 +179,49 @@ uint qHash(const FakeMetaObjectWithOrigin &fmoo)
     return qHash(fmoo.fakeMetaObject);
 }
 
+PropertyInfo::PropertyInfo(uint flags)
+    : flags(flags)
+{ }
+
+QString PropertyInfo::toString() const
+{
+    QStringList list;
+    if (isReadable())
+        list.append("Readable");
+
+    if (isWriteable())
+        list.append("Writeable");
+
+    if (isList())
+        list.append("ListType");
+
+    if (canBePointer())
+        list.append("Pointer");
+
+    if (canBeValue())
+        list.append("Value");
+
+    return list.join('|');
+}
+
+static QList<CustomImportsProvider *> g_customImportProviders;
+
+CustomImportsProvider::CustomImportsProvider(QObject *parent)
+    : QObject(parent)
+{
+    g_customImportProviders.append(this);
+}
+
+CustomImportsProvider::~CustomImportsProvider()
+{
+    g_customImportProviders.removeOne(this);
+}
+
+const QList<CustomImportsProvider *> CustomImportsProvider::allProviders()
+{
+    return g_customImportProviders;
+}
+
 } // namespace QmlJS
 
 CppComponentValue::CppComponentValue(FakeMetaObject::ConstPtr metaObject, const QString &className,
@@ -209,13 +245,8 @@ CppComponentValue::CppComponentValue(FakeMetaObject::ConstPtr metaObject, const
 
 CppComponentValue::~CppComponentValue()
 {
-#if QT_VERSION >= 0x050000
     delete m_metaSignatures.load();
     delete m_signalScopes.load();
-#else
-    delete m_metaSignatures;
-    delete m_signalScopes;
-#endif
 }
 
 static QString generatedSlotName(const QString &base)
@@ -253,23 +284,15 @@ void CppComponentValue::processMembers(MemberProcessor *processor) const
     QSet<QString> explicitSignals;
 
     // make MetaFunction instances lazily when first needed
-#if QT_VERSION >= 0x050000
     QList<const Value *> *signatures = m_metaSignatures.load();
-#else
-    QList<const Value *> *signatures = m_metaSignatures;
-#endif
     if (!signatures) {
         signatures = new QList<const Value *>;
         signatures->reserve(m_metaObject->methodCount());
         for (int index = 0; index < m_metaObject->methodCount(); ++index)
-            signatures->append(new Internal::MetaFunction(m_metaObject->method(index), valueOwner()));
+            signatures->append(new MetaFunction(m_metaObject->method(index), valueOwner()));
         if (!m_metaSignatures.testAndSetOrdered(0, signatures)) {
             delete signatures;
-#if QT_VERSION >= 0x050000
             signatures = m_metaSignatures.load();
-#else
-            signatures = m_metaSignatures;
-#endif
         }
     }
 
@@ -303,7 +326,17 @@ void CppComponentValue::processMembers(MemberProcessor *processor) const
             continue;
 
         const QString propertyName = prop.name();
-        processor->processProperty(propertyName, valueForCppName(prop.typeName()));
+        uint propertyFlags = PropertyInfo::Readable;
+        if (isWritable(propertyName))
+            propertyFlags |= PropertyInfo::Writeable;
+        if (isListProperty(propertyName))
+            propertyFlags |= PropertyInfo::ListType;
+        if (isPointer(propertyName))
+            propertyFlags |= PropertyInfo::PointerType;
+        else
+            propertyFlags |= PropertyInfo::ValueType;
+        processor->processProperty(propertyName, valueForCppName(prop.typeName()),
+                                   PropertyInfo(propertyFlags));
 
         // every property always has a onXyzChanged slot, even if the NOTIFY
         // signal has a different name
@@ -487,11 +520,7 @@ const QmlEnumValue *CppComponentValue::getEnumValue(const QString &typeName, con
 
 const ObjectValue *CppComponentValue::signalScope(const QString &signalName) const
 {
-#if QT_VERSION >= 0x050000
     QHash<QString, const ObjectValue *> *scopes = m_signalScopes.load();
-#else
-    QHash<QString, const ObjectValue *> *scopes = m_signalScopes;
-#endif
     if (!scopes) {
         scopes = new QHash<QString, const ObjectValue *>;
         // usually not all methods are signals
@@ -517,11 +546,7 @@ const ObjectValue *CppComponentValue::signalScope(const QString &signalName) con
         }
         if (!m_signalScopes.testAndSetOrdered(0, scopes)) {
             delete scopes;
-#if QT_VERSION >= 0x050000
             scopes = m_signalScopes.load();
-#else
-            scopes = m_signalScopes;
-#endif
         }
     }
 
@@ -801,7 +826,7 @@ const Function *Value::asFunction() const
     return 0;
 }
 
-const Internal::MetaFunction *Value::asMetaFunction() const
+const MetaFunction *Value::asMetaFunction() const
 {
     return 0;
 }
@@ -951,7 +976,7 @@ MemberProcessor::~MemberProcessor()
 {
 }
 
-bool MemberProcessor::processProperty(const QString &, const Value *)
+bool MemberProcessor::processProperty(const QString &, const Value *, const PropertyInfo &)
 {
     return true;
 }
@@ -1024,7 +1049,12 @@ void ObjectValue::setPrototype(const Value *prototype)
 
 void ObjectValue::setMember(const QString &name, const Value *value)
 {
-    m_members[name] = value;
+    m_members[name].value = value;
+}
+
+void ObjectValue::setPropertyInfo(const QString &name, const PropertyInfo &propertyInfo)
+{
+    m_members[name].propertyInfo = propertyInfo;
 }
 
 void ObjectValue::removeMember(const QString &name)
@@ -1063,12 +1093,12 @@ bool ObjectValue::checkPrototype(const ObjectValue *, QSet<const ObjectValue *>
 
 void ObjectValue::processMembers(MemberProcessor *processor) const
 {
-    QHashIterator<QString, const Value *> it(m_members);
+    QHashIterator<QString, PropertyData> it(m_members);
 
     while (it.hasNext()) {
         it.next();
 
-        if (! processor->processProperty(it.key(), it.value()))
+        if (! processor->processProperty(it.key(), it.value().value, it.value().propertyInfo))
             break;
     }
 }
@@ -1077,7 +1107,7 @@ const Value *ObjectValue::lookupMember(const QString &name, const Context *conte
                                        const ObjectValue **foundInObject,
                                        bool examinePrototypes) const
 {
-    if (const Value *m = m_members.value(name)) {
+    if (const Value *m = m_members.value(name).value) {
         if (foundInObject)
             *foundInObject = this;
         return m;
@@ -1091,11 +1121,13 @@ const Value *ObjectValue::lookupMember(const QString &name, const Context *conte
         }
     }
 
+    const ObjectValue *prototypeObject = 0;
+
     if (examinePrototypes && context) {
         PrototypeIterator iter(this, context);
         iter.next(); // skip this
         while (iter.hasNext()) {
-            const ObjectValue *prototypeObject = iter.next();
+            prototypeObject = iter.next();
             if (const Value *m = prototypeObject->lookupMember(name, context, foundInObject, false))
                 return m;
         }
@@ -1103,6 +1135,7 @@ const Value *ObjectValue::lookupMember(const QString &name, const Context *conte
 
     if (foundInObject)
         *foundInObject = 0;
+
     return 0;
 }
 
@@ -1320,6 +1353,7 @@ CppQmlTypesLoader::BuiltinObjects CppQmlTypesLoader::defaultQtObjects;
 CppQmlTypesLoader::BuiltinObjects CppQmlTypesLoader::loadQmlTypes(const QFileInfoList &qmlTypeFiles, QStringList *errors, QStringList *warnings)
 {
     QHash<QString, FakeMetaObject::ConstPtr> newObjects;
+    QStringList newDependencies;
 
     foreach (const QFileInfo &qmlTypeFile, qmlTypeFiles) {
         QString error, warning;
@@ -1329,7 +1363,8 @@ CppQmlTypesLoader::BuiltinObjects CppQmlTypesLoader::loadQmlTypes(const QFileInf
             file.close();
 
 
-            parseQmlTypeDescriptions(contents, &newObjects, 0, &error, &warning, qmlTypeFile.absoluteFilePath());
+            parseQmlTypeDescriptions(contents, &newObjects, 0, &newDependencies, &error, &warning,
+                                     qmlTypeFile.absoluteFilePath());
         } else {
             error = file.errorString();
         }
@@ -1351,26 +1386,27 @@ CppQmlTypesLoader::BuiltinObjects CppQmlTypesLoader::loadQmlTypes(const QFileInf
 void CppQmlTypesLoader::parseQmlTypeDescriptions(const QByteArray &contents,
                                                  BuiltinObjects *newObjects,
                                                  QList<ModuleApiInfo> *newModuleApis,
+                                                 QStringList *newDependencies,
                                                  QString *errorMessage,
                                                  QString *warningMessage, const QString &fileName)
 {
-    if (!contents.isEmpty()) {
-        unsigned char c = contents.at(0);
-        switch (c) {
-        case 0xfe:
-        case 0xef:
-        case 0xff:
-        case 0xee:
-        case 0x00:
-            qWarning() << QApplication::translate("CppQmlTypesLoader", "%1 seems not to be encoded in UTF8 or has a BOM.").arg(fileName);
-        default: break;
-        }
+    if (contents.isEmpty())
+        return;
+    unsigned char c = contents.at(0);
+    switch (c) {
+    case 0xfe:
+    case 0xef:
+    case 0xff:
+    case 0xee:
+    case 0x00:
+        qWarning() << fileName << "seems not to be encoded in UTF8 or has a BOM.";
+    default: break;
     }
 
     errorMessage->clear();
     warningMessage->clear();
     TypeDescriptionReader reader(fileName, QString::fromUtf8(contents));
-    if (!reader(newObjects, newModuleApis)) {
+    if (!reader(newObjects, newModuleApis, newDependencies)) {
         if (reader.errorMessage().isEmpty())
             *errorMessage = QLatin1String("unknown error");
         else
@@ -1422,9 +1458,10 @@ void CppQmlTypes::load(const QString &originId, const T &fakeMetaObjects, const
             object->setPrototype(proto);
     }
 }
+
 // explicitly instantiate load for list and hash
-template void CppQmlTypes::load< QList<FakeMetaObject::ConstPtr> >(const QString &, const QList<FakeMetaObject::ConstPtr> &, const QString &);
-template void CppQmlTypes::load< QHash<QString, FakeMetaObject::ConstPtr> >(const QString &, const QHash<QString, FakeMetaObject::ConstPtr> &, const QString &);
+template QMLJS_EXPORT void CppQmlTypes::load< QList<FakeMetaObject::ConstPtr> >(const QString &, const QList<FakeMetaObject::ConstPtr> &, const QString &);
+template QMLJS_EXPORT void CppQmlTypes::load< QHash<QString, FakeMetaObject::ConstPtr> >(const QString &, const QHash<QString, FakeMetaObject::ConstPtr> &, const QString &);
 
 QList<const CppComponentValue *> CppQmlTypes::createObjectsForImport(const QString &package, ComponentVersion version)
 {
@@ -1456,6 +1493,7 @@ QList<const CppComponentValue *> CppQmlTypes::createObjectsForImport(const QStri
         if (m_objectsByQualifiedName.contains(key))
             continue;
 
+        ComponentVersion cppVersion;
         foreach (const FakeMetaObject::Export &bestExport, bestExports) {
             QString name = bestExport.type;
             bool exported = true;
@@ -1470,7 +1508,10 @@ QList<const CppComponentValue *> CppQmlTypes::createObjectsForImport(const QStri
                         fmoo.originId);
 
             // use package.cppname importversion as key
-            m_objectsByQualifiedName.insert(key, newComponent);
+            if (cppVersion <= bestExport.version) {
+                cppVersion = bestExport.version;
+                m_objectsByQualifiedName.insert(key, newComponent);
+            }
             if (exported) {
                 if (!exportedObjects.contains(name) // we might have the same type in different versions
                         || (newComponent->componentVersion() > exportedObjects.value(name)->componentVersion()))
@@ -1815,7 +1856,7 @@ ASTObjectValue::ASTObjectValue(UiQualifiedId *typeName,
         for (UiObjectMemberList *it = m_initializer->members; it; it = it->next) {
             UiObjectMember *member = it->member;
             if (UiPublicMember *def = cast<UiPublicMember *>(member)) {
-                if (def->type == UiPublicMember::Property && !def->name.isEmpty() && !def->memberType.isEmpty()) {
+                if (def->type == UiPublicMember::Property && !def->name.isEmpty() && def->isValid()) {
                     ASTPropertyReference *ref = new ASTPropertyReference(def, m_doc, valueOwner);
                     m_properties.append(ref);
                     if (def->defaultToken.isValid())
@@ -1849,7 +1890,10 @@ bool ASTObjectValue::getSourceLocation(QString *fileName, int *line, int *column
 void ASTObjectValue::processMembers(MemberProcessor *processor) const
 {
     foreach (ASTPropertyReference *ref, m_properties) {
-        processor->processProperty(ref->ast()->name.toString(), ref);
+        uint pFlags = PropertyInfo::Readable;
+        if (!ref->ast()->isReadonlyMember)
+            pFlags |= PropertyInfo::Writeable;
+        processor->processProperty(ref->ast()->name.toString(), ref, PropertyInfo(pFlags));
         // ### Should get a different value?
         processor->processGeneratedSlot(ref->onChangedSlotName(), ref);
     }
@@ -2039,6 +2083,11 @@ UiQualifiedId *QmlPrototypeReference::qmlTypeName() const
     return m_qmlTypeName;
 }
 
+const Document *QmlPrototypeReference::document() const
+{
+    return m_doc;
+}
+
 const Value *QmlPrototypeReference::value(ReferenceContext *referenceContext) const
 {
     return referenceContext->context()->lookupType(m_doc, m_qmlTypeName);
@@ -2072,10 +2121,10 @@ bool ASTPropertyReference::getSourceLocation(QString *fileName, int *line, int *
 const Value *ASTPropertyReference::value(ReferenceContext *referenceContext) const
 {
     if (m_ast->statement
-            && (m_ast->memberType.isEmpty()
-                || m_ast->memberType == QLatin1String("variant")
-                || m_ast->memberType == QLatin1String("var")
-                || m_ast->memberType == QLatin1String("alias"))) {
+            && (!m_ast->isValid()
+                || m_ast->memberTypeName() == QLatin1String("variant")
+                || m_ast->memberTypeName() == QLatin1String("var")
+                || m_ast->memberTypeName() == QLatin1String("alias"))) {
 
         // Adjust the context for the current location - expensive!
         // ### Improve efficiency by caching the 'use chain' constructed in ScopeBuilder.
@@ -2091,7 +2140,7 @@ const Value *ASTPropertyReference::value(ReferenceContext *referenceContext) con
         return evaluator(m_ast->statement);
     }
 
-    const QString memberType = m_ast->memberType.toString();
+    const QString memberType = m_ast->memberTypeName().toString();
 
     const Value *builtin = valueOwner()->defaultValueForBuiltinType(memberType);
     if (!builtin->asUndefinedValue())
@@ -2115,7 +2164,7 @@ ASTSignal::ASTSignal(UiPublicMember *ast, const Document *doc, ValueOwner *value
     ObjectValue *v = valueOwner->newObject(/*prototype=*/0);
     for (UiParameterList *it = ast->parameters; it; it = it->next) {
         if (!it->name.isEmpty())
-            v->setMember(it->name.toString(), valueOwner->defaultValueForBuiltinType(it->type.toString()));
+            v->setMember(it->name.toString(), valueOwner->defaultValueForBuiltinType(it->type->name.toString()));
     }
     m_bodyScope = v;
 }
@@ -2142,9 +2191,9 @@ const Value *ASTSignal::argument(int index) const
     UiParameterList *param = m_ast->parameters;
     for (int i = 0; param && i < index; ++i)
         param = param->next;
-    if (!param || param->type.isEmpty())
+    if (!param || param->type->name.isEmpty())
         return valueOwner()->unknownValue();
-    return valueOwner()->defaultValueForBuiltinType(param->type.toString());
+    return valueOwner()->defaultValueForBuiltinType(param->type->name.toString());
 }
 
 QString ASTSignal::argumentName(int index) const
@@ -2185,7 +2234,7 @@ ImportInfo ImportInfo::moduleImport(QString uri, ComponentVersion version,
     info.m_type = ImportType::Library;
     info.m_name = uri;
     info.m_path = uri;
-    info.m_path.replace(QLatin1Char('.'), QDir::separator());
+    info.m_path.replace(QLatin1Char('.'), QLatin1Char('/'));
     info.m_version = version;
     info.m_as = as;
     info.m_ast = ast;
@@ -2200,7 +2249,7 @@ ImportInfo ImportInfo::pathImport(const QString &docPath, const QString &path,
 
     QFileInfo importFileInfo(path);
     if (!importFileInfo.isAbsolute())
-        importFileInfo = QFileInfo(docPath + QDir::separator() + path);
+        importFileInfo = QFileInfo(docPath + QLatin1Char('/') + path);
     info.m_path = importFileInfo.absoluteFilePath();
 
     if (importFileInfo.isFile()) {
@@ -2208,11 +2257,13 @@ ImportInfo ImportInfo::pathImport(const QString &docPath, const QString &path,
     } else if (importFileInfo.isDir()) {
         info.m_type = ImportType::Directory;
     } else if (path.startsWith(QLatin1String("qrc:"))) {
+        ModelManagerInterface *model = ModelManagerInterface::instance();
         info.m_path = path;
-        if (ModelManagerInterface::instance()->filesAtQrcPath(info.path()).isEmpty())
-            info.m_type = ImportType::QrcDirectory;
-        else
-            info.m_type = ImportType::QrcFile;
+        info.m_type = !model
+                ? ImportType::UnknownFile
+                : model->filesAtQrcPath(info.path()).isEmpty()
+                  ? ImportType::QrcDirectory
+                  : ImportType::QrcFile;
     } else {
         info.m_type = ImportType::UnknownFile;
     }
@@ -2238,6 +2289,14 @@ ImportInfo ImportInfo::implicitDirectoryImport(const QString &directory)
     return info;
 }
 
+ImportInfo ImportInfo::qrcDirectoryImport(const QString &directory)
+{
+    ImportInfo info;
+    info.m_type = ImportType::QrcDirectory;
+    info.m_path = directory;
+    return info;
+}
+
 bool ImportInfo::isValid() const
 {
     return m_type != ImportType::Invalid;
@@ -2312,8 +2371,10 @@ const Value *TypeScope::lookupMember(const QString &name, const Context *context
             continue;
         }
 
-        if (const Value *v = import->lookupMember(name, context, foundInObject))
+        if (const Value *v = import->lookupMember(name, context, foundInObject)) {
+            i.used = true;
             return v;
+        }
     }
     if (foundInObject)
         *foundInObject = 0;
@@ -2334,7 +2395,7 @@ void TypeScope::processMembers(MemberProcessor *processor) const
             continue;
 
         if (!info.as().isEmpty())
-            processor->processProperty(info.as(), import);
+            processor->processProperty(info.as(), import, PropertyInfo(PropertyInfo::Readable));
         else
             import->processMembers(processor);
     }
@@ -2387,7 +2448,7 @@ void JSImportScope::processMembers(MemberProcessor *processor) const
         const ImportInfo &info = i.info;
 
         if (info.type() == ImportType::File || info.type() == ImportType::QrcFile)
-            processor->processProperty(info.as(), import);
+            processor->processProperty(info.as(), import, PropertyInfo(PropertyInfo::Readable));
     }
 }
 
@@ -2491,7 +2552,7 @@ bool Imports::importFailed() const
     return m_importFailed;
 }
 
-QList<Import> Imports::all() const
+const QList<Import> &Imports::all() const
 {
     return m_imports;
 }
@@ -2513,9 +2574,9 @@ class MemberDumper: public MemberProcessor
 public:
     MemberDumper() {}
 
-    virtual bool processProperty(const QString &name, const Value *)
+    virtual bool processProperty(const QString &name, const Value *, const PropertyInfo &pInfo)
     {
-        qCDebug(qmljsLog) << "property: " << name;
+        qCDebug(qmljsLog) << "property: " << name << " flags:" << pInfo.toString();
         return true;
     }
 
