@@ -40,6 +40,7 @@
 #include <utils/algorithm.h>
 #include <utils/environment.h>
 #include <utils/hostosinfo.h>
+#include <utils/synchronousprocess.h>
 
 #include <QDir>
 #include <QDirIterator>
@@ -91,6 +92,56 @@ AndroidToolChain::AndroidToolChain(const AndroidToolChain &tc) :
 AndroidToolChain::~AndroidToolChain()
 { }
 
+static QString getArch(const QString &triple)
+{
+    if (triple.indexOf("x86_64") == 0)
+        return QString::fromUtf8("x86_64");
+    if (triple.indexOf("i686") == 0)
+        return QString::fromUtf8("x86");
+    if (triple.indexOf("mips64") == 0)
+        return QString::fromUtf8("mips64");
+    if (triple.indexOf("mips") == 0)
+        return QString::fromUtf8("mips");
+    if (triple.indexOf("aarch64") == 0)
+        return QString::fromUtf8("arm64-v8a");
+    return QString::fromUtf8("armeabi-v7a");
+}
+
+// Paths added here are those that were used by qmake. They were taken from
+// *qtsource*/qtbase/mkspecs/common/android-base-head.conf
+// Adding them here allows us to use them for all build systems.
+static void addSystemHeaderPaths(QList<ProjectExplorer::HeaderPath> &paths,
+                                 const QString &triple, const QString &version)
+{
+    const Utils::FileName ndkPath = AndroidConfigurations::currentConfig().ndkLocation();
+
+    // Get short version (for example 4.9)
+    const QString clangVersion = version.left(version.lastIndexOf('.'));
+    Utils::FileName stdcppPath = ndkPath;
+    stdcppPath.appendPath("sources/cxx-stl/gnu-libstdc++/" + clangVersion);
+    Utils::FileName includePath = stdcppPath;
+    Utils::FileName cppLibsPath = stdcppPath;
+    cppLibsPath.appendPath("libs/" + getArch(triple) + "/include/");
+    paths.prepend({cppLibsPath.toString(), ProjectExplorer::HeaderPath::GlobalHeaderPath});
+    includePath.appendPath("include/");
+    paths.prepend({includePath.toString(), ProjectExplorer::HeaderPath::GlobalHeaderPath});
+
+    paths.prepend({ndkPath.toString() + "/sysroot/usr/include/" + triple,
+                  ProjectExplorer::HeaderPath::GlobalHeaderPath});
+    paths.prepend({ndkPath.toString() + "/sysroot/usr/include",
+                  ProjectExplorer::HeaderPath::GlobalHeaderPath});
+}
+
+AndroidToolChain::SystemHeaderPathsRunner AndroidToolChain::createSystemHeaderPathsRunner() const
+{
+    const QString triple = originalTargetTriple();
+    const QString version = this->version();
+    initExtraHeaderPathsFunction([triple, version] (QList<HeaderPath> &paths) {
+        addSystemHeaderPaths(paths, triple, version);
+    });
+    return GccToolChain::createSystemHeaderPathsRunner();
+}
+
 QString AndroidToolChain::typeDisplayName() const
 {
     return AndroidToolChainFactory::tr("Android GCC");
@@ -116,9 +167,8 @@ void AndroidToolChain::addToEnvironment(Environment &env) const
         env.set(QLatin1String("JAVA_HOME"), javaHome.toString());
         Utils::FileName javaBin = javaHome;
         javaBin.appendPath(QLatin1String("bin"));
-        const QString jb = javaBin.toUserOutput();
-        if (!Utils::contains(env.path(), [&jb](const QString &p) { return p == jb; }))
-            env.prependOrSetPath(jb);
+        if (!Utils::contains(env.path(), [&javaBin](const Utils::FileName &p) { return p == javaBin; }))
+            env.prependOrSetPath(javaBin.toUserOutput());
     }
     env.set(QLatin1String("ANDROID_HOME"), AndroidConfigurations::currentConfig().sdkLocation().toString());
     env.set(QLatin1String("ANDROID_SDK_ROOT"), AndroidConfigurations::currentConfig().sdkLocation().toString());
@@ -209,18 +259,22 @@ FileNameList AndroidToolChain::suggestedMkspecList() const
 
 QString AndroidToolChain::makeCommand(const Environment &env) const
 {
-    QStringList extraDirectories = AndroidConfigurations::currentConfig().makeExtraSearchDirectories();
+    FileName makePath = AndroidConfigurations::currentConfig().makePath();
+    if (makePath.exists())
+        return makePath.toString();
+    const Utils::FileNameList extraDirectories
+            = Utils::transform(AndroidConfigurations::currentConfig().makeExtraSearchDirectories(),
+                               [](const QString &s) { return Utils::FileName::fromString(s); });
     if (HostOsInfo::isWindowsHost()) {
-        FileName tmp = env.searchInPath(QLatin1String("ma-make.exe"), extraDirectories);
-        if (!tmp.isEmpty())
-            return QString();
-        tmp = env.searchInPath(QLatin1String("mingw32-make"), extraDirectories);
-        return tmp.isEmpty() ? QLatin1String("mingw32-make") : tmp.toString();
+        makePath = env.searchInPath("ma-make.exe", extraDirectories);
+        if (!makePath.isEmpty())
+            return makePath.toString();
+        makePath = env.searchInPath("mingw32-make", extraDirectories);
+        return makePath.isEmpty() ? QLatin1String("mingw32-make") : makePath.toString();
     }
 
-    QString make = QLatin1String("make");
-    FileName tmp = env.searchInPath(make, extraDirectories);
-    return tmp.isEmpty() ? make : tmp.toString();
+    makePath = env.searchInPath("make", extraDirectories);
+    return makePath.isEmpty() ? "make" : makePath.toString();
 }
 
 QString AndroidToolChain::ndkToolChainVersion() const
@@ -243,7 +297,9 @@ void AndroidToolChain::setSecondaryToolChain(bool b)
 
 GccToolChain::DetectedAbisResult AndroidToolChain::detectSupportedAbis() const
 {
-    return QList<Abi>() << targetAbi();
+    GccToolChain::DetectedAbisResult supportedAbis = GccToolChain::detectSupportedAbis();
+    supportedAbis.supportedAbis = {targetAbi()};
+    return supportedAbis;
 }
 
 // --------------------------------------------------------------------------
@@ -412,6 +468,7 @@ AndroidToolChainFactory::autodetectToolChainsForNdk(const FileName &ndkPath,
                                           ToolChain::AutoDetection);
                 tc->resetToolChain(compilerPath);
             }
+            QTC_ASSERT(!tc->originalTargetTriple().isEmpty(), continue);
             result.append(tc);
             toolChainBundle.append(tc);
         }
