@@ -39,9 +39,12 @@ namespace Core {
 class CORE_EXPORT IFindFilter : public QObject
 {
     Q_OBJECT
-public:
 
-    virtual ~IFindFilter() {}
+public:
+    IFindFilter();
+    virtual ~IFindFilter();
+
+    static const QList<IFindFilter *> allFindFilters();
 
     virtual QString id() const = 0;
     virtual QString displayName() const = 0;
@@ -50,6 +53,7 @@ public:
     virtual bool isValid() const { return true; }
     virtual QKeySequence defaultShortcut() const;
     virtual bool isReplaceSupported() const { return false; }
+    virtual bool showSearchTermInput() const { return true; }
     virtual FindFlags supportedFindFlags() const;
 
     virtual void findAll(const QString &txt, FindFlags findFlags) = 0;
@@ -64,6 +68,7 @@ public:
     static QString descriptionForFindFlags(FindFlags flags);
 signals:
     void enabledChanged(bool enabled);
+    void validChanged(bool valid);
     void displayNameChanged();
 };
 
