@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,47 +9,42 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
-#include "locator.h"
 #include "locatorfiltersfilter.h"
+
+#include "locator.h"
 #include "locatorwidget.h"
 
-#include <coreplugin/coreconstants.h>
+#include <utils/asconst.h>
+#include <utils/qtcassert.h>
+#include <utils/utilsicons.h>
 
 using namespace Core;
 using namespace Core::Internal;
 
 Q_DECLARE_METATYPE(ILocatorFilter*)
 
-LocatorFiltersFilter::LocatorFiltersFilter(Locator *plugin,
-                                               LocatorWidget *locatorWidget):
-    m_plugin(plugin),
-    m_locatorWidget(locatorWidget),
-    m_icon(QIcon(QLatin1String(Core::Constants::ICON_NEXT)))
+LocatorFiltersFilter::LocatorFiltersFilter():
+    m_icon(Utils::Icons::NEXT.icon())
 {
     setId("FiltersFilter");
     setDisplayName(tr("Available filters"));
     setIncludedByDefault(true);
     setHidden(true);
-    setPriority(High);
+    setPriority(Highest);
     setConfigurable(false);
 }
 
@@ -61,12 +56,13 @@ void LocatorFiltersFilter::prepareSearch(const QString &entry)
         return;
 
     QMap<QString, ILocatorFilter *> uniqueFilters;
-    foreach (ILocatorFilter *filter, m_plugin->filters()) {
-        const QString filterId = filter->shortcutString() + QLatin1Char(',') + filter->displayName();
+    const QList<ILocatorFilter *> allFilters = Locator::filters();
+    for (ILocatorFilter *filter : allFilters) {
+        const QString filterId = filter->shortcutString() + ',' + filter->displayName();
         uniqueFilters.insert(filterId, filter);
     }
 
-    foreach (ILocatorFilter *filter, uniqueFilters) {
+    for (ILocatorFilter *filter : Utils::asConst(uniqueFilters)) {
         if (!filter->shortcutString().isEmpty() && !filter->isHidden() && filter->isEnabled()) {
             m_filterShortcutStrings.append(filter->shortcutString());
             m_filterDisplayNames.append(filter->displayName());
@@ -74,7 +70,7 @@ void LocatorFiltersFilter::prepareSearch(const QString &entry)
     }
 }
 
-QList<LocatorFilterEntry> LocatorFiltersFilter::matchesFor(QFutureInterface<Core::LocatorFilterEntry> &future, const QString &entry)
+QList<LocatorFilterEntry> LocatorFiltersFilter::matchesFor(QFutureInterface<LocatorFilterEntry> &future, const QString &entry)
 {
     Q_UNUSED(entry) // search is already done in the GUI thread in prepareSearch
     QList<LocatorFilterEntry> entries;
@@ -83,7 +79,7 @@ QList<LocatorFilterEntry> LocatorFiltersFilter::matchesFor(QFutureInterface<Core
             break;
         LocatorFilterEntry filterEntry(this,
                                 m_filterShortcutStrings.at(i),
-                                m_filterShortcutStrings.at(i),
+                                i,
                                 m_icon);
         filterEntry.extraInfo = m_filterDisplayNames.at(i);
         entries.append(filterEntry);
@@ -91,12 +87,18 @@ QList<LocatorFilterEntry> LocatorFiltersFilter::matchesFor(QFutureInterface<Core
     return entries;
 }
 
-void LocatorFiltersFilter::accept(LocatorFilterEntry selection) const
+void LocatorFiltersFilter::accept(LocatorFilterEntry selection,
+                                  QString *newText, int *selectionStart, int *selectionLength) const
 {
-    const QString shortcutString = selection.internalData.toString();
-    if (!shortcutString.isEmpty())
-        m_locatorWidget->show(shortcutString + QLatin1Char(' '),
-                              shortcutString.length() + 1);
+    Q_UNUSED(selectionLength)
+    bool ok;
+    int index = selection.internalData.toInt(&ok);
+    QTC_ASSERT(ok && index >= 0 && index < m_filterShortcutStrings.size(), return);
+    const QString shortcutString = m_filterShortcutStrings.at(index);
+    if (!shortcutString.isEmpty()) {
+        *newText = shortcutString + ' ';
+        *selectionStart = shortcutString.length() + 1;
+    }
 }
 
 void LocatorFiltersFilter::refresh(QFutureInterface<void> &future)
