@@ -30,9 +30,13 @@
 #include "debuggercore.h"
 
 #include <coreplugin/icore.h>
+#include <coreplugin/variablechooser.h>
+
+#include <app/app_version.h>
 #include <utils/hostosinfo.h>
-#include <utils/savedaction.h>
+#include <utils/pathchooser.h>
 #include <utils/qtcassert.h>
+#include <utils/savedaction.h>
 
 #include <QCheckBox>
 #include <QCoreApplication>
@@ -40,6 +44,7 @@
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QSpinBox>
+#include <QTextEdit>
 #include <QTextStream>
 
 using namespace Core;
@@ -125,7 +130,9 @@ QWidget *CommonOptionsPage::widget()
         checkBoxSwitchModeOnExit->setText(tr("Switch to previous mode on debugger exit"));
 
         auto checkBoxBringToForegroundOnInterrrupt = new QCheckBox(behaviorBox);
-        checkBoxBringToForegroundOnInterrrupt->setText(tr("Bring Qt Creator to foreground when application interrupts"));
+        checkBoxBringToForegroundOnInterrrupt->setText(
+                    tr("Bring %1 to foreground when application interrupts")
+                    .arg(Core::Constants::IDE_DISPLAY_NAME));
 
         auto checkBoxShowQmlObjectTree = new QCheckBox(behaviorBox);
         checkBoxShowQmlObjectTree->setToolTip(tr("Shows QML object tree in Locals and Expressions when connected and not stepping."));
@@ -136,8 +143,12 @@ QWidget *CommonOptionsPage::widget()
         checkBoxBreakpointsFullPath->setText(tr("Set breakpoints using a full absolute path"));
 
         auto checkBoxRegisterForPostMortem = new QCheckBox(behaviorBox);
-        checkBoxRegisterForPostMortem->setToolTip(tr("Registers Qt Creator for debugging crashed applications."));
-        checkBoxRegisterForPostMortem->setText(tr("Use Qt Creator for post-mortem debugging"));
+        checkBoxRegisterForPostMortem->setToolTip(
+                    tr("Registers %1 for debugging crashed applications.")
+                    .arg(Core::Constants::IDE_DISPLAY_NAME));
+        checkBoxRegisterForPostMortem->setText(
+                    tr("Use %1 for post-mortem debugging")
+                    .arg(Core::Constants::IDE_DISPLAY_NAME));
 
         auto checkBoxWarnOnReleaseBuilds = new QCheckBox(behaviorBox);
         checkBoxWarnOnReleaseBuilds->setText(tr("Warn when debugging \"Release\" builds"));
@@ -304,9 +315,29 @@ QWidget *LocalsAndExpressionsOptionsPage::widget()
         label->setText(QLatin1String("<html><head/><body>\n<p>")
            + tr("The debugging helpers are used to produce a nice "
                 "display of objects of certain types like QString or "
-                "std::map in the &quot;Locals and Expressions&quot; view. ")
+                "std::map in the &quot;Locals and Expressions&quot; view.")
             + QLatin1String("</p></body></html>"));
 
+        auto groupBoxCustomDumperCommands = new QGroupBox(debuggingHelperGroupBox);
+        groupBoxCustomDumperCommands->setTitle(tr("Debugging Helper Customization"));
+        groupBoxCustomDumperCommands->setToolTip("<html><head/><body><p>"
+                        + tr("Python commands entered here will be executed after built-in "
+                             "debugging helpers have been loaded and fully initialized. You can "
+                             "load additional debugging helpers or modify existing ones here.")
+                        + "</p></body></html>");
+
+        auto textEditCustomDumperCommands = new QTextEdit(groupBoxCustomDumperCommands);
+        textEditCustomDumperCommands->setAcceptRichText(false);
+        textEditCustomDumperCommands->setToolTip(groupBoxCustomDumperCommands->toolTip());
+
+        auto groupBoxExtraDumperFile = new QGroupBox(debuggingHelperGroupBox);
+        groupBoxExtraDumperFile->setTitle(tr("Extra Debugging Helpers"));
+        groupBoxExtraDumperFile->setToolTip(tr(
+            "Path to a Python file containing additional data dumpers."));
+
+        auto pathChooserExtraDumperFile = new Utils::PathChooser(groupBoxExtraDumperFile);
+        pathChooserExtraDumperFile->setExpectedKind(Utils::PathChooser::File);
+
         auto checkBoxUseCodeModel = new QCheckBox(debuggingHelperGroupBox);
         auto checkBoxShowThreadNames = new QCheckBox(debuggingHelperGroupBox);
         auto checkBoxShowStdNamespace = new QCheckBox(m_widget);
@@ -325,10 +356,16 @@ QWidget *LocalsAndExpressionsOptionsPage::widget()
         spinBoxDisplayStringLimit->setSingleStep(10);
         spinBoxDisplayStringLimit->setValue(100);
 
-        auto verticalLayout = new QVBoxLayout(debuggingHelperGroupBox);
-        verticalLayout->addWidget(label);
-        verticalLayout->addWidget(checkBoxUseCodeModel);
-        verticalLayout->addWidget(checkBoxShowThreadNames);
+        auto chooser = new VariableChooser(m_widget);
+        chooser->addSupportedWidget(textEditCustomDumperCommands);
+        chooser->addSupportedWidget(pathChooserExtraDumperFile->lineEdit());
+
+        auto gridLayout = new QGridLayout(debuggingHelperGroupBox);
+        gridLayout->addWidget(label, 0, 0, 1, 1);
+        gridLayout->addWidget(checkBoxUseCodeModel, 1, 0, 1, 1);
+        gridLayout->addWidget(checkBoxShowThreadNames, 2, 0, 1, 1);
+        gridLayout->addWidget(groupBoxExtraDumperFile, 3, 0, 1, 1);
+        gridLayout->addWidget(groupBoxCustomDumperCommands, 0, 1, 4, 1);
 
         auto layout1 = new QFormLayout;
         layout1->addItem(new QSpacerItem(10, 10));
@@ -348,8 +385,16 @@ QWidget *LocalsAndExpressionsOptionsPage::widget()
         layout->addLayout(lowerLayout);
         layout->addStretch();
 
+        auto customDumperLayout = new QGridLayout(groupBoxCustomDumperCommands);
+        customDumperLayout->addWidget(textEditCustomDumperCommands, 0, 0, 1, 1);
+
+        auto extraDumperLayout = new QGridLayout(groupBoxExtraDumperFile);
+        extraDumperLayout->addWidget(pathChooserExtraDumperFile, 0, 0, 1, 1);
+
         m_group.clear();
         m_group.insert(action(UseDebuggingHelpers), debuggingHelperGroupBox);
+        m_group.insert(action(ExtraDumperFile), pathChooserExtraDumperFile);
+        m_group.insert(action(ExtraDumperCommands), textEditCustomDumperCommands);
         m_group.insert(action(UseCodeModel), checkBoxUseCodeModel);
         m_group.insert(action(ShowThreadNames), checkBoxShowThreadNames);
         m_group.insert(action(ShowStdNamespace), checkBoxShowStdNamespace);
