@@ -94,7 +94,7 @@ bool isIntType(const QString &type)
 
 bool isFloatType(const QString &type)
 {
-    return type == "float" || type == "double" || type == "qreal";
+    return type == "float" || type == "double" || type == "qreal" || type == "number";
 }
 
 bool isIntOrFloatType(const QString &type)
@@ -265,6 +265,7 @@ public:
                     case 8:
                         return decodeArrayHelper<qint64>(encoding.size);
                 }
+                break;
             case DebuggerEncoding::HexEncodedUnsignedInteger:
                 switch (encoding.size) {
                     case 1:
