@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,40 +9,34 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://www.qt.io/licensing.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
 #include "formeditorplugin.h"
 
-#if QT_VERSION < 0x050000
-#include <QtTest>
-#else
 #include "formeditorw.h"
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/testdatadir.h>
+#include <cpptools/builtineditordocumentprocessor.h>
 #include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolseditorsupport.h>
 #include <cpptools/cpptoolstestcase.h>
+#include <cpptools/editordocumenthandle.h>
 
 #include <cplusplus/CppDocument.h>
 #include <cplusplus/Overview.h>
+#include <utils/asconst.h>
 #include <utils/fileutils.h>
 
 #include <QDesignerFormEditorInterface>
@@ -127,7 +121,7 @@ protected:
     {
         QString declaration = m_overview.prettyName(symbol->name());
         if (!m_currentClass.isEmpty())
-            declaration = m_currentClass + QLatin1String("::") + declaration;
+            declaration = m_currentClass + "::" + declaration;
         if (m_referenceFunction == declaration)
             m_result = true;
         return false;
@@ -160,52 +154,55 @@ public:
         QCOMPARE(files.size(), 3);
 
         QList<TextEditor::BaseTextEditor *> editors;
-        foreach (const QString &file, files) {
+        for (const QString &file : files) {
             IEditor *editor = EditorManager::openEditor(file);
             TextEditor::BaseTextEditor *e = qobject_cast<TextEditor::BaseTextEditor *>(editor);
             QVERIFY(e);
             closeEditorAtEndOfTestCase(editor);
             editors << e;
         }
-        TextEditor::BaseTextEditor *cppFileEditor = editors.at(0);
-        TextEditor::BaseTextEditor *hFileEditor = editors.at(1);
 
         const QString cppFile = files.at(0);
         const QString hFile = files.at(1);
 
         QCOMPARE(DocumentModel::openedDocuments().size(), files.size());
-        waitForFilesInGlobalSnapshot(QStringList() << cppFile << hFile);
+        waitForFilesInGlobalSnapshot({cppFile, hFile});
 
         // Execute "Go To Slot"
-        FormEditorW *few = FormEditorW::instance();
-        QDesignerIntegrationInterface *integration = few->designerEditor()->integration();
+        QDesignerIntegrationInterface *integration = FormEditorW::designerEditor()->integration();
         QVERIFY(integration);
-        integration->emitNavigateToSlot(QLatin1String("pushButton"), QLatin1String("clicked()"),
-                                        QStringList());
+        integration->emitNavigateToSlot("pushButton", "clicked()", QStringList());
 
-        QCOMPARE(EditorManager::currentDocument()->filePath(), cppFile);
+        QCOMPARE(EditorManager::currentDocument()->filePath().toString(), cppFile);
         QVERIFY(EditorManager::currentDocument()->isModified());
 
         // Wait for updated documents
-        foreach (TextEditor::BaseTextEditor *editor, editors) {
-            if (CppEditorSupport *editorSupport = m_modelManager->cppEditorSupport(editor)) {
-                while (editorSupport->isUpdatingDocument())
+        for (TextEditor::BaseTextEditor *editor : Utils::asConst(editors)) {
+            const QString filePath = editor->document()->filePath().toString();
+            if (auto parser = BuiltinEditorDocumentParser::get(filePath)) {
+                forever {
+                    if (Document::Ptr document = parser->document()) {
+                        if (document->editorRevision() == 2)
+                            break;
+                    }
                     QApplication::processEvents();
+                }
             }
         }
 
         // Compare
-        const Document::Ptr cppDocument
-            = m_modelManager->cppEditorSupport(cppFileEditor)->snapshotUpdater()->document();
+        const auto cppDocumentParser = BuiltinEditorDocumentParser::get(cppFile);
+        QVERIFY(cppDocumentParser);
+        const Document::Ptr cppDocument = cppDocumentParser->document();
         QVERIFY(checkDiagsnosticMessages(cppDocument));
-        const Document::Ptr hDocument
-            = m_modelManager->cppEditorSupport(hFileEditor)->snapshotUpdater()->document();
+
+        const auto hDocumentParser = BuiltinEditorDocumentParser::get(hFile);
+        QVERIFY(hDocumentParser);
+        const Document::Ptr hDocument = hDocumentParser->document();
         QVERIFY(checkDiagsnosticMessages(hDocument));
 
-        QVERIFY(documentContainsFunctionDefinition(cppDocument,
-            QLatin1String("Form::on_pushButton_clicked")));
-        QVERIFY(documentContainsMemberFunctionDeclaration(hDocument,
-            QLatin1String("Form::on_pushButton_clicked")));
+        QVERIFY(documentContainsFunctionDefinition(cppDocument, "Form::on_pushButton_clicked"));
+        QVERIFY(documentContainsMemberFunctionDeclaration(hDocument, "Form::on_pushButton_clicked"));
     }
 
     static bool checkDiagsnosticMessages(const Document::Ptr &document)
@@ -215,11 +212,11 @@ public:
 
         // Since no project is opened and the ui_*.h is not generated,
         // the following diagnostic messages will be ignored.
-        const QStringList ignoreList = QStringList()
-            << QLatin1String("ui_form.h: No such file or directory")
-            << QLatin1String("QWidget: No such file or directory");
+        const QStringList ignoreList = QStringList({"ui_form.h: No such file or directory",
+                                                    "QWidget: No such file or directory"});
         QList<Document::DiagnosticMessage> cleanedDiagnosticMessages;
-        foreach (const Document::DiagnosticMessage &message, document->diagnosticMessages()) {
+        const auto diagnosticMessages = document->diagnosticMessages();
+        for (const Document::DiagnosticMessage &message : diagnosticMessages) {
             if (!ignoreList.contains(message.text()))
                 cleanedDiagnosticMessages << message;
         }
@@ -229,32 +226,28 @@ public:
 };
 
 } // anonymous namespace
-#endif
+
+namespace Designer {
+namespace Internal {
 
 /// Check: Executes "Go To Slot..." on a QPushButton in a *.ui file and checks if the respective
 /// header and source files are correctly updated.
-void Designer::Internal::FormEditorPlugin::test_gotoslot()
+void FormEditorPlugin::test_gotoslot()
 {
-#if QT_VERSION >= 0x050000
     QFETCH(QStringList, files);
     (GoToSlotTestCase(files));
-#else
-    QSKIP("Available only with >= Qt5", SkipSingle);
-#endif
 }
 
-void Designer::Internal::FormEditorPlugin::test_gotoslot_data()
+void FormEditorPlugin::test_gotoslot_data()
 {
-#if QT_VERSION >= 0x050000
     typedef QLatin1String _;
     QTest::addColumn<QStringList>("files");
 
     MyTestDataDir testDataDirWithoutProject(_("gotoslot_withoutProject"));
     QTest::newRow("withoutProject")
-        << (QStringList()
-            << testDataDirWithoutProject.file(_("form.cpp"))
-            << testDataDirWithoutProject.file(_("form.h"))
-            << testDataDirWithoutProject.file(_("form.ui")));
+        << QStringList({testDataDirWithoutProject.file(_("form.cpp")),
+                        testDataDirWithoutProject.file(_("form.h")),
+                        testDataDirWithoutProject.file(_("form.ui"))});
 
     // Finding the right class for inserting definitions/declarations is based on
     // finding a class with a member whose type is the class from the "ui_xxx.h" header.
@@ -265,23 +258,19 @@ void Designer::Internal::FormEditorPlugin::test_gotoslot_data()
 
     testDataDir = MyTestDataDir(_("gotoslot_insertIntoCorrectClass_pointer"));
     QTest::newRow("insertIntoCorrectClass_pointer")
-        << (QStringList()
-            << testDataDir.file(_("form.cpp"))
-            << testDataDir.file(_("form.h"))
-            << testDataDirWithoutProject.file(_("form.ui"))); // reuse
+        << QStringList({testDataDir.file(_("form.cpp")), testDataDir.file(_("form.h")),
+                        testDataDirWithoutProject.file(_("form.ui"))}); // reuse
 
     testDataDir = MyTestDataDir(_("gotoslot_insertIntoCorrectClass_non-pointer"));
     QTest::newRow("insertIntoCorrectClass_non-pointer")
-        << (QStringList()
-            << testDataDir.file(_("form.cpp"))
-            << testDataDir.file(_("form.h"))
-            << testDataDirWithoutProject.file(_("form.ui"))); // reuse
+        << QStringList({testDataDir.file(_("form.cpp")), testDataDir.file(_("form.h")),
+                        testDataDirWithoutProject.file(_("form.ui"))}); // reuse
 
     testDataDir = MyTestDataDir(_("gotoslot_insertIntoCorrectClass_pointer_ns_using"));
     QTest::newRow("insertIntoCorrectClass_pointer_ns_using")
-        << (QStringList()
-            << testDataDir.file(_("form.cpp"))
-            << testDataDir.file(_("form.h"))
-            << testDataDir.file(_("form.ui")));
-#endif
+        << QStringList({testDataDir.file(_("form.cpp")), testDataDir.file(_("form.h")),
+                        testDataDir.file(_("form.ui"))});
 }
+
+} // namespace Internal
+} // namespace Designer
