@@ -73,6 +73,7 @@
 #include <utils/hostosinfo.h>
 #include <utils/qtcassert.h>
 #include <utils/pathchooser.h>
+#include <utils/qtcassert.h>
 #include <utils/savedaction.h>
 #include <utils/stylehelper.h>
 
@@ -137,14 +138,15 @@ public:
                      int messageLevel, FakeVimHandler *eventFilter)
     {
         if (cursorPos != -1) {
-            m_edit->blockSignals(true);
-            m_label->clear();
-            m_edit->setText(contents);
-            if (anchorPos != -1 && anchorPos != cursorPos)
-                m_edit->setSelection(anchorPos, cursorPos - anchorPos);
-            else
-                m_edit->setCursorPosition(cursorPos);
-            m_edit->blockSignals(false);
+            {
+                QSignalBlocker blocker(m_edit);
+                m_label->clear();
+                m_edit->setText(contents);
+                if (anchorPos != -1 && anchorPos != cursorPos)
+                    m_edit->setSelection(anchorPos, cursorPos - anchorPos);
+                else
+                    m_edit->setCursorPosition(cursorPos);
+            }
             setCurrentWidget(m_edit);
             m_edit->setFocus();
         } else {
@@ -547,10 +549,13 @@ private:
 
     ExCommandMap &exCommandMap();
     ExCommandMap &defaultExCommandMap();
+    ExCommandMap exCommandMapFromWidget();
 
     FakeVimPluginPrivate *m_q;
     QGroupBox *m_commandBox;
     FancyLineEdit *m_commandEdit;
+
+    friend class FakeVimExCommandsPage; // allow the page accessing the ExCommandMaps
 };
 
 FakeVimExCommandsWidget::FakeVimExCommandsWidget(FakeVimPluginPrivate *q, QWidget *parent)
@@ -600,12 +605,12 @@ public:
     }
 
     QWidget *widget() override;
-    void apply() override {}
-    void finish() override;
+    void apply() override;
+    void finish() override {}
 
 private:
     FakeVimPluginPrivate *m_q;
-    QPointer<QWidget> m_widget;
+    QPointer<FakeVimExCommandsWidget> m_widget;
 };
 
 QWidget *FakeVimExCommandsPage::widget()
@@ -615,9 +620,44 @@ QWidget *FakeVimExCommandsPage::widget()
     return m_widget;
 }
 
-void FakeVimExCommandsPage::finish()
+const char exCommandMapGroup[] = "FakeVimExCommand";
+const char userCommandMapGroup[] = "FakeVimUserCommand";
+const char reKey[] = "RegEx";
+const char cmdKey[] = "Cmd";
+const char idKey[] = "Command";
+
+void FakeVimExCommandsPage::apply()
 {
-    delete m_widget;
+    if (!m_widget) // page has not been shown at all
+        return;
+    // now save the mappings if necessary
+    const ExCommandMap &newMapping = m_widget->exCommandMapFromWidget();
+    ExCommandMap &globalCommandMapping = m_widget->exCommandMap();
+
+    if (newMapping != globalCommandMapping) {
+        const ExCommandMap &defaultMap = m_widget->defaultExCommandMap();
+        QSettings *settings = ICore::settings();
+        settings->beginWriteArray(exCommandMapGroup);
+        int count = 0;
+        typedef ExCommandMap::const_iterator Iterator;
+        const Iterator end = newMapping.constEnd();
+        for (Iterator it = newMapping.constBegin(); it != end; ++it) {
+            const QString id = it.key();
+            const QRegExp re = it.value();
+
+            if ((defaultMap.contains(id) && defaultMap[id] != re)
+                || (!defaultMap.contains(id) && !re.pattern().isEmpty())) {
+                settings->setArrayIndex(count);
+                settings->setValue(idKey, id);
+                settings->setValue(reKey, re.pattern());
+                ++count;
+            }
+        }
+        settings->endArray();
+        globalCommandMapping.clear();
+        globalCommandMapping.unite(defaultMap);
+        globalCommandMapping.unite(newMapping);
+    }
 }
 
 void FakeVimExCommandsWidget::initialize()
@@ -657,7 +697,7 @@ void FakeVimExCommandsWidget::initialize()
             setModified(item, true);
     }
 
-    handleCurrentCommandChanged(0);
+    handleCurrentCommandChanged(nullptr);
 }
 
 void FakeVimExCommandsWidget::handleCurrentCommandChanged(QTreeWidgetItem *current)
@@ -680,10 +720,8 @@ void FakeVimExCommandsWidget::commandChanged()
     const QString name =  current->data(0, CommandRole).toString();
     const QString regex = m_commandEdit->text();
 
-    if (current->data(0, Qt::UserRole).isValid()) {
+    if (current->data(0, Qt::UserRole).isValid())
         current->setText(2, regex);
-        exCommandMap()[name] = QRegExp(regex);
-    }
 
     setModified(current, regex != defaultExCommandMap()[name].pattern());
 }
@@ -733,6 +771,8 @@ public:
     FakeVimUserCommandsModel(FakeVimPluginPrivate *q) : m_q(q) {}
     ~FakeVimUserCommandsModel() {}
 
+    void initCommandMap(const UserCommandMap &commandMap) { m_commandMap = commandMap; }
+    UserCommandMap commandMap() const { return m_commandMap; }
     int rowCount(const QModelIndex &parent) const;
     int columnCount(const QModelIndex &parent) const;
     QVariant data(const QModelIndex &index, int role) const;
@@ -742,6 +782,7 @@ public:
 
 private:
     FakeVimPluginPrivate *m_q;
+    UserCommandMap m_commandMap;
 };
 
 int FakeVimUserCommandsModel::rowCount(const QModelIndex &parent) const
@@ -813,17 +854,19 @@ public:
         setCategoryIcon(Utils::Icon(SETTINGS_CATEGORY_FAKEVIM_ICON));
     }
 
-    void apply();
-    void finish() {}
+    void apply() override;
+    void finish() override {}
 
     QWidget *widget();
     void initialize() {}
     UserCommandMap &userCommandMap();
     UserCommandMap &defaultUserCommandMap();
+    UserCommandMap currentCommandMap();
 
 private:
     FakeVimPluginPrivate *m_q;
     QPointer<QWidget> m_widget;
+    FakeVimUserCommandsModel *m_model;
 };
 
 QWidget *FakeVimUserCommandsPage::widget()
@@ -831,10 +874,11 @@ QWidget *FakeVimUserCommandsPage::widget()
     if (!m_widget) {
         m_widget = new QWidget;
 
-        auto model = new FakeVimUserCommandsModel(m_q);
+        m_model = new FakeVimUserCommandsModel(m_q);
+        m_model->initCommandMap(userCommandMap());
         auto widget = new QTreeView;
-        model->setParent(widget);
-        widget->setModel(model);
+        m_model->setParent(widget);
+        widget->setModel(m_model);
         widget->resizeColumnToContents(0);
 
         auto delegate = new FakeVimUserCommandsDelegate(widget);
@@ -849,8 +893,37 @@ QWidget *FakeVimUserCommandsPage::widget()
 
 void FakeVimUserCommandsPage::apply()
 {
-    //m_q->writeSettings();
-    delete m_widget;
+    if (!m_widget) // page has not been shown at all
+        return;
+
+    // now save the mappings if necessary
+    const UserCommandMap &current = currentCommandMap();
+    UserCommandMap &userMap = userCommandMap();
+
+    if (current != userMap) {
+        QSettings *settings = ICore::settings();
+        settings->beginWriteArray(userCommandMapGroup);
+        int count = 0;
+        typedef UserCommandMap::const_iterator Iterator;
+        const Iterator end = current.constEnd();
+        for (Iterator it = current.constBegin(); it != end; ++it) {
+            const int key = it.key();
+            const QString cmd = it.value();
+
+            if ((defaultUserCommandMap().contains(key)
+                 && defaultUserCommandMap()[key] != cmd)
+                    || (!defaultUserCommandMap().contains(key) && !cmd.isEmpty())) {
+                settings->setArrayIndex(count);
+                settings->setValue(idKey, key);
+                settings->setValue(cmdKey, cmd);
+                ++count;
+            }
+        }
+        settings->endArray();
+        userMap.clear();
+        userMap.unite(defaultUserCommandMap());
+        userMap.unite(current);
+    }
 }
 
 
@@ -863,11 +936,6 @@ void FakeVimUserCommandsPage::apply()
 class FakeVimCompletionAssistProvider : public CompletionAssistProvider
 {
 public:
-    bool supportsEditor(Id ) const
-    {
-        return false;
-    }
-
     IAssistProcessor *createProcessor() const;
 
     void setActive(const QString &needle, bool forward, FakeVimHandler *handler)
@@ -908,7 +976,7 @@ public:
     }
 
 private:
-    FakeVimHandler *m_handler;
+    FakeVimHandler *m_handler = nullptr;
     QString m_needle;
 };
 
@@ -1035,6 +1103,7 @@ private:
     void onCoreAboutToClose();
     void editorOpened(Core::IEditor *);
     void editorAboutToClose(Core::IEditor *);
+    void currentEditorAboutToChange(Core::IEditor *);
 
     void allDocumentsRenamed(const QString &oldName, const QString &newName);
     void documentRenamed(Core::IDocument *document, const QString &oldName, const QString &newName);
@@ -1043,33 +1112,33 @@ private:
     void setUseFakeVim(const QVariant &value);
     void setUseFakeVimInternal(bool on);
     void quitFakeVim();
-    void triggerCompletions();
-    void triggerSimpleCompletions(const QString &needle, bool forward);
-    void windowCommand(const QString &key, int count);
+    void triggerCompletions(FakeVimHandler *handler);
+    void triggerSimpleCompletions(FakeVimHandler *handler, const QString &needle, bool forward);
+    void windowCommand(FakeVimHandler *handler, const QString &key, int count);
     void find(bool reverse);
     void findNext(bool reverse);
-    void foldToggle(int depth);
-    void foldAll(bool fold);
-    void fold(int depth, bool fold);
-    void foldGoTo(int count, bool current);
-    void jumpToGlobalMark(QChar mark, bool backTickMode, const QString &fileName);
+    void foldToggle(FakeVimHandler *handler, int depth);
+    void foldAll(FakeVimHandler *handler, bool fold);
+    void fold(FakeVimHandler *handler, int depth, bool fold);
+    void foldGoTo(FakeVimHandler *handler, int count, bool current);
+    void jumpToGlobalMark(FakeVimHandler *, QChar mark, bool backTickMode, const QString &fileName);
     void maybeReadVimRc();
-    void disableBlockSelection();
-    void setBlockSelection(const QTextCursor&);
-    void blockSelection(QTextCursor *);
-    void hasBlockSelection(bool*);
+    void disableBlockSelection(FakeVimHandler *handler);
+    void setBlockSelection(FakeVimHandler *handler, const QTextCursor &cursor);
+    void blockSelection(FakeVimHandler *handler, QTextCursor *cursor);
+    void hasBlockSelection(FakeVimHandler *handler, bool *on);
     void setShowRelativeLineNumbers(const QVariant &value);
 
     void resetCommandBuffer();
-    void showCommandBuffer(const QString &contents, int cursorPos, int anchorPos,
-                           int messageLevel, FakeVimHandler *eventFilter);
-    void showExtraInformation(const QString &msg);
-    void changeSelection(const QList<QTextEdit::ExtraSelection> &selections);
-    void highlightMatches(const QString &needle);
-    void moveToMatchingParenthesis(bool *moved, bool *forward, QTextCursor *cursor);
-    void checkForElectricCharacter(bool *result, QChar c);
-    void indentRegion(int beginBlock, int endBlock, QChar typedChar);
-    void handleExCommand(bool *handled, const ExCommand &cmd);
+    void showCommandBuffer(FakeVimHandler *handler, const QString &contents,
+                           int cursorPos, int anchorPos, int messageLevel);
+    void showExtraInformation(FakeVimHandler *handler, const QString &msg);
+    void changeSelection(FakeVimHandler *handler, const QList<QTextEdit::ExtraSelection> &selections);
+    void highlightMatches(FakeVimHandler *handler, const QString &needle);
+    void moveToMatchingParenthesis(FakeVimHandler *handler, bool *moved, bool *forward, QTextCursor *cursor);
+    void checkForElectricCharacter(FakeVimHandler *handler, bool *result, QChar c);
+    void indentRegion(FakeVimHandler *handler, int beginBlock, int endBlock, QChar typedChar);
+    void handleExCommand(FakeVimHandler *handler, bool *handled, const ExCommand &cmd);
 
     void writeSettings();
     void readSettings();
@@ -1098,7 +1167,7 @@ private:
     void setActionChecked(Id id, bool check);
 
     typedef int (*DistFunction)(const QRect &cursor, const QRect &other);
-    void moveSomewhere(DistFunction f, int count);
+    void moveSomewhere(FakeVimHandler *handler, DistFunction f, int count);
 
     void keepOnlyWindow(); // :only
 
@@ -1128,7 +1197,7 @@ QVariant FakeVimUserCommandsModel::data(const QModelIndex &index, int role) cons
         case 0: // Action
             return Tr::tr("User command #%1").arg(index.row() + 1);
         case 1: // Command
-            return m_q->userCommandMap().value(index.row() + 1);
+            return m_commandMap.value(index.row() + 1);
         }
     }
 
@@ -1140,7 +1209,7 @@ bool FakeVimUserCommandsModel::setData(const QModelIndex &index,
 {
     if (role == Qt::DisplayRole || role == Qt::EditRole)
         if (index.column() == 1)
-            m_q->userCommandMap()[index.row() + 1] = data.toString();
+            m_commandMap[index.row() + 1] = data.toString();
     return true;
 }
 
@@ -1246,6 +1315,8 @@ bool FakeVimPluginPrivate::initialize()
             this, &FakeVimPluginPrivate::editorAboutToClose);
     connect(EditorManager::instance(), &EditorManager::editorOpened,
             this, &FakeVimPluginPrivate::editorOpened);
+    connect(EditorManager::instance(), &EditorManager::currentEditorAboutToChange,
+            this, &FakeVimPluginPrivate::currentEditorAboutToChange);
 
     connect(DocumentManager::instance(), &DocumentManager::allDocumentsRenamed,
             this, &FakeVimPluginPrivate::allDocumentsRenamed);
@@ -1305,58 +1376,10 @@ void FakeVimPluginPrivate::createRelativeNumberWidget(IEditor *editor)
     }
 }
 
-const char exCommandMapGroup[] = "FakeVimExCommand";
-const char userCommandMapGroup[] = "FakeVimUserCommand";
-const char reKey[] = "RegEx";
-const char cmdKey[] = "Cmd";
-const char idKey[] = "Command";
-
 void FakeVimPluginPrivate::writeSettings()
 {
     QSettings *settings = ICore::settings();
-
     theFakeVimSettings()->writeSettings(settings);
-
-    { // block
-    settings->beginWriteArray(exCommandMapGroup);
-    int count = 0;
-    typedef ExCommandMap::const_iterator Iterator;
-    const Iterator end = exCommandMap().constEnd();
-    for (Iterator it = exCommandMap().constBegin(); it != end; ++it) {
-        const QString id = it.key();
-        const QRegExp re = it.value();
-
-        if ((defaultExCommandMap().contains(id) && defaultExCommandMap()[id] != re)
-            || (!defaultExCommandMap().contains(id) && !re.pattern().isEmpty())) {
-            settings->setArrayIndex(count);
-            settings->setValue(idKey, id);
-            settings->setValue(reKey, re.pattern());
-            ++count;
-        }
-    }
-    settings->endArray();
-    } // block
-
-    { // block
-    settings->beginWriteArray(userCommandMapGroup);
-    int count = 0;
-    typedef UserCommandMap::const_iterator Iterator;
-    const Iterator end = userCommandMap().constEnd();
-    for (Iterator it = userCommandMap().constBegin(); it != end; ++it) {
-        const int key = it.key();
-        const QString cmd = it.value();
-
-        if ((defaultUserCommandMap().contains(key)
-                && defaultUserCommandMap()[key] != cmd)
-            || (!defaultUserCommandMap().contains(key) && !cmd.isEmpty())) {
-            settings->setArrayIndex(count);
-            settings->setValue(idKey, key);
-            settings->setValue(cmdKey, cmd);
-            ++count;
-        }
-    }
-    settings->endArray();
-    } // block
 }
 
 void FakeVimPluginPrivate::readSettings()
@@ -1467,7 +1490,7 @@ static int moveDownWeight(const QRect &cursor, const QRect &other)
     return w;
 }
 
-void FakeVimPluginPrivate::windowCommand(const QString &map, int count)
+void FakeVimPluginPrivate::windowCommand(FakeVimHandler *handler, const QString &map, int count)
 {
     // normalize mapping
     const QString key = map.toUpper();
@@ -1487,31 +1510,31 @@ void FakeVimPluginPrivate::windowCommand(const QString &map, int count)
     else if (key == "W" || key == "<C-W>")
         triggerAction(Core::Constants::GOTO_NEXT_SPLIT);
     else if (key.contains("RIGHT") || key == "L" || key == "<S-L>")
-        moveSomewhere(&moveRightWeight, key == "<S-L>" ? -1 : count);
+        moveSomewhere(handler, &moveRightWeight, key == "<S-L>" ? -1 : count);
     else if (key.contains("LEFT")  || key == "H" || key == "<S-H>")
-        moveSomewhere(&moveLeftWeight, key == "<S-H>" ? -1 : count);
+        moveSomewhere(handler, &moveLeftWeight, key == "<S-H>" ? -1 : count);
     else if (key.contains("UP")    || key == "K" || key == "<S-K>")
-        moveSomewhere(&moveUpWeight, key == "<S-K>" ? -1 : count);
+        moveSomewhere(handler, &moveUpWeight, key == "<S-K>" ? -1 : count);
     else if (key.contains("DOWN")  || key == "J" || key == "<S-J>")
-        moveSomewhere(&moveDownWeight, key == "<S-J>" ? -1 : count);
+        moveSomewhere(handler, &moveDownWeight, key == "<S-J>" ? -1 : count);
     else
         qDebug() << "UNKNOWN WINDOW COMMAND: <C-W>" << map;
 }
 
-void FakeVimPluginPrivate::moveSomewhere(DistFunction f, int count)
+void FakeVimPluginPrivate::moveSomewhere(FakeVimHandler *handler, DistFunction f, int count)
 {
-    IEditor *currentEditor = EditorManager::currentEditor();
-    QWidget *w = currentEditor->widget();
+    QTC_ASSERT(handler, return);
+    QWidget *w = handler->widget();
     QPlainTextEdit *pe = qobject_cast<QPlainTextEdit *>(w);
     QTC_ASSERT(pe, return);
     QRect rc = pe->cursorRect();
-    QRect cursorRect(w->mapToGlobal(rc.topLeft()),
-            w->mapToGlobal(rc.bottomRight()));
+    QRect cursorRect(w->mapToGlobal(rc.topLeft()), w->mapToGlobal(rc.bottomRight()));
     //qDebug() << "\nCURSOR: " << cursorRect;
 
     IEditor *bestEditor = 0;
     int repeat = count;
 
+    IEditor *currentEditor = EditorManager::currentEditor();
     QList<IEditor *> editors = EditorManager::visibleEditors();
     while (repeat < 0 || repeat-- > 0) {
         editors.removeOne(currentEditor);
@@ -1569,28 +1592,21 @@ void FakeVimPluginPrivate::findNext(bool reverse)
         triggerAction(Core::Constants::FIND_NEXT);
 }
 
-void FakeVimPluginPrivate::foldToggle(int depth)
+void FakeVimPluginPrivate::foldToggle(FakeVimHandler *handler, int depth)
 {
-    IEditor *ieditor = EditorManager::currentEditor();
-    FakeVimHandler *handler = m_editorToHandler.value(ieditor, 0);
-    QTC_ASSERT(handler != 0, return);
-
+    QTC_ASSERT(handler, return);
     QTextBlock block = handler->textCursor().block();
-    fold(depth, !TextDocumentLayout::isFolded(block));
+    fold(handler, depth, !TextDocumentLayout::isFolded(block));
 }
 
-void FakeVimPluginPrivate::foldAll(bool fold)
+void FakeVimPluginPrivate::foldAll(FakeVimHandler *handler, bool fold)
 {
-    IEditor *ieditor = EditorManager::currentEditor();
-    TextEditorWidget *editor = qobject_cast<TextEditorWidget *>(ieditor->widget());
-    QTC_ASSERT(editor != 0, return);
-
-    QTextDocument *doc = editor->document();
-    TextDocumentLayout *documentLayout =
-            qobject_cast<TextDocumentLayout*>(doc->documentLayout());
+    QTC_ASSERT(handler, return);
+    QTextDocument *document = handler->textCursor().document();
+    auto documentLayout = qobject_cast<TextDocumentLayout*>(document->documentLayout());
     QTC_ASSERT(documentLayout != 0, return);
 
-    QTextBlock block = editor->document()->firstBlock();
+    QTextBlock block = document->firstBlock();
     while (block.isValid()) {
         TextDocumentLayout::doFoldOrUnfold(block, !fold);
         block = block.next();
@@ -1600,18 +1616,13 @@ void FakeVimPluginPrivate::foldAll(bool fold)
     documentLayout->emitDocumentSizeChanged();
 }
 
-void FakeVimPluginPrivate::fold(int depth, bool fold)
+void FakeVimPluginPrivate::fold(FakeVimHandler *handler, int depth, bool fold)
 {
-    IEditor *ieditor = EditorManager::currentEditor();
-    FakeVimHandler *handler = m_editorToHandler.value(ieditor, 0);
-    QTC_ASSERT(handler != 0, return);
-    TextEditorWidget *editor = qobject_cast<TextEditorWidget *>(ieditor->widget());
-    QTC_ASSERT(editor != 0, return);
-
-    QTextDocument *doc = editor->document();
-    TextDocumentLayout *documentLayout =
-            qobject_cast<TextDocumentLayout*>(doc->documentLayout());
-    QTC_ASSERT(documentLayout != 0, return);
+    QTC_ASSERT(handler, return);
+    QTextDocument *doc = handler->textCursor().document();
+    QTC_ASSERT(doc, return);
+    auto documentLayout = qobject_cast<TextDocumentLayout*>(doc->documentLayout());
+    QTC_ASSERT(documentLayout, return);
 
     QTextBlock block = handler->textCursor().block();
     int indent = TextDocumentLayout::foldingIndent(block);
@@ -1638,13 +1649,10 @@ void FakeVimPluginPrivate::fold(int depth, bool fold)
         if (TextDocumentLayout::isFolded(block)) {
             if (depth < 0) {
                 // recursively open fold
-                while (depth < 0 && block.isValid()
+                while (block.isValid()
                     && TextDocumentLayout::foldingIndent(block) >= indent) {
-                    if (TextDocumentLayout::canFold(block)) {
+                    if (TextDocumentLayout::canFold(block))
                         TextDocumentLayout::doFoldOrUnfold(block, true);
-                        if (depth > 0)
-                            --depth;
-                    }
                     block = block.next();
                 }
             } else {
@@ -1661,12 +1669,9 @@ void FakeVimPluginPrivate::fold(int depth, bool fold)
     documentLayout->emitDocumentSizeChanged();
 }
 
-void FakeVimPluginPrivate::foldGoTo(int count, bool current)
+void FakeVimPluginPrivate::foldGoTo(FakeVimHandler *handler, int count, bool current)
 {
-    IEditor *ieditor = EditorManager::currentEditor();
-    FakeVimHandler *handler = m_editorToHandler.value(ieditor, 0);
-    QTC_ASSERT(handler != 0, return);
-
+    QTC_ASSERT(handler, return);
     QTextCursor tc = handler->textCursor();
     QTextBlock block = tc.block();
 
@@ -1718,8 +1723,8 @@ void FakeVimPluginPrivate::foldGoTo(int count, bool current)
     }
 }
 
-void FakeVimPluginPrivate::jumpToGlobalMark(QChar mark, bool backTickMode,
-    const QString &fileName)
+void FakeVimPluginPrivate::jumpToGlobalMark(FakeVimHandler *,
+    QChar mark, bool backTickMode, const QString &fileName)
 {
     IEditor *iedit = EditorManager::openEditor(fileName);
     if (!iedit)
@@ -1822,6 +1827,11 @@ void FakeVimPluginPrivate::editorOpened(IEditor *editor)
     connect(ICore::instance(), &ICore::saveSettingsRequested,
             this, &FakeVimPluginPrivate::writeSettings);
 
+    connect(handler, &FakeVimHandler::tabNextRequested,
+            this, [this] { triggerAction(Core::Constants::GOTONEXTINHISTORY); });
+    connect(handler, &FakeVimHandler::tabPreviousRequested,
+            this, [this] { triggerAction(Core::Constants::GOTOPREVINHISTORY); });
+
     handler->setCurrentFileName(editor->document()->filePath().toString());
     handler->installEventFilter();
 
@@ -1841,6 +1851,12 @@ void FakeVimPluginPrivate::editorAboutToClose(IEditor *editor)
     m_editorToHandler.remove(editor);
 }
 
+void FakeVimPluginPrivate::currentEditorAboutToChange(IEditor *editor)
+{
+    if (FakeVimHandler *handler = m_editorToHandler.value(editor, 0))
+        handler->enterCommandMode();
+}
+
 void FakeVimPluginPrivate::allDocumentsRenamed(const QString &oldName, const QString &newName)
 {
     renameFileNameInEditors(oldName, newName);
@@ -1890,55 +1906,46 @@ void FakeVimPluginPrivate::setUseFakeVimInternal(bool on)
     }
 }
 
-void FakeVimPluginPrivate::triggerCompletions()
+void FakeVimPluginPrivate::triggerCompletions(FakeVimHandler *handler)
 {
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
+    QTC_ASSERT(handler, return);
     if (TextEditorWidget *editor = qobject_cast<TextEditorWidget *>(handler->widget()))
         editor->invokeAssist(Completion, m_wordProvider);
 //        CompletionSupport::instance()->complete(editor->editor(), TextCompletion, false);
 }
 
-void FakeVimPluginPrivate::triggerSimpleCompletions(const QString &needle, bool forward)
+void FakeVimPluginPrivate::triggerSimpleCompletions(FakeVimHandler *handler, const QString &needle, bool forward)
 {
-//    m_wordCompletion->setActive(needle, forward, qobject_cast<FakeVimHandler *>(sender()));
-    m_wordProvider->setActive(needle, forward, qobject_cast<FakeVimHandler *>(sender()));
+    QTC_ASSERT(handler, return);
+//    m_wordCompletion->setActive(needle, forward, handler);
+    m_wordProvider->setActive(needle, forward, handler);
 }
 
-void FakeVimPluginPrivate::disableBlockSelection()
+void FakeVimPluginPrivate::disableBlockSelection(FakeVimHandler *handler)
 {
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
+    QTC_ASSERT(handler, return);
     if (TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget()))
         bt->setBlockSelection(false);
 }
 
-void FakeVimPluginPrivate::setBlockSelection(const QTextCursor &cursor)
+void FakeVimPluginPrivate::setBlockSelection(FakeVimHandler *handler, const QTextCursor &cursor)
 {
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
+    QTC_ASSERT(handler, return);
     if (TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget()))
         bt->setBlockSelection(cursor);
 }
 
-void FakeVimPluginPrivate::blockSelection(QTextCursor *cursor)
+void FakeVimPluginPrivate::blockSelection(FakeVimHandler *handler, QTextCursor *cursor)
 {
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
+    QTC_ASSERT(handler, return);
     if (TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget()))
         if (cursor)
             *cursor = bt->blockSelection();
 }
 
-void FakeVimPluginPrivate::hasBlockSelection(bool *on)
+void FakeVimPluginPrivate::hasBlockSelection(FakeVimHandler *handler, bool *on)
 {
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
+    QTC_ASSERT(handler, return);
     if (TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget()))
         *on = bt->hasBlockSelection();
 }
@@ -1951,26 +1958,21 @@ void FakeVimPluginPrivate::setShowRelativeLineNumbers(const QVariant &value)
     }
 }
 
-void FakeVimPluginPrivate::checkForElectricCharacter(bool *result, QChar c)
+void FakeVimPluginPrivate::checkForElectricCharacter(FakeVimHandler *handler, bool *result, QChar c)
 {
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
+    QTC_ASSERT(handler, return);
     if (TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget()))
         *result = bt->textDocument()->indenter()->isElectricCharacter(c);
 }
 
-void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
+void FakeVimPluginPrivate::handleExCommand(FakeVimHandler *handler, bool *handled, const ExCommand &cmd)
 {
+    QTC_ASSERT(handler, return);
     using namespace Core;
     //qDebug() << "PLUGIN HANDLE: " << cmd.cmd << cmd.count;
 
     *handled = false;
 
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
-
     // Focus editor first so actions can be executed in correct context.
     QWidget *editor = handler->widget();
     if (editor)
@@ -1979,28 +1981,31 @@ void FakeVimPluginPrivate::handleExCommand(bool *handled, const ExCommand &cmd)
     *handled = true;
     if ((cmd.matches("w", "write") || cmd.cmd == "wq") && cmd.args.isEmpty()) {
         // :w[rite]
+        bool saved = false;
         IEditor *editor = m_editorToHandler.key(handler);
         const QString fileName = handler->currentFileName();
         if (editor && editor->document()->filePath().toString() == fileName) {
-            // Handle that as a special case for nicer interaction with core
-            DocumentManager::saveDocument(editor->document());
-            // Check result by reading back.
-            QFile file3(fileName);
-            file3.open(QIODevice::ReadOnly);
-            QByteArray ba = file3.readAll();
-            handler->showMessage(MessageInfo, Tr::tr("\"%1\" %2 %3L, %4C written")
-                .arg(fileName).arg(' ').arg(ba.count('\n')).arg(ba.size()));
-            if (cmd.cmd == "wq")
-                delayedQuitRequested(cmd.hasBang, m_editorToHandler.key(handler));
-        } else {
-            handler->showMessage(MessageError, Tr::tr("File not saved"));
+            triggerAction(Core::Constants::SAVE);
+            saved = !editor->document()->isModified();
+            if (saved) {
+                QFile file3(fileName);
+                if (file3.open(QIODevice::ReadOnly)) {
+                    const QByteArray ba = file3.readAll();
+                    handler->showMessage(MessageInfo, Tr::tr("\"%1\" %2 %3L, %4C written")
+                        .arg(fileName).arg(' ').arg(ba.count('\n')).arg(ba.size()));
+                    if (cmd.cmd == "wq")
+                        delayedQuitRequested(cmd.hasBang, m_editorToHandler.key(handler));
+                }
+            }
         }
+
+        if (!saved)
+            handler->showMessage(MessageError, Tr::tr("File not saved"));
     } else if (cmd.matches("wa", "wall")) {
         // :w[all]
-        QList<IDocument *> toSave = DocumentManager::modifiedDocuments();
-        QList<IDocument *> failed;
-        bool success = DocumentManager::saveModifiedDocuments(toSave, QString(), 0, QString(), 0, &failed);
-        if (!success)
+        triggerAction(Core::Constants::SAVEALL);
+        const QList<IDocument *> failed = DocumentManager::modifiedDocuments();
+        if (failed.isEmpty())
             handler->showMessage(MessageInfo, Tr::tr("Saving succeeded"));
         else
             handler->showMessage(MessageError, Tr::tr("%n files not saved", 0, failed.size()));
@@ -2084,7 +2089,7 @@ void FakeVimPluginPrivate::handleDelayedQuitAll(bool forced)
     EditorManager::closeAllEditors(!forced);
 }
 
-void FakeVimPluginPrivate::moveToMatchingParenthesis(bool *moved, bool *forward,
+void FakeVimPluginPrivate::moveToMatchingParenthesis(FakeVimHandler *, bool *moved, bool *forward,
         QTextCursor *cursor)
 {
     *moved = false;
@@ -2120,12 +2125,10 @@ void FakeVimPluginPrivate::moveToMatchingParenthesis(bool *moved, bool *forward,
     }
 }
 
-void FakeVimPluginPrivate::indentRegion(int beginBlock, int endBlock,
-      QChar typedChar)
+void FakeVimPluginPrivate::indentRegion(FakeVimHandler *handler,
+        int beginBlock, int endBlock, QChar typedChar)
 {
-    FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender());
-    if (!handler)
-        return;
+    QTC_ASSERT(handler, return);
 
     TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget());
     if (!bt)
@@ -2165,18 +2168,18 @@ void FakeVimPluginPrivate::quitFakeVim()
 
 void FakeVimPluginPrivate::resetCommandBuffer()
 {
-    showCommandBuffer(QString(), -1, -1, 0, 0);
+    showCommandBuffer(nullptr, QString(), -1, -1, 0);
 }
 
-void FakeVimPluginPrivate::showCommandBuffer(const QString &contents, int cursorPos, int anchorPos,
-                                             int messageLevel, FakeVimHandler *eventFilter)
+void FakeVimPluginPrivate::showCommandBuffer(FakeVimHandler *handler, const QString &contents, int cursorPos, int anchorPos,
+                                             int messageLevel)
 {
     //qDebug() << "SHOW COMMAND BUFFER" << contents;
     if (MiniBuffer *w = qobject_cast<MiniBuffer *>(m_statusBar->widget()))
-        w->setContents(contents, cursorPos, anchorPos, messageLevel, eventFilter);
+        w->setContents(contents, cursorPos, anchorPos, messageLevel, handler);
 }
 
-void FakeVimPluginPrivate::showExtraInformation(const QString &text)
+void FakeVimPluginPrivate::showExtraInformation(FakeVimHandler *, const QString &text)
 {
     EditorManager::splitSideBySide();
     QString title = "stdout.txt";
@@ -2187,14 +2190,14 @@ void FakeVimPluginPrivate::showExtraInformation(const QString &text)
     handler->handleCommand("0");
 }
 
-void FakeVimPluginPrivate::changeSelection(const QList<QTextEdit::ExtraSelection> &selection)
+void FakeVimPluginPrivate::changeSelection(FakeVimHandler *handler, const QList<QTextEdit::ExtraSelection> &selection)
 {
-    if (FakeVimHandler *handler = qobject_cast<FakeVimHandler *>(sender()))
-        if (TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget()))
-            bt->setExtraSelections(TextEditorWidget::FakeVimSelection, selection);
+    QTC_ASSERT(handler, return);
+    if (TextEditorWidget *bt = qobject_cast<TextEditorWidget *>(handler->widget()))
+        bt->setExtraSelections(TextEditorWidget::FakeVimSelection, selection);
 }
 
-void FakeVimPluginPrivate::highlightMatches(const QString &needle)
+void FakeVimPluginPrivate::highlightMatches(FakeVimHandler *, const QString &needle)
 {
     foreach (IEditor *editor, EditorManager::visibleEditors()) {
         QWidget *w = editor->widget();
@@ -2207,9 +2210,12 @@ void FakeVimPluginPrivate::highlightMatches(const QString &needle)
 int FakeVimPluginPrivate::currentFile() const
 {
     IEditor *editor = EditorManager::currentEditor();
-    if (!editor)
-        return -1;
-    return DocumentModel::indexOfDocument(editor->document());
+    if (editor) {
+        const Utils::optional<int> index = DocumentModel::indexOfDocument(editor->document());
+        if (QTC_GUARD(index))
+            return index.value();
+    }
+    return -1;
 }
 
 void FakeVimPluginPrivate::switchToFile(int n)
@@ -2232,6 +2238,26 @@ ExCommandMap &FakeVimExCommandsWidget::defaultExCommandMap()
     return m_q->defaultExCommandMap();
 }
 
+ExCommandMap FakeVimExCommandsWidget::exCommandMapFromWidget()
+{
+    ExCommandMap map;
+    int n = commandList()->topLevelItemCount();
+    for (int i = 0; i != n; ++i) {
+        QTreeWidgetItem *section = commandList()->topLevelItem(i);
+        int m = section->childCount();
+        for (int j = 0; j != m; ++j) {
+            QTreeWidgetItem *item = section->child(j);
+            const QString name = item->data(0, CommandRole).toString();
+            const QString regex = item->data(2, Qt::DisplayRole).toString();
+            if ((regex.isEmpty() && defaultExCommandMap().value(name).isEmpty())
+                    || (!regex.isEmpty() && defaultExCommandMap().value(name).pattern() == regex))
+                continue;
+            map[name] = QRegExp(regex);
+        }
+    }
+    return map;
+}
+
 UserCommandMap &FakeVimUserCommandsPage::userCommandMap()
 {
     return m_q->userCommandMap();
@@ -2242,6 +2268,11 @@ UserCommandMap &FakeVimUserCommandsPage::defaultUserCommandMap()
     return m_q->defaultUserCommandMap();
 }
 
+UserCommandMap FakeVimUserCommandsPage::currentCommandMap()
+{
+    return m_model->commandMap();
+}
+
 ///////////////////////////////////////////////////////////////////////
 //
 // FakeVimPlugin
