@@ -328,25 +328,12 @@ void GerritPlugin::push(const QString &topLevel)
 
     dialog.storeTopic();
     m_reviewers = dialog.reviewers();
+    GitPlugin::client()->push(topLevel, {dialog.selectedRemoteName(), dialog.pushTarget()});
+}
 
-    QString target = dialog.selectedCommit();
-    if (target.isEmpty())
-        target = "HEAD";
-    target += ":refs/" + dialog.selectedPushType() +
-            '/' + dialog.selectedRemoteBranchName();
-    const QString topic = dialog.selectedTopic();
-    if (!topic.isEmpty())
-        target += '/' + topic;
-
-    QStringList options;
-    const QStringList reviewers = m_reviewers.split(',', QString::SkipEmptyParts);
-    for (const QString &reviewer : reviewers)
-        options << "r=" + reviewer;
-
-    if (!options.isEmpty())
-        target += '%' + options.join(',');
-
-    GitPlugin::client()->push(topLevel, {dialog.selectedRemoteName(), target});
+static QString currentRepository()
+{
+    return GitPlugin::instance()->currentState().topLevel();
 }
 
 // Open or raise the Gerrit dialog window.
@@ -359,8 +346,7 @@ void GerritPlugin::openView()
             if (!ICore::showOptionsDialog("Gerrit"))
                 return;
         }
-        const QString repository = GitPlugin::instance()->currentState().topLevel();
-        GerritDialog *gd = new GerritDialog(m_parameters, m_server, repository, ICore::mainWindow());
+        GerritDialog *gd = new GerritDialog(m_parameters, m_server, currentRepository(), ICore::mainWindow());
         gd->setModal(false);
         connect(gd, &GerritDialog::fetchDisplay, this,
                 [this](const QSharedPointer<GerritChange> &change) { fetch(change, FetchDisplay); });
@@ -372,6 +358,7 @@ void GerritPlugin::openView()
         connect(this, &GerritPlugin::fetchFinished, gd, &GerritDialog::fetchFinished);
         m_dialog = gd;
     } else {
+        m_dialog->setCurrentPath(currentRepository());
         m_dialog->refresh();
     }
     const Qt::WindowStates state = m_dialog->windowState();
@@ -383,7 +370,7 @@ void GerritPlugin::openView()
 
 void GerritPlugin::push()
 {
-    push(GitPlugin::instance()->currentState().topLevel());
+    push(currentRepository());
 }
 
 Utils::FileName GerritPlugin::gitBinDirectory()
