@@ -25,11 +25,12 @@
 
 #include "kit.h"
 
+#include "devicesupport/idevicefactory.h"
+#include "kitinformation.h"
 #include "kitmanager.h"
 #include "ioutputparser.h"
 #include "osparser.h"
 #include "projectexplorerconstants.h"
-#include "projectexplorericons.h"
 
 #include <utils/algorithm.h>
 #include <utils/fileutils.h>
@@ -78,7 +79,6 @@ public:
             m_id = Id::fromString(QUuid::createUuid().toString());
 
         m_unexpandedDisplayName = QCoreApplication::translate("ProjectExplorer::Kit", "Unnamed");
-        m_iconPath = FileName::fromLatin1(Constants::DESKTOP_DEVICE_ICON);
 
         m_macroExpander.setDisplayName(tr("Kit"));
         m_macroExpander.setAccumulating(true);
@@ -116,7 +116,7 @@ public:
     bool m_hasWarning = false;
     bool m_hasValidityInfo = false;
     bool m_mustNotify = false;
-    QIcon m_icon;
+    QIcon m_cachedIcon;
     FileName m_iconPath;
 
     QHash<Id, QVariant> m_data;
@@ -136,8 +136,6 @@ Kit::Kit(Id id) :
 {
     foreach (KitInformation *sti, KitManager::kitInformation())
         d->m_data.insert(sti->id(), sti->defaultValue(this));
-
-    d->m_icon = icon(d->m_iconPath);
 }
 
 Kit::Kit(const QVariantMap &data) :
@@ -160,7 +158,6 @@ Kit::Kit(const QVariantMap &data) :
     d->m_fileSystemFriendlyName = data.value(QLatin1String(FILESYSTEMFRIENDLYNAME_KEY)).toString();
     d->m_iconPath = FileName::fromString(data.value(QLatin1String(ICON_KEY),
                                                     d->m_iconPath.toString()).toString());
-    d->m_icon = icon(d->m_iconPath);
 
     QVariantMap extra = data.value(QLatin1String(DATA_KEY)).toMap();
     d->m_data.clear(); // remove default values
@@ -213,7 +210,7 @@ Kit *Kit::clone(bool keepName) const
     k->d->m_data = d->m_data;
     // Do not clone m_fileSystemFriendlyName, needs to be unique
     k->d->m_isValid = d->m_isValid;
-    k->d->m_icon = d->m_icon;
+    k->d->m_cachedIcon = d->m_cachedIcon;
     k->d->m_iconPath = d->m_iconPath;
     k->d->m_sticky = d->m_sticky;
     k->d->m_mutable = d->m_mutable;
@@ -225,7 +222,7 @@ void Kit::copyFrom(const Kit *k)
     KitGuard g(this);
     d->m_data = k->d->m_data;
     d->m_iconPath = k->d->m_iconPath;
-    d->m_icon = k->d->m_icon;
+    d->m_cachedIcon = k->d->m_cachedIcon;
     d->m_autodetected = k->d->m_autodetected;
     d->m_autoDetectionSource = k->d->m_autoDetectionSource;
     d->m_unexpandedDisplayName = k->d->m_unexpandedDisplayName;
@@ -292,6 +289,15 @@ void Kit::setup()
         info.at(i)->setup(this);
 }
 
+void Kit::upgrade()
+{
+    KitGuard g(this);
+    // Process the KitInfos in reverse order: They may only be based on other information lower in
+    // the stack.
+    for (KitInformation *ki : KitManager::kitInformation())
+        ki->upgrade(this);
+}
+
 QString Kit::unexpandedDisplayName() const
 {
     return d->m_unexpandedDisplayName;
@@ -359,26 +365,34 @@ Id Kit::id() const
     return d->m_id;
 }
 
-QIcon Kit::icon() const
+static QIcon iconForDeviceType(Core::Id deviceType)
 {
-    return d->m_icon;
+    const IDeviceFactory *factory = Utils::findOrDefault(IDeviceFactory::allDeviceFactories(),
+        [&deviceType](const IDeviceFactory *factory) {
+            return factory->availableCreationIds().contains(deviceType);
+        });
+    return factory ? factory->iconForId(deviceType) : QIcon();
 }
 
-QIcon Kit::icon(const FileName &path)
+QIcon Kit::icon() const
 {
-    if (path.isEmpty())
-        return QIcon();
+    if (!d->m_cachedIcon.isNull())
+        return d->m_cachedIcon;
 
-    if (path == FileName::fromLatin1(Constants::DESKTOP_DEVICE_ICON))
-        return creatorTheme()->flag(Theme::FlatSideBarIcons)
-                ? Icon::combinedIcon({Icons::DESKTOP_DEVICE.icon(),
-                                      Icons::DESKTOP_DEVICE_SMALL.icon()})
-                : QApplication::style()->standardIcon(QStyle::SP_ComputerIcon);
+    if (!d->m_iconPath.isEmpty() && d->m_iconPath.exists()) {
+        d->m_cachedIcon = QIcon(d->m_iconPath.toString());
+        return d->m_cachedIcon;
+    }
 
-    QFileInfo fi = path.toFileInfo();
-    if (fi.isFile() && fi.isReadable())
-        return QIcon(path.toString());
-    return QIcon();
+    const Core::Id deviceType = DeviceTypeKitInformation::deviceTypeId(this);
+    const QIcon deviceTypeIcon = iconForDeviceType(deviceType);
+    if (!deviceTypeIcon.isNull()) {
+        d->m_cachedIcon = deviceTypeIcon;
+        return d->m_cachedIcon;
+    }
+
+    d->m_cachedIcon = iconForDeviceType(Constants::DESKTOP_DEVICE_TYPE);
+    return d->m_cachedIcon;
 }
 
 FileName Kit::iconPath() const
@@ -391,10 +405,14 @@ void Kit::setIconPath(const FileName &path)
     if (d->m_iconPath == path)
         return;
     d->m_iconPath = path;
-    d->m_icon = icon(path);
     kitUpdated();
 }
 
+QList<Id> Kit::allKeys() const
+{
+    return d->m_data.keys();
+}
+
 QVariant Kit::value(Id key, const QVariant &unset) const
 {
     return d->m_data.value(key, unset);
@@ -545,8 +563,17 @@ QString Kit::toHtml(const QList<Task> &additional) const
     QList<KitInformation *> infoList = KitManager::kitInformation();
     foreach (KitInformation *ki, infoList) {
         KitInformation::ItemList list = ki->toUserOutput(this);
-        foreach (const KitInformation::Item &j, list)
-            str << "<tr><td><b>" << j.first << ":</b></td><td>" << j.second << "</td></tr>";
+        foreach (const KitInformation::Item &j, list) {
+            QString contents = j.second;
+            if (contents.count() > 256) {
+                int pos = contents.lastIndexOf("<br>", 256);
+                if (pos < 0) // no linebreak, so cut early.
+                    pos = 80;
+                contents = contents.mid(0, pos);
+                contents += "&lt;...&gt;";
+            }
+            str << "<tr><td><b>" << j.first << ":</b></td><td>" << contents << "</td></tr>";
+        }
     }
     str << "</table></body></html>";
     return rc;
@@ -554,18 +581,24 @@ QString Kit::toHtml(const QList<Task> &additional) const
 
 void Kit::setAutoDetected(bool detected)
 {
+    if (d->m_autodetected == detected)
+        return;
     d->m_autodetected = detected;
     kitUpdated();
 }
 
 void Kit::setAutoDetectionSource(const QString &autoDetectionSource)
 {
+    if (d->m_autoDetectionSource == autoDetectionSource)
+        return;
     d->m_autoDetectionSource = autoDetectionSource;
     kitUpdated();
 }
 
 void Kit::setSdkProvided(bool sdkProvided)
 {
+    if (d->m_sdkProvided == sdkProvided)
+        return;
     d->m_sdkProvided = sdkProvided;
     kitUpdated();
 }
@@ -580,6 +613,9 @@ void Kit::makeSticky()
 
 void Kit::setSticky(Id id, bool b)
 {
+    if (d->m_sticky.contains(id) == b)
+        return;
+
     if (b)
         d->m_sticky.insert(id);
     else
@@ -589,12 +625,17 @@ void Kit::setSticky(Id id, bool b)
 
 void Kit::makeUnSticky()
 {
+    if (d->m_sticky.isEmpty())
+        return;
     d->m_sticky.clear();
     kitUpdated();
 }
 
 void Kit::setMutable(Id id, bool b)
 {
+    if (d->m_mutable.contains(id) == b)
+        return;
+
     if (b)
         d->m_mutable.insert(id);
     else
@@ -647,6 +688,7 @@ void Kit::kitUpdated()
         return;
     }
     d->m_hasValidityInfo = false;
+    d->m_cachedIcon = QIcon();
     KitManager::notifyAboutUpdate(this);
     d->m_mustNotify = false;
 }
