@@ -26,7 +26,6 @@
 #include "taskwindow.h"
 
 #include "itaskhandler.h"
-#include "projectexplorerconstants.h"
 #include "projectexplorericons.h"
 #include "session.h"
 #include "task.h"
@@ -35,13 +34,13 @@
 
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/command.h>
-#include <coreplugin/coreicons.h>
 #include <coreplugin/icore.h>
 #include <coreplugin/icontext.h>
-#include <extensionsystem/pluginmanager.h>
+
 #include <utils/algorithm.h>
 #include <utils/qtcassert.h>
 #include <utils/itemviews.h>
+#include <utils/utilsicons.h>
 
 #include <QDir>
 #include <QPainter>
@@ -57,6 +56,19 @@ const char SESSION_FILTER_WARNINGS[] = "TaskWindow.IncludeWarnings";
 }
 
 namespace ProjectExplorer {
+
+static QList<ITaskHandler *> g_taskHandlers;
+
+ITaskHandler::ITaskHandler()
+{
+    g_taskHandlers.append(this);
+}
+
+ITaskHandler::~ITaskHandler()
+{
+    g_taskHandlers.removeOne(this);
+}
+
 namespace Internal {
 
 class TaskView : public Utils::ListView
@@ -88,7 +100,6 @@ public:
     // TaskView uses this method if the size of the taskview changes
     void emitSizeHintChanged(const QModelIndex &index);
 
-public slots:
     void currentChanged(const QModelIndex &current, const QModelIndex &previous);
 
 private:
@@ -199,22 +210,29 @@ void TaskView::resizeEvent(QResizeEvent *e)
 class TaskWindowPrivate
 {
 public:
+    ITaskHandler *handler(const QAction *action)
+    {
+        ITaskHandler *handler = m_actionToHandlerMap.value(action, nullptr);
+        return g_taskHandlers.contains(handler) ? handler : nullptr;
+    }
+
     Internal::TaskModel *m_model;
     Internal::TaskFilterModel *m_filter;
     Internal::TaskView *m_listview;
     Internal::TaskWindowContext *m_taskWindowContext;
     QMenu *m_contextMenu;
-    ITaskHandler *m_defaultHandler;
+    QMap<const QAction *, ITaskHandler *> m_actionToHandlerMap;
+    ITaskHandler *m_defaultHandler = nullptr;
     QToolButton *m_filterWarningsButton;
     QToolButton *m_categoriesButton;
     QMenu *m_categoriesMenu;
     QList<QAction *> m_actions;
 };
 
-static QToolButton *createFilterButton(QIcon icon, const QString &toolTip,
+static QToolButton *createFilterButton(const QIcon &icon, const QString &toolTip,
                                        QObject *receiver, std::function<void(bool)> lambda)
 {
-    QToolButton *button = new QToolButton;
+    auto button = new QToolButton;
     button->setIcon(icon);
     button->setToolTip(toolTip);
     button->setCheckable(true);
@@ -227,15 +245,13 @@ static QToolButton *createFilterButton(QIcon icon, const QString &toolTip,
 
 TaskWindow::TaskWindow() : d(new TaskWindowPrivate)
 {
-    d->m_defaultHandler = 0;
-
     d->m_model = new Internal::TaskModel(this);
     d->m_filter = new Internal::TaskFilterModel(d->m_model);
     d->m_listview = new Internal::TaskView;
 
     d->m_listview->setModel(d->m_filter);
     d->m_listview->setFrameStyle(QFrame::NoFrame);
-    d->m_listview->setWindowTitle(tr("Issues"));
+    d->m_listview->setWindowTitle(displayName());
     d->m_listview->setSelectionMode(QAbstractItemView::SingleSelection);
     Internal::TaskDelegate *tld = new Internal::TaskDelegate(this);
     d->m_listview->setItemDelegate(tld);
@@ -260,11 +276,11 @@ TaskWindow::TaskWindow() : d(new TaskWindowPrivate)
     d->m_listview->setContextMenuPolicy(Qt::ActionsContextMenu);
 
     d->m_filterWarningsButton = createFilterButton(
-                Core::Icons::WARNING_TOOLBAR.icon(),
+                Utils::Icons::WARNING_TOOLBAR.icon(),
                 tr("Show Warnings"), this, [this](bool show) { setShowWarnings(show); });
 
     d->m_categoriesButton = new QToolButton;
-    d->m_categoriesButton->setIcon(Core::Icons::FILTER.icon());
+    d->m_categoriesButton->setIcon(Utils::Icons::FILTER.icon());
     d->m_categoriesButton->setToolTip(tr("Filter by categories"));
     d->m_categoriesButton->setProperty("noArrow", true);
     d->m_categoriesButton->setAutoRaise(true);
@@ -309,14 +325,6 @@ TaskWindow::~TaskWindow()
     delete d;
 }
 
-static ITaskHandler *handler(QAction *action)
-{
-    QVariant prop = action->property("ITaskHandler");
-    ITaskHandler *handler = qobject_cast<ITaskHandler *>(prop.value<QObject *>());
-    QTC_CHECK(handler);
-    return handler;
-}
-
 void TaskWindow::delayedInitialization()
 {
     static bool alreadyDone = false;
@@ -325,14 +333,13 @@ void TaskWindow::delayedInitialization()
 
     alreadyDone = true;
 
-    QList<ITaskHandler *> handlers = ExtensionSystem::PluginManager::getObjects<ITaskHandler>();
-    foreach (ITaskHandler *h, handlers) {
+    for (ITaskHandler *h : g_taskHandlers) {
         if (h->isDefaultHandler() && !d->m_defaultHandler)
             d->m_defaultHandler = h;
 
         QAction *action = h->createAction(this);
         QTC_ASSERT(action, continue);
-        action->setProperty("ITaskHandler", qVariantFromValue(qobject_cast<QObject*>(h)));
+        d->m_actionToHandlerMap.insert(action, h);
         connect(action, &QAction::triggered, this, &TaskWindow::actionTriggered);
         d->m_actions << action;
 
@@ -351,7 +358,7 @@ void TaskWindow::delayedInitialization()
 
 QList<QWidget*> TaskWindow::toolBarWidgets() const
 {
-    return QList<QWidget*>() << d->m_filterWarningsButton << d->m_categoriesButton;
+    return {d->m_filterWarningsButton, d->m_categoriesButton};
 }
 
 QWidget *TaskWindow::outputWidget(QWidget *)
@@ -387,7 +394,7 @@ void TaskWindow::currentChanged(const QModelIndex &index)
 {
     const Task task = index.isValid() ? d->m_filter->task(index) : Task();
     foreach (QAction *action, d->m_actions) {
-        ITaskHandler *h = handler(action);
+        ITaskHandler *h = d->handler(action);
         action->setEnabled((task.isNull() || !h) ? false : h->canHandle(task));
     }
 }
@@ -500,10 +507,10 @@ void TaskWindow::triggerDefaultHandler(const QModelIndex &index)
 
 void TaskWindow::actionTriggered()
 {
-    QAction *action = qobject_cast<QAction *>(sender());
+    auto action = qobject_cast<QAction *>(sender());
     if (!action || !action->isEnabled())
         return;
-    ITaskHandler *h = handler(action);
+    ITaskHandler *h = d->handler(action);
     if (!h)
         return;
 
@@ -537,7 +544,7 @@ void TaskWindow::updateCategoriesMenu()
     for (NameToIdsConstIt it = nameToIds.constBegin(); it != cend; ++it) {
         const QString &displayName = it.key();
         const Core::Id categoryId = it.value();
-        QAction *action = new QAction(d->m_categoriesMenu);
+        auto action = new QAction(d->m_categoriesMenu);
         action->setCheckable(true);
         action->setText(displayName);
         action->setChecked(!filteredCategories.contains(categoryId));
@@ -673,7 +680,7 @@ QSize TaskDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelInd
     QStyleOptionViewItem opt = option;
     initStyleOption(&opt, index);
 
-    const QAbstractItemView * view = qobject_cast<const QAbstractItemView *>(opt.widget);
+    auto view = qobject_cast<const QAbstractItemView *>(opt.widget);
     const bool selected = (view->selectionModel()->currentIndex() == index);
     QSize s;
     s.setWidth(option.rect.width());
@@ -687,7 +694,7 @@ QSize TaskDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelInd
     int fontHeight = fm.height();
     int fontLeading = fm.leading();
 
-    TaskModel *model = static_cast<TaskFilterModel *>(view->model())->taskModel();
+    auto model = static_cast<TaskFilterModel *>(view->model())->taskModel();
     Positions positions(option, model);
 
     if (selected) {
@@ -697,7 +704,7 @@ QSize TaskDelegate::sizeHint(const QStyleOptionViewItem &option, const QModelInd
         int height = 0;
         description.replace(QLatin1Char('\n'), QChar::LineSeparator);
         QTextLayout tl(description);
-        tl.setAdditionalFormats(index.data(TaskModel::Task_t).value<Task>().formats);
+        tl.setFormats(index.data(TaskModel::Task_t).value<Task>().formats);
         tl.beginLayout();
         while (true) {
             QTextLine line = tl.createLine();
@@ -746,7 +753,7 @@ void TaskDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
     QColor backgroundColor;
     QColor textColor;
 
-    const QAbstractItemView * view = qobject_cast<const QAbstractItemView *>(opt.widget);
+    auto view = qobject_cast<const QAbstractItemView *>(opt.widget);
     bool selected = view->selectionModel()->currentIndex() == index;
 
     if (selected) {
@@ -767,7 +774,7 @@ void TaskDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
 
     painter->setPen(textColor);
 
-    TaskModel *model = static_cast<TaskFilterModel *>(view->model())->taskModel();
+    auto model = static_cast<TaskFilterModel *>(view->model())->taskModel();
     Positions positions(opt, model);
 
     // Paint TaskIconArea:
@@ -797,7 +804,7 @@ void TaskDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
         int height = 0;
         description.replace(QLatin1Char('\n'), QChar::LineSeparator);
         QTextLayout tl(description);
-        tl.setAdditionalFormats(index.data(TaskModel::Task_t).value<Task>().formats);
+        tl.setFormats(index.data(TaskModel::Task_t).value<Task>().formats);
         tl.beginLayout();
         while (true) {
             QTextLine line = tl.createLine();
@@ -878,7 +885,8 @@ void TaskDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option,
 
     // Separator lines
     painter->setPen(QColor::fromRgb(150,150,150));
-    painter->drawLine(0, opt.rect.bottom(), opt.rect.right(), opt.rect.bottom());
+    const QRectF borderRect = QRectF(opt.rect).adjusted(0.5, 0.5, -0.5, -0.5);
+    painter->drawLine(borderRect.bottomLeft(), borderRect.bottomRight());
     painter->restore();
 }
 
