@@ -30,8 +30,9 @@
 
 #include "addsignalhandlerdialog.h"
 
-#include <cmath>
-#include <nodeabstractproperty.h>
+#include <bindingproperty.h>
+#include <nodelistproperty.h>
+#include <nodehints.h>
 #include <nodemetainfo.h>
 #include <modelnode.h>
 #include <qmlitemnode.h>
@@ -48,7 +49,6 @@
 
 #include <coreplugin/messagebox.h>
 #include <coreplugin/editormanager/editormanager.h>
-#include <utils/algorithm.h>
 
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/modemanager.h>
@@ -56,10 +56,19 @@
 
 #include <qmljseditor/qmljsfindreferences.h>
 
+#include <projectexplorer/project.h>
+#include <projectexplorer/projectnodes.h>
+#include <projectexplorer/projecttree.h>
+
+#include <utils/algorithm.h>
+#include <utils/qtcassert.h>
+
 #include <QCoreApplication>
 #include <QByteArray>
 
+#include <algorithm>
 #include <functional>
+#include <cmath>
 
 namespace QmlDesigner {
 
@@ -151,7 +160,7 @@ void goIntoComponent(const ModelNode &modelNode)
 void select(const SelectionContext &selectionState)
 {
     if (selectionState.view())
-        selectionState.view()->setSelectedModelNodes(QList<ModelNode>() << selectionState.targetNode());
+        selectionState.view()->setSelectedModelNodes({selectionState.targetNode()});
 }
 
 void deSelect(const SelectionContext &selectionState)
@@ -214,45 +223,52 @@ void toBack(const SelectionContext &selectionState)
     }
 }
 
-void raise(const SelectionContext &selectionState)
+enum OderAction {RaiseItem, LowerItem};
+
+void changeOrder(const SelectionContext &selectionState, OderAction orderAction)
 {
     if (!selectionState.view())
         return;
 
+    QTC_ASSERT(selectionState.singleNodeIsSelected(), return);
+    ModelNode modelNode = selectionState.currentSingleSelectedNode();
+
+    if (modelNode.isRootNode())
+        return;
+    if (!modelNode.parentProperty().isNodeListProperty())
+        return;
+
     try {
         RewriterTransaction transaction(selectionState.view(), QByteArrayLiteral("DesignerActionManager|raise"));
-        foreach (ModelNode modelNode, selectionState.selectedModelNodes()) {
-            QmlItemNode node = modelNode;
-            if (node.isValid()) {
-                signed int z  = node.instanceValue("z").toInt();
-                z++;
-                node.setVariantProperty("z", z);
-            }
+
+        ModelNode modelNode = selectionState.currentSingleSelectedNode();
+        NodeListProperty parentProperty = modelNode.parentProperty().toNodeListProperty();
+        const int index = parentProperty.indexOf(modelNode);
+
+        if (orderAction == RaiseItem) {
+
+            if (index < parentProperty.count() - 1)
+                parentProperty.slide(index, index + 1);
+        } else if (orderAction == LowerItem) {
+            if (index > 0)
+                parentProperty.slide(index, index - 1);
         }
+
+        transaction.commit();
     } catch (const RewritingException &e) { //better save then sorry
          e.showException();
     }
 }
 
+void raise(const SelectionContext &selectionState)
+{
+    changeOrder(selectionState, RaiseItem);
+}
+
 void lower(const SelectionContext &selectionState)
 {
 
-    if (!selectionState.view())
-        return;
-
-    try {
-        RewriterTransaction transaction(selectionState.view(), QByteArrayLiteral("DesignerActionManager|lower"));
-        foreach (ModelNode modelNode, selectionState.selectedModelNodes()) {
-            QmlItemNode node = modelNode;
-            if (node.isValid()) {
-                signed int z  = node.instanceValue("z").toInt();
-                z--;
-                node.setVariantProperty("z", z);
-            }
-        }
-    } catch (const RewritingException &e) { //better save then sorry
-        e.showException();
-    }
+    changeOrder(selectionState, LowerItem);
 }
 
 void paste(const SelectionContext &)
@@ -273,7 +289,7 @@ void setVisible(const SelectionContext &selectionState)
         return;
 
     try {
-        selectionState.selectedModelNodes().first().variantProperty("visible").setValue(selectionState.toggled());
+        selectionState.selectedModelNodes().constFirst().variantProperty("visible").setValue(selectionState.toggled());
     } catch (const RewritingException &e) { //better save then sorry
         e.showException();
     }
@@ -313,8 +329,9 @@ void resetSize(const SelectionContext &selectionState)
     try {
         RewriterTransaction transaction(selectionState.view(), QByteArrayLiteral("DesignerActionManager|resetSize"));
         foreach (ModelNode node, selectionState.selectedModelNodes()) {
-            node.removeProperty("width");
-            node.removeProperty("height");
+            QmlItemNode itemNode(node);
+            itemNode.removeProperty("width");
+            itemNode.removeProperty("height");
         }
     } catch (const RewritingException &e) { //better save then sorry
         e.showException();
@@ -329,15 +346,17 @@ void resetPosition(const SelectionContext &selectionState)
     try {
         RewriterTransaction transaction(selectionState.view(), QByteArrayLiteral("DesignerActionManager|resetPosition"));
         foreach (ModelNode node, selectionState.selectedModelNodes()) {
-            node.removeProperty("x");
-            node.removeProperty("y");
+            QmlItemNode itemNode(node);
+            itemNode.removeProperty("x");
+            itemNode.removeProperty("y");
         }
+        transaction.commit();
     } catch (const RewritingException &e) { //better save then sorry
         e.showException();
     }
 }
 
-void goIntoComponent(const SelectionContext &selectionState)
+void goIntoComponentOperation(const SelectionContext &selectionState)
 {
     goIntoComponent(selectionState.currentSingleSelectedNode());
 }
@@ -353,7 +372,8 @@ void resetZ(const SelectionContext &selectionState)
 
     RewriterTransaction transaction(selectionState.view(), QByteArrayLiteral("DesignerActionManager|resetZ"));
     foreach (ModelNode node, selectionState.selectedModelNodes()) {
-        node.removeProperty("z");
+        QmlItemNode itemNode(node);
+        itemNode.removeProperty("z");
     }
 }
 
@@ -467,7 +487,7 @@ static void layoutHelperFunction(const SelectionContext &selectionContext,
         return;
 
     if (QmlItemNode::isValidQmlItemNode(selectionContext.firstSelectedModelNode())) {
-        QmlItemNode qmlItemNode = QmlItemNode(selectionContext.firstSelectedModelNode());
+        const QmlItemNode qmlItemNode = QmlItemNode(selectionContext.firstSelectedModelNode());
 
         if (qmlItemNode.hasInstanceParentItem()) {
 
@@ -568,7 +588,7 @@ static inline QList<IOptionsPage*> sortedOptionsPages()
 static PropertyNameList sortedPropertyNameList(const PropertyNameList &nameList)
 {
     PropertyNameList sortedPropertyNameList = nameList;
-    qStableSort(sortedPropertyNameList);
+    std::stable_sort(sortedPropertyNameList.begin(), sortedPropertyNameList.end());
     return sortedPropertyNameList;
 }
 
@@ -641,7 +661,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
 {
     ModelNode modelNode;
     if (selectionState.singleNodeIsSelected())
-        modelNode = selectionState.selectedModelNodes().first();
+        modelNode = selectionState.selectedModelNodes().constFirst();
 
     bool isModelNodeRoot = true;
 
@@ -662,6 +682,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
 
             QmlObjectNode qmlObjectNode(modelNode);
             qmlObjectNode.ensureAliasExport();
+            transaction.commit();
         }  catch (RewritingException &exception) { //better safe than sorry! There always might be cases where we fail
             exception.showException();
         }
@@ -669,10 +690,11 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
 
     QString itemId = modelNode.id();
 
-    const QString fileName = QmlDesignerPlugin::instance()->documentManager().currentDesignDocument()->fileName().toString();
-    const QString typeName = QmlDesignerPlugin::instance()->documentManager().currentDesignDocument()->fileName().toFileInfo().baseName();
+    const Utils::FileName currentDesignDocument = QmlDesignerPlugin::instance()->documentManager().currentDesignDocument()->fileName();
+    const QString fileName = currentDesignDocument.toString();
+    const QString typeName = currentDesignDocument.toFileInfo().baseName();
 
-    QStringList signalNames = cleanSignalNames(getSortedSignalNameList(selectionState.selectedModelNodes().first()));
+    QStringList signalNames = cleanSignalNames(getSortedSignalNameList(selectionState.selectedModelNodes().constFirst()));
 
     QList<QmlJSEditor::FindReferences::Usage> usages = QmlJSEditor::FindReferences::findUsageOfType(fileName, typeName);
 
@@ -683,12 +705,12 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
         return;
     }
 
-    usages = FindImplementation::run(usages.first().path, typeName, itemId);
+    usages = FindImplementation::run(usages.constFirst().path, typeName, itemId);
 
     Core::ModeManager::activateMode(Core::Constants::MODE_EDIT);
 
     if (usages.count() > 0 && (addAlwaysNewSlot || usages.count() < 2)  && (!isModelNodeRoot  || addAlwaysNewSlot)) {
-        Core::EditorManager::openEditorAt(usages.first().path, usages.first().line, usages.first().col);
+        Core::EditorManager::openEditorAt(usages.constFirst().path, usages.constFirst().line, usages.constFirst().col);
 
         if (!signalNames.isEmpty()) {
             AddSignalHandlerDialog *dialog = new AddSignalHandlerDialog(Core::ICore::dialogParent());
@@ -714,7 +736,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
                 //Move cursor to correct curser position
                 const QString filePath = Core::EditorManager::currentDocument()->filePath().toString();
                 QList<QmlJSEditor::FindReferences::Usage> usages = FindImplementation::run(filePath, typeName, itemId);
-                Core::EditorManager::openEditorAt(filePath, usages.first().line, usages.first().col + 1);
+                Core::EditorManager::openEditorAt(filePath, usages.constFirst().line, usages.constFirst().col + 1);
             } );
             dialog->show();
 
@@ -722,7 +744,7 @@ void addSignalHandlerOrGotoImplementation(const SelectionContext &selectionState
         return;
     }
 
-    Core::EditorManager::openEditorAt(usages.first().path, usages.first().line, usages.first().col + 1);
+    Core::EditorManager::openEditorAt(usages.constFirst().path, usages.constFirst().line, usages.constFirst().col + 1);
 }
 
 void removeLayout(const SelectionContext &selectionContext)
@@ -776,7 +798,7 @@ void moveToComponent(const SelectionContext &selectionContext)
 {
     ModelNode modelNode;
     if (selectionContext.singleNodeIsSelected())
-        modelNode = selectionContext.selectedModelNodes().first();
+        modelNode = selectionContext.selectedModelNodes().constFirst();
 
     if (modelNode.isValid())
         selectionContext.view()->model()->rewriterView()->moveToComponent(modelNode);
@@ -792,6 +814,237 @@ void addNewSignalHandler(const SelectionContext &selectionState)
     addSignalHandlerOrGotoImplementation(selectionState, true);
 }
 
+void addItemToStackedContainer(const SelectionContext &selectionContext)
+{
+    AbstractView *view = selectionContext.view();
+
+    QTC_ASSERT(view && selectionContext.hasSingleSelectedModelNode(), return);
+    ModelNode container = selectionContext.currentSingleSelectedNode();
+    QTC_ASSERT(container.isValid(), return);
+    QTC_ASSERT(container.metaInfo().isValid(), return);
+
+    const PropertyName propertyName = getIndexPropertyName(container);
+    QTC_ASSERT(container.metaInfo().hasProperty(propertyName), return);
+    BindingProperty binding = container.bindingProperty(propertyName);
+
+    /* Check if there is already a TabBar attached. */
+    ModelNode potentialTabBar;
+    if (binding.isValid()) {
+        AbstractProperty bindingTarget = binding.resolveToProperty();
+        if (bindingTarget.isValid()) { // In this case the stacked container might be hooked up to a TabBar
+            potentialTabBar = bindingTarget.parentModelNode();
+
+            if (!(potentialTabBar.metaInfo().isValid()
+                  && potentialTabBar.metaInfo().isSubclassOf("QtQuick.Controls.TabBar")))
+                potentialTabBar = ModelNode();
+        }
+    }
+
+    try {
+        RewriterTransaction transaction =
+                view->beginRewriterTransaction(QByteArrayLiteral("DesignerActionManager:addItemToStackedContainer"));
+
+        NodeMetaInfo itemMetaInfo = view->model()->metaInfo("QtQuick.Item", -1, -1);
+        QTC_ASSERT(itemMetaInfo.isValid(), return);
+        QTC_ASSERT(itemMetaInfo.majorVersion() == 2, return);
+
+        QmlDesigner::ModelNode itemNode =
+                view->createModelNode("QtQuick.Item", itemMetaInfo.majorVersion(), itemMetaInfo.minorVersion());
+
+        container.defaultNodeListProperty().reparentHere(itemNode);
+
+        if (potentialTabBar.isValid()) {// The stacked container is hooked up to a TabBar
+            NodeMetaInfo tabButtonMetaInfo = view->model()->metaInfo("QtQuick.Controls.TabButton", -1, -1);
+            if (tabButtonMetaInfo.isValid()) {
+                const int buttonIndex = potentialTabBar.directSubModelNodes().count();
+                ModelNode tabButtonNode =
+                        view->createModelNode("QtQuick.Controls.TabButton",
+                                              tabButtonMetaInfo.majorVersion(),
+                                              tabButtonMetaInfo.minorVersion());
+
+                tabButtonNode.variantProperty("text").setValue(QString::fromLatin1("Tab %1").arg(buttonIndex));
+                potentialTabBar.defaultNodeListProperty().reparentHere(tabButtonNode);
+
+            }
+        }
+
+        transaction.commit();
+    }  catch (RewritingException &exception) { //better safe than sorry! There always might be cases where we fail
+        exception.showException();
+    }
+}
+
+PropertyName getIndexPropertyName(const ModelNode &modelNode)
+{
+    const PropertyName propertyName = NodeHints::fromModelNode(modelNode).indexPropertyForStackedContainer().toUtf8();
+
+    if (modelNode.metaInfo().hasProperty(propertyName))
+        return propertyName;
+
+    if (modelNode.metaInfo().hasProperty("currentIndex"))
+        return "currentIndex";
+
+    if (modelNode.metaInfo().hasProperty("index"))
+        return "index";
+
+    return PropertyName();
+}
+
+static void setIndexProperty(const AbstractProperty &property, const QVariant &value)
+{
+    if (!property.exists() || property.isVariantProperty()) {
+        /* Using QmlObjectNode ensures we take states into account. */
+        property.parentQmlObjectNode().setVariantProperty(property.name(), value);
+        return;
+    } else if (property.isBindingProperty()) {
+        /* Track one binding to the original source, incase a TabBar is attached */
+        const AbstractProperty orignalProperty = property.toBindingProperty().resolveToProperty();
+        if (orignalProperty.isValid() && (orignalProperty.isVariantProperty() || !orignalProperty.exists())) {
+            orignalProperty.parentQmlObjectNode().setVariantProperty(orignalProperty.name(), value);
+            return;
+        }
+    }
+
+    const QString propertyName = QString::fromUtf8(property.name());
+
+    QString title = QCoreApplication::translate("ModelNodeOperations", "Cannot Set Property %1").arg(propertyName);
+    QString description = QCoreApplication::translate("ModelNodeOperations", "The property %1 is bound to an expression.").arg(propertyName);
+    Core::AsynchronousMessageBox::warning(title, description);
+}
+
+void increaseIndexOfStackedContainer(const SelectionContext &selectionContext)
+{
+    AbstractView *view = selectionContext.view();
+
+    QTC_ASSERT(view && selectionContext.hasSingleSelectedModelNode(), return);
+    ModelNode container = selectionContext.currentSingleSelectedNode();
+    QTC_ASSERT(container.isValid(), return);
+    QTC_ASSERT(container.metaInfo().isValid(), return);
+
+    const PropertyName propertyName = getIndexPropertyName(container);
+    QTC_ASSERT(container.metaInfo().hasProperty(propertyName), return);
+
+    QmlItemNode containerItemNode(container);
+    QTC_ASSERT(containerItemNode.isValid(), return);
+
+    int value = containerItemNode.instanceValue(propertyName).toInt();
+    ++value;
+
+    const int maxValue = container.directSubModelNodes().count();
+
+    QTC_ASSERT(value < maxValue, return);
+
+    setIndexProperty(container.property(propertyName), value);
+}
+
+void decreaseIndexOfStackedContainer(const SelectionContext &selectionContext)
+{
+    AbstractView *view = selectionContext.view();
+
+    QTC_ASSERT(view && selectionContext.hasSingleSelectedModelNode(), return);
+    ModelNode container = selectionContext.currentSingleSelectedNode();
+    QTC_ASSERT(container.isValid(), return);
+    QTC_ASSERT(container.metaInfo().isValid(), return);
+
+    const PropertyName propertyName = getIndexPropertyName(container);
+    QTC_ASSERT(container.metaInfo().hasProperty(propertyName), return);
+
+    QmlItemNode containerItemNode(container);
+    QTC_ASSERT(containerItemNode.isValid(), return);
+
+    int value = containerItemNode.instanceValue(propertyName).toInt();
+    --value;
+
+    QTC_ASSERT(value > -1, return);
+
+    setIndexProperty(container.property(propertyName), value);
+}
+
+void addTabBarToStackedContainer(const SelectionContext &selectionContext)
+{
+    AbstractView *view = selectionContext.view();
+
+    QTC_ASSERT(view && selectionContext.hasSingleSelectedModelNode(), return);
+    ModelNode container = selectionContext.currentSingleSelectedNode();
+    QTC_ASSERT(container.isValid(), return);
+    QTC_ASSERT(container.metaInfo().isValid(), return);
+
+    NodeMetaInfo tabBarMetaInfo = view->model()->metaInfo("QtQuick.Controls.TabBar", -1, -1);
+    QTC_ASSERT(tabBarMetaInfo.isValid(), return);
+    QTC_ASSERT(tabBarMetaInfo.majorVersion() == 2, return);
+
+    NodeMetaInfo tabButtonMetaInfo = view->model()->metaInfo("QtQuick.Controls.TabButton", -1, -1);
+    QTC_ASSERT(tabButtonMetaInfo.isValid(), return);
+    QTC_ASSERT(tabButtonMetaInfo.majorVersion() == 2, return);
+
+    QmlItemNode containerItemNode(container);
+    QTC_ASSERT(containerItemNode.isValid(), return);
+
+    const PropertyName indexPropertyName = getIndexPropertyName(container);
+    QTC_ASSERT(container.metaInfo().hasProperty(indexPropertyName), return);
+
+    try {
+        RewriterTransaction transaction =
+                view->beginRewriterTransaction(QByteArrayLiteral("DesignerActionManager:addItemToStackedContainer"));
+
+        ModelNode tabBarNode =
+                view->createModelNode("QtQuick.Controls.TabBar",
+                                      tabBarMetaInfo.majorVersion(),
+                                      tabBarMetaInfo.minorVersion());
+
+        container.parentProperty().reparentHere(tabBarNode);
+
+        const int maxValue = container.directSubModelNodes().count();
+
+        QmlItemNode tabBarItem(tabBarNode);
+
+        tabBarItem.anchors().setAnchor(AnchorLineLeft, containerItemNode, AnchorLineLeft);
+        tabBarItem.anchors().setAnchor(AnchorLineRight, containerItemNode, AnchorLineRight);
+        tabBarItem.anchors().setAnchor(AnchorLineBottom, containerItemNode, AnchorLineTop);
+
+        for (int i = 0; i < maxValue; ++i) {
+            ModelNode tabButtonNode =
+                    view->createModelNode("QtQuick.Controls.TabButton",
+                                          tabButtonMetaInfo.majorVersion(),
+                                          tabButtonMetaInfo.minorVersion());
+
+            tabButtonNode.variantProperty("text").setValue(QString::fromLatin1("Tab %1").arg(i));
+            tabBarNode.defaultNodeListProperty().reparentHere(tabButtonNode);
+        }
+
+        const QString id = tabBarNode.validId();
+
+        container.removeProperty(indexPropertyName);
+        const QString expression = id + "." + QString::fromLatin1(indexPropertyName);
+        container.bindingProperty(indexPropertyName).setExpression(expression);
+
+        transaction.commit();
+    }  catch (RewritingException &exception) { //better safe than sorry! There always might be cases where we fail
+        exception.showException();
+    }
+}
+
+bool addImageToProject(const QString &fileName, const QString &directory)
+{
+    const QString targetFile = directory + "/" + QFileInfo(fileName).fileName();
+    const bool success = QFile::copy(fileName, targetFile);
+
+    auto document = QmlDesignerPlugin::instance()->currentDesignDocument();
+
+    QTC_ASSERT(document, return false);
+
+    if (success) {
+        ProjectExplorer::Node *node = ProjectExplorer::ProjectTree::nodeForFile(document->fileName());
+        if (node) {
+            ProjectExplorer::FolderNode *containingFolder = node->parentFolderNode();
+            if (containingFolder)
+                containingFolder->addFiles(QStringList(targetFile));
+        }
+    }
+
+    return success;
+}
+
 } // namespace Mode
 
 } //QmlDesigner
