@@ -34,6 +34,7 @@
 #include <model.h>
 #include <modelnode.h>
 #include <metainfo.h>
+#include <nodehints.h>
 #include <rewriterview.h>
 
 #include "abstractproperty.h"
@@ -43,6 +44,9 @@
 #include "nodelistproperty.h"
 #include "nodeproperty.h"
 #include "qmlchangeset.h"
+#include "qmlstate.h"
+#include "qmltimelinemutator.h"
+#include "qmltimelinekeyframes.h"
 
 #include "createscenecommand.h"
 #include "createinstancescommand.h"
@@ -122,7 +126,7 @@ NodeInstanceView::~NodeInstanceView()
 
 //\{
 
-bool isSkippedRootNode(const ModelNode &node)
+bool static isSkippedRootNode(const ModelNode &node)
 {
     static const PropertyNameList skipList({"Qt.ListModel", "QtQuick.ListModel", "Qt.ListModel", "QtQuick.ListModel"});
 
@@ -133,7 +137,7 @@ bool isSkippedRootNode(const ModelNode &node)
 }
 
 
-bool isSkippedNode(const ModelNode &node)
+bool static isSkippedNode(const ModelNode &node)
 {
     static const PropertyNameList skipList({"QtQuick.XmlRole", "Qt.XmlRole", "QtQuick.ListElement", "Qt.ListElement"});
 
@@ -143,6 +147,22 @@ bool isSkippedNode(const ModelNode &node)
     return false;
 }
 
+bool static parentTakesOverRendering(const ModelNode &modelNode)
+{
+    if (!modelNode.isValid())
+        return false;
+
+    ModelNode currentNode = modelNode;
+
+    while (currentNode.hasParentProperty()) {
+        currentNode = currentNode.parentProperty().parentModelNode();
+        if (NodeHints::fromModelNode(currentNode).takesOverRenderingOfChildren())
+            return true;
+    }
+
+    return false;
+}
+
 /*!
     Notifies the view that it was attached to \a model. For every model node in
     the model, a NodeInstance will be created.
@@ -202,6 +222,7 @@ void NodeInstanceView::restartProcess()
     if (rootNodeInstance().isValid())
         rootNodeInstance().setError({});
     emitInstanceErrorChange({});
+    emitDocumentMessage({}, {});
 
     if (m_restartProcessTimerId)
         killTimer(m_restartProcessTimerId);
@@ -244,9 +265,9 @@ void NodeInstanceView::nodeCreated(const ModelNode &createdNode)
     propertyList.append(createdNode.variantProperty("y"));
     updatePosition(propertyList);
 
-    nodeInstanceServer()->createInstances(createCreateInstancesCommand(QList<NodeInstance>() << instance));
+    nodeInstanceServer()->createInstances(createCreateInstancesCommand({instance}));
     nodeInstanceServer()->changePropertyValues(createChangeValueCommand(createdNode.variantProperties()));
-    nodeInstanceServer()->completeComponent(createComponentCompleteCommand(QList<NodeInstance>() << instance));
+    nodeInstanceServer()->completeComponent(createComponentCompleteCommand({instance}));
 }
 
 /*! Notifies the view that \a removedNode will be removed.
@@ -420,7 +441,7 @@ void NodeInstanceView::nodeIdChanged(const ModelNode& node, const QString& /*new
 {
     if (hasInstanceForModelNode(node)) {
         NodeInstance instance = instanceForModelNode(node);
-        nodeInstanceServer()->changeIds(createChangeIdsCommand(QList<NodeInstance>() << instance));
+        nodeInstanceServer()->changeIds(createChangeIdsCommand({instance}));
     }
 }
 
@@ -459,16 +480,16 @@ void NodeInstanceView::auxiliaryDataChanged(const ModelNode &node, const Propert
             QVariant value = data;
             if (value.isValid()) {
                 PropertyValueContainer container(instance.instanceId(), name, value, TypeName());
-                ChangeAuxiliaryCommand changeAuxiliaryCommand(QVector<PropertyValueContainer>() << container);
+                ChangeAuxiliaryCommand changeAuxiliaryCommand({container});
                 nodeInstanceServer()->changeAuxiliaryValues(changeAuxiliaryCommand);
             } else {
                 if (node.hasVariantProperty(name)) {
                     PropertyValueContainer container(instance.instanceId(), name, node.variantProperty(name).value(), TypeName());
-                    ChangeValuesCommand changeValueCommand(QVector<PropertyValueContainer>() << container);
+                    ChangeValuesCommand changeValueCommand({container});
                     nodeInstanceServer()->changePropertyValues(changeValueCommand);
                 } else if (node.hasBindingProperty(name)) {
                     PropertyBindingContainer container(instance.instanceId(), name, node.bindingProperty(name).expression(), TypeName());
-                    ChangeBindingsCommand changeValueCommand(QVector<PropertyBindingContainer>() << container);
+                    ChangeBindingsCommand changeValueCommand({container});
                     nodeInstanceServer()->changePropertyBindings(changeValueCommand);
                 }
             }
@@ -651,13 +672,13 @@ void NodeInstanceView::updateChildren(const NodeAbstractProperty &newPropertyPar
 void setXValue(NodeInstance &instance, const VariantProperty &variantProperty, QMultiHash<ModelNode, InformationName> &informationChangeHash)
 {
     instance.setX(variantProperty.value().toDouble());
-    informationChangeHash.insert(variantProperty.parentModelNode(), Transform);
+    informationChangeHash.insert(instance.modelNode(), Transform);
 }
 
 void setYValue(NodeInstance &instance, const VariantProperty &variantProperty, QMultiHash<ModelNode, InformationName> &informationChangeHash)
 {
     instance.setY(variantProperty.value().toDouble());
-    informationChangeHash.insert(variantProperty.parentModelNode(), Transform);
+    informationChangeHash.insert(instance.modelNode(), Transform);
 }
 
 
@@ -668,7 +689,7 @@ void NodeInstanceView::updatePosition(const QList<VariantProperty> &propertyList
     foreach (const VariantProperty &variantProperty, propertyList) {
         if (variantProperty.name() == "x") {
             const ModelNode modelNode = variantProperty.parentModelNode();
-            if (QmlPropertyChanges::isValidQmlPropertyChanges(modelNode)) {
+            if (!currentState().isBaseState() && QmlPropertyChanges::isValidQmlPropertyChanges(modelNode)) {
                 ModelNode targetModelNode = QmlPropertyChanges(modelNode).target();
                 if (targetModelNode.isValid()) {
                     NodeInstance instance = instanceForModelNode(targetModelNode);
@@ -680,7 +701,7 @@ void NodeInstanceView::updatePosition(const QList<VariantProperty> &propertyList
             }
         } else if (variantProperty.name() == "y") {
             const ModelNode modelNode = variantProperty.parentModelNode();
-            if (QmlPropertyChanges::isValidQmlPropertyChanges(modelNode)) {
+            if (!currentState().isBaseState() && QmlPropertyChanges::isValidQmlPropertyChanges(modelNode)) {
                 ModelNode targetModelNode = QmlPropertyChanges(modelNode).target();
                 if (targetModelNode.isValid()) {
                     NodeInstance instance = instanceForModelNode(targetModelNode);
@@ -690,6 +711,21 @@ void NodeInstanceView::updatePosition(const QList<VariantProperty> &propertyList
                 NodeInstance instance = instanceForModelNode(modelNode);
                 setYValue(instance, variantProperty, informationChangeHash);
             }
+        } else if (currentTimeline().isValid()
+                   && variantProperty.name() == "value"
+                   &&  QmlTimelineFrames::isValidKeyframe(variantProperty.parentModelNode())) {
+
+            QmlTimelineFrames frames = QmlTimelineFrames::keyframesForKeyframe(variantProperty.parentModelNode());
+
+            if (frames.isValid() && frames.propertyName() == "x" && frames.target().isValid()) {
+
+                NodeInstance instance = instanceForModelNode(frames.target());
+                setXValue(instance, variantProperty, informationChangeHash);
+            } else if (frames.isValid() && frames.propertyName() == "y" && frames.target().isValid()) {
+                NodeInstance instance = instanceForModelNode(frames.target());
+                setYValue(instance, variantProperty, informationChangeHash);
+            }
+
         }
     }
 
@@ -798,6 +834,11 @@ CreateSceneCommand NodeInstanceView::createCreateSceneCommand()
         if (instance.modelNode().metaInfo().isSubclassOf("QtQuick.Item"))
             nodeMetaType = InstanceContainer::ItemMetaType;
 
+        InstanceContainer::NodeFlags nodeFlags;
+
+        if (parentTakesOverRendering(instance.modelNode()))
+            nodeFlags |= InstanceContainer::ParentTakesOverRendering;
+
         InstanceContainer container(instance.instanceId(),
                                     instance.modelNode().type(),
                                     instance.modelNode().majorVersion(),
@@ -805,8 +846,8 @@ CreateSceneCommand NodeInstanceView::createCreateSceneCommand()
                                     instance.modelNode().metaInfo().componentFileName(),
                                     instance.modelNode().nodeSource(),
                                     nodeSourceType,
-                                    nodeMetaType
-                                   );
+                                    nodeMetaType,
+                                    nodeFlags);
 
         instanceContainerList.append(container);
     }
@@ -862,8 +903,8 @@ CreateSceneCommand NodeInstanceView::createCreateSceneCommand()
 
         if (versionString.contains(QStringLiteral("."))) {
             const QStringList splittedString = versionString.split(QStringLiteral("."));
-            majorVersion = splittedString.first().toInt();
-            minorVersion = splittedString.last().toInt();
+            majorVersion = splittedString.constFirst().toInt();
+            minorVersion = splittedString.constLast().toInt();
         }
 
         bool isItem = false;
@@ -939,8 +980,20 @@ CreateInstancesCommand NodeInstanceView::createCreateInstancesCommand(const QLis
         if (instance.modelNode().metaInfo().isSubclassOf("QtQuick.Item"))
             nodeMetaType = InstanceContainer::ItemMetaType;
 
-        InstanceContainer container(instance.instanceId(), instance.modelNode().type(), instance.modelNode().majorVersion(), instance.modelNode().minorVersion(),
-                                    instance.modelNode().metaInfo().componentFileName(), instance.modelNode().nodeSource(), nodeSourceType, nodeMetaType);
+        InstanceContainer::NodeFlags nodeFlags;
+
+        if (parentTakesOverRendering(instance.modelNode()))
+            nodeFlags |= InstanceContainer::ParentTakesOverRendering;
+
+        InstanceContainer container(instance.instanceId(),
+                                    instance.modelNode().type(),
+                                    instance.modelNode().majorVersion(),
+                                    instance.modelNode().minorVersion(),
+                                    instance.modelNode().metaInfo().componentFileName(),
+                                    instance.modelNode().nodeSource(),
+                                    nodeSourceType,
+                                    nodeMetaType,
+                                    nodeFlags);
         containerList.append(container);
     }
 
@@ -1082,7 +1135,7 @@ RemovePropertiesCommand NodeInstanceView::createRemovePropertiesCommand(const QL
 
 RemoveSharedMemoryCommand NodeInstanceView::createRemoveSharedMemoryCommand(const QString &sharedMemoryTypeName, quint32 keyNumber)
 {
-    return RemoveSharedMemoryCommand(sharedMemoryTypeName, QVector<qint32>() << keyNumber);
+    return RemoveSharedMemoryCommand(sharedMemoryTypeName, {static_cast<qint32>(keyNumber)});
 }
 
 RemoveSharedMemoryCommand NodeInstanceView::createRemoveSharedMemoryCommand(const QString &sharedMemoryTypeName, const QList<ModelNode> &nodeList)
@@ -1107,7 +1160,7 @@ void NodeInstanceView::valuesChanged(const ValuesChangedCommand &command)
             NodeInstance instance = instanceForId(container.instanceId());
             if (instance.isValid()) {
                 instance.setProperty(container.name(), container.value());
-                valuePropertyChangeList.append(qMakePair(instance.modelNode(), container.name()));
+                valuePropertyChangeList.append({instance.modelNode(), container.name()});
             }
         }
     }
