@@ -29,8 +29,6 @@
 #include <model.h>
 #include <nodemetainfo.h>
 #include "internalnode_p.h"
-#include <QHash>
-#include <QTextStream>
 #include "invalidargumentexception.h"
 #include "invalididexception.h"
 #include "invalidmodelnodeexception.h"
@@ -44,6 +42,11 @@
 #include "nodeproperty.h"
 #include <rewriterview.h>
 
+#include <QHash>
+#include <QRegExp>
+#include <QSet>
+#include <QTextStream>
+
 namespace QmlDesigner {
 using namespace QmlDesigner::Internal;
 
@@ -143,12 +146,76 @@ QString ModelNode::validId()
 
 static bool idIsQmlKeyWord(const QString& id)
 {
-    QStringList keywords;
-    keywords << QLatin1String("import") << QLatin1String("as");
+    static const QSet<QString> keywords = {
+        "as",
+        "break",
+        "case",
+        "catch",
+        "continue",
+        "debugger",
+        "default",
+        "delete",
+        "do",
+        "else",
+        "finally",
+        "for",
+        "function",
+        "if",
+        "import",
+        "in",
+        "instanceof",
+        "new",
+        "return",
+        "switch",
+        "this",
+        "throw",
+        "try",
+        "typeof",
+        "var",
+        "void",
+        "while",
+        "with"
+    };
 
     return keywords.contains(id);
 }
 
+static bool isIdToAvoid(const QString& id)
+{
+    static const QSet<QString> ids = {
+        "top",
+        "bottom",
+        "left",
+        "right",
+        "width",
+        "height",
+        "x",
+        "y",
+        "opacity",
+        "parent",
+        "item",
+        "flow",
+        "color",
+        "margin",
+        "padding",
+        "border",
+        "font",
+        "text",
+        "source",
+        "state",
+        "visible",
+        "focus",
+        "data",
+        "clip",
+        "layer",
+        "scale",
+        "enabled",
+        "anchors"
+    };
+
+    return ids.contains(id);
+}
+
 static bool idContainsWrongLetter(const QString& id)
 {
     static QRegExp idExpr(QStringLiteral("[a-z_][a-zA-Z0-9_]*"));
@@ -157,7 +224,7 @@ static bool idContainsWrongLetter(const QString& id)
 
 bool ModelNode::isValidId(const QString &id)
 {
-    return id.isEmpty() || (!idContainsWrongLetter(id) && !idIsQmlKeyWord(id));
+    return id.isEmpty() || (!idContainsWrongLetter(id) && !idIsQmlKeyWord(id) && !isIdToAvoid(id));
 }
 
 bool ModelNode::hasId() const
@@ -243,7 +310,14 @@ QString ModelNode::simplifiedTypeName() const
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
     }
 
-    return QString::fromUtf8(type().split('.').last());
+    return QString::fromUtf8(type().split('.').constLast());
+}
+
+QString ModelNode::displayName() const
+{
+    if (hasId())
+        return id();
+    return simplifiedTypeName();
 }
 
 /*! \brief Returns whether the node is valid
@@ -342,6 +416,17 @@ void ModelNode::setParentProperty(NodeAbstractProperty parent)
     parent.reparentHere(*this);
 }
 
+void ModelNode::changeType(const TypeName &typeName, int majorVersion, int minorVersion)
+{
+    if (!isValid()) {
+        Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
+        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
+    }
+
+    model()->d->changeNodeType(internalNode(), typeName, majorVersion, minorVersion);
+
+}
+
 void ModelNode::setParentProperty(const ModelNode &newParentNode, const PropertyName &propertyName)
 {
     setParentProperty(newParentNode.nodeAbstractProperty(propertyName));
@@ -645,11 +730,6 @@ void ModelNode::destroy()
   */
 bool operator ==(const ModelNode &firstNode, const ModelNode &secondNode)
 {
-    if (firstNode.m_internalNode.isNull() || secondNode.m_internalNode.isNull()) {
-        Q_ASSERT_X(0, Q_FUNC_INFO, "model node is invalid");
-        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
-
     return firstNode.internalId() == secondNode.internalId();
 }
 
@@ -658,31 +738,17 @@ bool operator ==(const ModelNode &firstNode, const ModelNode &secondNode)
   */
 bool operator !=(const ModelNode &firstNode, const ModelNode &secondNode)
 {
-    if (firstNode.m_internalNode.isNull() || secondNode.m_internalNode.isNull()) {
-        Q_ASSERT_X(0, Q_FUNC_INFO, "model node is invalid");
-        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
-
     return firstNode.internalId() != secondNode.internalId();
 }
 
 bool operator <(const ModelNode &firstNode, const ModelNode &secondNode)
 {
-    if (firstNode.m_internalNode.isNull() || secondNode.m_internalNode.isNull()) {
-        Q_ASSERT_X(0, Q_FUNC_INFO, "model node is invalid");
-        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
-
     return firstNode.internalId() < secondNode.internalId();
 }
 
 
 Internal::InternalNodePointer ModelNode::internalNode() const
 {
-    if (!isValid()) {
-        Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
-        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
     return m_internalNode;
 }
 
@@ -793,10 +859,9 @@ bool ModelNode::isSelected() const
 */
 bool ModelNode::isRootNode() const
 {
-    if (!isValid()) {
-        Q_ASSERT_X(isValid(), Q_FUNC_INFO, "model node is invalid");
-        throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
-    }
+    if (!isValid())
+        return false;
+
     return view()->rootModelNode() == *this;
 }
 
@@ -1042,6 +1107,9 @@ bool ModelNode::isComponent() const
     if (!isValid())
         throw InvalidModelNodeException(__LINE__, __FUNCTION__, __FILE__);
 
+    if (!metaInfo().isValid())
+        return false;
+
     if (metaInfo().isFileComponent())
         return true;
 
@@ -1049,11 +1117,16 @@ bool ModelNode::isComponent() const
         return true;
 
     if (metaInfo().isView() && hasNodeProperty("delegate")) {
-        if (nodeProperty("delegate").modelNode().metaInfo().isFileComponent())
-            return true;
-
-        if (nodeProperty("delegate").modelNode().nodeSourceType() == ModelNode::NodeWithComponentSource)
-            return true;
+        const ModelNode delegateNode = nodeProperty("delegate").modelNode();
+        if (delegateNode.isValid()) {
+            if (delegateNode.hasMetaInfo()) {
+                const NodeMetaInfo delegateMetaInfo = delegateNode.metaInfo();
+                if (delegateMetaInfo.isValid() && delegateMetaInfo.isFileComponent())
+                    return true;
+            }
+            if (delegateNode.nodeSourceType() == ModelNode::NodeWithComponentSource)
+                return true;
+        }
     }
 
     if (metaInfo().isSubclassOf("QtQuick.Loader")) {
@@ -1065,7 +1138,7 @@ bool ModelNode::isComponent() const
          * the default property is always implcitly a NodeListProperty. This is something that has to be fixed.
          */
 
-            ModelNode componentNode = nodeListProperty("component").toModelNodeList().first();
+            ModelNode componentNode = nodeListProperty("component").toModelNodeList().constFirst();
             if (componentNode.nodeSourceType() == ModelNode::NodeWithComponentSource)
                 return true;
             if (componentNode.metaInfo().isFileComponent())
@@ -1094,4 +1167,20 @@ bool ModelNode::isSubclassOf(const TypeName &typeName, int majorVersion, int min
     return false;
 }
 
+QIcon ModelNode::typeIcon() const
+{
+    if (isValid()) {
+        // if node has no own icon, search for it in the itemlibrary
+        const ItemLibraryInfo *libraryInfo = model()->metaInfo().itemLibraryInfo();
+        QList <ItemLibraryEntry> itemLibraryEntryList = libraryInfo->entriesForType(
+                    type(), majorVersion(), minorVersion());
+        if (!itemLibraryEntryList.isEmpty())
+            return itemLibraryEntryList.constFirst().typeIcon();
+        else if (metaInfo().isValid())
+            return QIcon(QStringLiteral(":/ItemLibrary/images/item-default-icon.png"));
+    }
+
+    return QIcon(QStringLiteral(":/ItemLibrary/images/item-invalid-icon.png"));
+}
+
 }
