@@ -25,6 +25,8 @@
 
 #include "designersettings.h"
 
+#include <qmldesignerplugin.h>
+
 #include <QSettings>
 
 namespace QmlDesigner {
@@ -52,13 +54,16 @@ void DesignerSettings::fromSettings(QSettings *settings)
     restoreValue(settings, DesignerSettingsKey::CONTAINERPADDING, 8);
     restoreValue(settings, DesignerSettingsKey::CANVASWIDTH, 10000);
     restoreValue(settings, DesignerSettingsKey::CANVASHEIGHT, 10000);
+    restoreValue(settings, DesignerSettingsKey::ROOT_ELEMENT_INIT_WIDTH, 640);
+    restoreValue(settings, DesignerSettingsKey::ROOT_ELEMENT_INIT_HEIGHT, 480);
     restoreValue(settings, DesignerSettingsKey::WARNING_FOR_FEATURES_IN_DESIGNER, true);
+    restoreValue(settings, DesignerSettingsKey::WARNING_FOR_QML_FILES_INSTEAD_OF_UIQML_FILES, true);
     restoreValue(settings, DesignerSettingsKey::WARNING_FOR_DESIGNER_FEATURES_IN_EDITOR, false);
     restoreValue(settings, DesignerSettingsKey::SHOW_DEBUGVIEW, false);
     restoreValue(settings, DesignerSettingsKey::ENABLE_DEBUGVIEW, false);
     restoreValue(settings, DesignerSettingsKey::ALWAYS_SAFE_IN_CRUMBLEBAR, false);
     restoreValue(settings, DesignerSettingsKey::USE_ONLY_FALLBACK_PUPPET, true);
-    restoreValue(settings, DesignerSettingsKey::USE_QSTR_FUNCTION, true);
+    restoreValue(settings, DesignerSettingsKey::TYPE_OF_QSTR_FUNCTION, 0);
     restoreValue(settings, DesignerSettingsKey::PUPPET_FALLBACK_DIRECTORY);
     restoreValue(settings, DesignerSettingsKey::PUPPET_TOPLEVEL_BUILD_DIRECTORY);
     restoreValue(settings, DesignerSettingsKey::CONTROLS_STYLE);
@@ -67,6 +72,11 @@ void DesignerSettings::fromSettings(QSettings *settings)
     restoreValue(settings, DesignerSettingsKey::PUPPET_KILL_TIMEOUT, 3000); // this has no ui at the moment
     restoreValue(settings, DesignerSettingsKey::DEBUG_PUPPET, QString());
     restoreValue(settings, DesignerSettingsKey::FORWARD_PUPPET_OUTPUT, QString());
+    restoreValue(settings, DesignerSettingsKey::REFORMAT_UI_QML_FILES, true);
+    restoreValue(settings, DesignerSettingsKey::IGNORE_DEVICE_PIXEL_RATIO, false);
+    restoreValue(settings, DesignerSettingsKey::STATESEDITOR_EXPANDED, false);
+    restoreValue(settings, DesignerSettingsKey::NAVIGATOR_SHOW_ONLY_VISIBLE_ITEMS, true);
+    restoreValue(settings, DesignerSettingsKey::STANDALONE_MODE, false);
 
     settings->endGroup();
     settings->endGroup();
@@ -94,4 +104,17 @@ void DesignerSettings::toSettings(QSettings *settings) const
     settings->endGroup();
 }
 
+QVariant DesignerSettings::getValue(const QByteArray &key)
+{
+    DesignerSettings settings = QmlDesignerPlugin::instance()->settings();
+    return settings.value(key);
+}
+
+void DesignerSettings::setValue(const QByteArray &key, const QVariant &value)
+{
+    DesignerSettings settings = QmlDesignerPlugin::instance()->settings();
+    settings.insert(key, value);
+    QmlDesignerPlugin::instance()->setSettings(settings);
+}
+
 } // namespace QmlDesigner
