@@ -36,15 +36,17 @@
 #include "qmlprofilerrunconfigurationaspect.h"
 #include "qmlprofilersettings.h"
 #include "qmlprofilerplugin.h"
+#include "qmlprofilertextmark.h"
+
+#include <app/app_version.h>
 
 #include <debugger/debuggericons.h>
 #include <debugger/analyzer/analyzermanager.h>
-#include <debugger/analyzer/analyzerruncontrol.h>
-#include <debugger/analyzer/analyzerstartparameters.h>
 
 #include <utils/fancymainwindow.h>
 #include <utils/fileinprojectfinder.h>
 #include <utils/qtcassert.h>
+#include <utils/url.h>
 #include <utils/utilsicons.h>
 #include <projectexplorer/environmentaspect.h>
 #include <projectexplorer/projectexplorer.h>
@@ -53,10 +55,10 @@
 #include <projectexplorer/session.h>
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/runnables.h>
+#include <projectexplorer/taskhub.h>
 #include <texteditor/texteditor.h>
 
 #include <coreplugin/coreconstants.h>
-#include <coreplugin/coreicons.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/find/findplugin.h>
 #include <coreplugin/icore.h>
@@ -70,7 +72,10 @@
 
 #include <qtsupport/qtkitinformation.h>
 
+#include <utils/qtcfallthrough.h>
+
 #include <QApplication>
+#include <QDockWidget>
 #include <QFileDialog>
 #include <QHBoxLayout>
 #include <QLabel>
@@ -100,7 +105,6 @@ public:
     QmlProfilerModelManager *m_profilerModelManager = 0;
 
     QmlProfilerViewManager *m_viewContainer = 0;
-    Utils::FileInProjectFinder m_projectFinder;
     QToolButton *m_recordButton = 0;
     QMenu *m_recordFeaturesMenu = 0;
 
@@ -108,11 +112,6 @@ public:
     QAction *m_stopAction = 0;
     QToolButton *m_clearButton = 0;
 
-    // elapsed time display
-    QTimer m_recordingTimer;
-    QTime m_recordingElapsedTime;
-    QLabel *m_timeLabel = 0;
-
     // open search
     QToolButton *m_searchButton = 0;
 
@@ -124,6 +123,11 @@ public:
     QAction *m_saveQmlTrace = 0;
     QAction *m_loadQmlTrace = 0;
 
+    // elapsed time display
+    QLabel *m_timeLabel = 0;
+    QTimer m_recordingTimer;
+    QTime m_recordingElapsedTime;
+
     bool m_toolBusy = false;
 };
 
@@ -135,19 +139,17 @@ QmlProfilerTool::QmlProfilerTool(QObject *parent)
     d->m_profilerState = new QmlProfilerStateManager(this);
     connect(d->m_profilerState, &QmlProfilerStateManager::stateChanged,
             this, &QmlProfilerTool::profilerStateChanged);
-    connect(d->m_profilerState, &QmlProfilerStateManager::clientRecordingChanged,
-            this, &QmlProfilerTool::clientRecordingChanged);
     connect(d->m_profilerState, &QmlProfilerStateManager::serverRecordingChanged,
             this, &QmlProfilerTool::serverRecordingChanged);
     connect(d->m_profilerState, &QmlProfilerStateManager::recordedFeaturesChanged,
             this, &QmlProfilerTool::setRecordedFeatures);
 
     d->m_profilerConnections = new QmlProfilerClientManager(this);
-    d->m_profilerConnections->registerProfilerStateManager(d->m_profilerState);
+    d->m_profilerConnections->setProfilerStateManager(d->m_profilerState);
     connect(d->m_profilerConnections, &QmlProfilerClientManager::connectionClosed,
             this, &QmlProfilerTool::clientsDisconnected);
 
-    d->m_profilerModelManager = new QmlProfilerModelManager(&d->m_projectFinder, this);
+    d->m_profilerModelManager = new QmlProfilerModelManager(this);
     connect(d->m_profilerModelManager, &QmlProfilerModelManager::stateChanged,
             this, &QmlProfilerTool::profilerDataModelStateChanged);
     connect(d->m_profilerModelManager, &QmlProfilerModelManager::error,
@@ -163,20 +165,20 @@ QmlProfilerTool::QmlProfilerTool(QObject *parent)
     Command *command = 0;
 
     ActionContainer *menu = ActionManager::actionContainer(M_DEBUG_ANALYZER);
-    ActionContainer *options = ActionManager::createMenu(M_DEBUG_ANALYZER_QML_OPTIONS);
+    ActionContainer *options = ActionManager::createMenu("Analyzer.Menu.QMLOptions");
     options->menu()->setTitle(tr("QML Profiler Options"));
     menu->addMenu(options, G_ANALYZER_OPTIONS);
     options->menu()->setEnabled(true);
 
     QAction *act = d->m_loadQmlTrace = new QAction(tr("Load QML Trace"), options);
     command = ActionManager::registerAction(act, Constants::QmlProfilerLoadActionId);
-    connect(act, &QAction::triggered, this, &QmlProfilerTool::showLoadDialog);
+    connect(act, &QAction::triggered, this, &QmlProfilerTool::showLoadDialog, Qt::QueuedConnection);
     options->addAction(command);
 
     act = d->m_saveQmlTrace = new QAction(tr("Save QML Trace"), options);
     d->m_saveQmlTrace->setEnabled(false);
     command = ActionManager::registerAction(act, Constants::QmlProfilerSaveActionId);
-    connect(act, &QAction::triggered, this, &QmlProfilerTool::showSaveDialog);
+    connect(act, &QAction::triggered, this, &QmlProfilerTool::showSaveDialog, Qt::QueuedConnection);
     options->addAction(command);
 
     d->m_recordingTimer.setInterval(100);
@@ -202,10 +204,8 @@ QmlProfilerTool::QmlProfilerTool(QObject *parent)
     connect(d->m_recordFeaturesMenu, &QMenu::triggered,
             this, &QmlProfilerTool::toggleRequestedFeature);
 
-    setRecording(d->m_profilerState->clientRecording());
-
     d->m_clearButton = new QToolButton;
-    d->m_clearButton->setIcon(Core::Icons::CLEAN_TOOLBAR.icon());
+    d->m_clearButton->setIcon(Utils::Icons::CLEAN_TOOLBAR.icon());
     d->m_clearButton->setToolTip(tr("Discard data"));
 
     connect(d->m_clearButton, &QAbstractButton::clicked, [this](){
@@ -218,9 +218,10 @@ QmlProfilerTool::QmlProfilerTool(QObject *parent)
     d->m_searchButton->setToolTip(tr("Search timeline event notes."));
 
     connect(d->m_searchButton, &QToolButton::clicked, this, &QmlProfilerTool::showTimeLineSearch);
+    d->m_searchButton->setEnabled(d->m_viewContainer->traceView()->isUsable());
 
     d->m_displayFeaturesButton = new QToolButton;
-    d->m_displayFeaturesButton->setIcon(Core::Icons::FILTER.icon());
+    d->m_displayFeaturesButton->setIcon(Utils::Icons::FILTER.icon());
     d->m_displayFeaturesButton->setToolTip(tr("Hide or show event categories."));
     d->m_displayFeaturesButton->setPopupMode(QToolButton::InstantPopup);
     d->m_displayFeaturesButton->setProperty("noArrow", true);
@@ -235,17 +236,14 @@ QmlProfilerTool::QmlProfilerTool(QObject *parent)
     d->m_timeLabel->setPalette(palette);
     d->m_timeLabel->setIndent(10);
     updateTimeDisplay();
+    connect(d->m_timeLabel, &QObject::destroyed, &d->m_recordingTimer, &QTimer::stop);
 
     setAvailableFeatures(d->m_profilerModelManager->availableFeatures());
     setRecordedFeatures(0);
 
     // When the widgets are requested we assume that the session data
     // is available, then we can populate the file finder
-    populateFileFinder();
-
-    auto runControlCreator = [this](RunConfiguration *runConfiguration, Core::Id) {
-        return createRunControl(runConfiguration);
-    };
+    d->m_profilerModelManager->populateFileFinder();
 
     QString description = tr("The QML Profiler can be used to find performance "
                              "bottlenecks in applications using QML.");
@@ -253,25 +251,26 @@ QmlProfilerTool::QmlProfilerTool(QObject *parent)
     d->m_startAction = Debugger::createStartAction();
     d->m_stopAction = Debugger::createStopAction();
 
-    ActionDescription desc;
-    desc.setText(tr("QML Profiler"));
-    desc.setToolTip(description);
-    desc.setPerspectiveId(Constants::QmlProfilerPerspectiveId);
-    desc.setRunControlCreator(runControlCreator);
-    desc.setToolPreparer([this] { return prepareTool(); });
-    desc.setRunMode(ProjectExplorer::Constants::QML_PROFILER_RUN_MODE);
-    desc.setMenuGroup(Debugger::Constants::G_ANALYZER_TOOLS);
-    Debugger::registerAction(Constants::QmlProfilerLocalActionId, desc, d->m_startAction);
+    act = new QAction(tr("QML Profiler"), this);
+    act->setToolTip(description);
+    menu->addAction(ActionManager::registerAction(act, "QmlProfiler.Internal"),
+                    Debugger::Constants::G_ANALYZER_TOOLS);
+    QObject::connect(act, &QAction::triggered, this, [this] {
+         if (!prepareTool())
+             return;
+        Debugger::selectPerspective(Constants::QmlProfilerPerspectiveId);
+        ProjectExplorerPlugin::runStartupProject(ProjectExplorer::Constants::QML_PROFILER_RUN_MODE);
+    });
+    QObject::connect(d->m_startAction, &QAction::triggered, act, &QAction::triggered);
+    QObject::connect(d->m_startAction, &QAction::changed, act, [act, this] {
+        act->setEnabled(d->m_startAction->isEnabled());
+    });
 
-    desc.setText(tr("QML Profiler (External)"));
-    desc.setToolTip(description);
-    desc.setPerspectiveId(Constants::QmlProfilerPerspectiveId);
-    desc.setRunControlCreator(runControlCreator);
-    desc.setCustomToolStarter([this](RunConfiguration *rc) { startRemoteTool(rc); });
-    desc.setToolPreparer([this] { return prepareTool(); });
-    desc.setRunMode(ProjectExplorer::Constants::QML_PROFILER_RUN_MODE);
-    desc.setMenuGroup(Debugger::Constants::G_ANALYZER_REMOTE_TOOLS);
-    Debugger::registerAction(Constants::QmlProfilerRemoteActionId, desc);
+    act = new QAction(tr("QML Profiler (Attach to Waiting Application)"), this);
+    act->setToolTip(description);
+    menu->addAction(ActionManager::registerAction(act, "QmlProfiler.AttachToWaitingApplication"),
+                    Debugger::Constants::G_ANALYZER_REMOTE_TOOLS);
+    QObject::connect(act, &QAction::triggered, this, &QmlProfilerTool::attachToWaitingApplication);
 
     Utils::ToolbarDescription toolbar;
     toolbar.addAction(d->m_startAction);
@@ -285,6 +284,55 @@ QmlProfilerTool::QmlProfilerTool(QObject *parent)
 
     connect(ProjectExplorerPlugin::instance(), &ProjectExplorerPlugin::updateRunActions,
             this, &QmlProfilerTool::updateRunActions);
+
+    QmlProfilerTextMarkModel *model = d->m_profilerModelManager->textMarkModel();
+    if (EditorManager *editorManager = EditorManager::instance()) {
+        connect(editorManager, &EditorManager::editorCreated,
+                model, [this, model](Core::IEditor *editor, const QString &fileName) {
+            Q_UNUSED(editor);
+            model->createMarks(d->m_viewContainer, fileName);
+        });
+    }
+
+    auto updateRecordButton = [this]() {
+        const bool recording =
+                d->m_profilerState->currentState() != QmlProfilerStateManager::AppRunning
+                ? d->m_profilerState->clientRecording() : d->m_profilerState->serverRecording();
+
+        const static QIcon recordOn = Debugger::Icons::RECORD_ON.icon();
+        const static QIcon recordOff = Debugger::Icons::RECORD_OFF.icon();
+
+        // update display
+        d->m_recordButton->setToolTip(recording ? tr("Disable Profiling") : tr("Enable Profiling"));
+        d->m_recordButton->setIcon(recording ? recordOn : recordOff);
+        d->m_recordButton->setChecked(recording);
+
+        switch (d->m_profilerModelManager->state()) {
+        case QmlProfilerModelManager::Empty:
+        case QmlProfilerModelManager::AcquiringData:
+        case QmlProfilerModelManager::Done:
+            // Don't change the recording button if the application cannot react to it.
+            d->m_recordButton->setEnabled(d->m_profilerState->currentState()
+                                          != QmlProfilerStateManager::AppStopRequested
+                                          && d->m_profilerState->currentState()
+                                          != QmlProfilerStateManager::AppDying);
+            break;
+        case QmlProfilerModelManager::ProcessingData:
+        case QmlProfilerModelManager::ClearingData:
+            d->m_recordButton->setEnabled(false);
+            break;
+        }
+    };
+
+    connect(d->m_profilerState, &QmlProfilerStateManager::stateChanged,
+            d->m_recordButton, updateRecordButton);
+    connect(d->m_profilerState, &QmlProfilerStateManager::serverRecordingChanged,
+            d->m_recordButton, updateRecordButton);
+    connect(d->m_profilerState, &QmlProfilerStateManager::clientRecordingChanged,
+            d->m_recordButton, updateRecordButton);
+    connect(d->m_profilerModelManager, &QmlProfilerModelManager::stateChanged,
+            d->m_recordButton, updateRecordButton);
+    updateRecordButton();
 }
 
 QmlProfilerTool::~QmlProfilerTool()
@@ -308,101 +356,80 @@ void QmlProfilerTool::updateRunActions()
     }
 }
 
-static QString sysroot(RunConfiguration *runConfig)
-{
-    QTC_ASSERT(runConfig, return QString());
-    Kit *k = runConfig->target()->kit();
-    if (k && SysRootKitInformation::hasSysRoot(k))
-        return SysRootKitInformation::sysRoot(runConfig->target()->kit()).toString();
-    return QString();
-}
-
-AnalyzerRunControl *QmlProfilerTool::createRunControl(RunConfiguration *runConfiguration)
+void QmlProfilerTool::finalizeRunControl(QmlProfilerRunner *runWorker)
 {
     d->m_toolBusy = true;
+    auto runControl = runWorker->runControl();
+    auto runConfiguration = runControl->runConfiguration();
     if (runConfiguration) {
-        QmlProfilerRunConfigurationAspect *aspect = static_cast<QmlProfilerRunConfigurationAspect *>(
+        auto aspect = static_cast<QmlProfilerRunConfigurationAspect *>(
                     runConfiguration->extraAspect(Constants::SETTINGS));
         if (aspect) {
             if (QmlProfilerSettings *settings = static_cast<QmlProfilerSettings *>(aspect->currentSettings())) {
                 d->m_profilerConnections->setFlushInterval(settings->flushEnabled() ?
                                                                settings->flushInterval() : 0);
-                d->m_profilerConnections->setAggregateTraces(settings->aggregateTraces());
+                d->m_profilerModelManager->setAggregateTraces(settings->aggregateTraces());
             }
         }
     }
 
-    auto runControl = new QmlProfilerRunControl(runConfiguration, this);
-    connect(runControl, &RunControl::finished, this, [this, runControl] {
+    auto handleStop = [this, runControl]() {
         d->m_toolBusy = false;
         updateRunActions();
-        disconnect(d->m_stopAction, &QAction::triggered, runControl, &QmlProfilerRunControl::stop);
+        disconnect(d->m_stopAction, &QAction::triggered, runControl, &RunControl::initiateStop);
+    };
+
+    connect(runControl, &RunControl::stopped, this, handleStop);
+    connect(runControl, &RunControl::finished, this, [this, runControl, handleStop] {
+        if (d->m_toolBusy)
+            handleStop();
     });
 
-    connect(d->m_stopAction, &QAction::triggered, runControl, &QmlProfilerRunControl::stop);
+    connect(d->m_stopAction, &QAction::triggered, runControl, &RunControl::initiateStop);
 
     updateRunActions();
-    return runControl;
-}
-
-void QmlProfilerTool::finalizeRunControl(QmlProfilerRunControl *runControl)
-{
-    runControl->registerProfilerStateManager(d->m_profilerState);
-
-    QTC_ASSERT(runControl->connection().is<AnalyzerConnection>(), return);
-    // FIXME: Check that there's something sensible in sp.connParams
-    auto connection = runControl->connection().as<AnalyzerConnection>();
-    if (!connection.analyzerSocket.isEmpty())
-        d->m_profilerConnections->setLocalSocket(connection.analyzerSocket);
-    else
-        d->m_profilerConnections->setTcpConnection(connection.analyzerHost, connection.analyzerPort);
+    runWorker->registerProfilerStateManager(d->m_profilerState);
 
     //
     // Initialize m_projectFinder
     //
 
-    RunConfiguration *runConfiguration = runControl->runConfiguration();
-    if (runConfiguration) {
-        QString projectDirectory;
-        Project *project = runConfiguration->target()->project();
-        projectDirectory = project->projectDirectory().toString();
-        populateFileFinder(projectDirectory, sysroot(runConfiguration));
-    }
+    d->m_profilerModelManager->populateFileFinder(runConfiguration ? runConfiguration->target()
+                                                                   : nullptr);
 
-    if (connection.analyzerSocket.isEmpty())
-        connect(runControl, &QmlProfilerRunControl::processRunning,
-                d->m_profilerConnections, &QmlProfilerClientManager::connectTcpClient);
     connect(d->m_profilerConnections, &QmlProfilerClientManager::connectionFailed,
-            runControl, &QmlProfilerRunControl::cancelProcess);
-}
+            runWorker, [this, runWorker]() {
+        QMessageBox *infoBox = new QMessageBox(ICore::mainWindow());
+        infoBox->setIcon(QMessageBox::Critical);
+        infoBox->setWindowTitle(Core::Constants::IDE_DISPLAY_NAME);
+        infoBox->setText(QmlProfilerTool::tr("Could not connect to the in-process QML profiler.\n"
+                                             "Do you want to retry?"));
+        infoBox->setStandardButtons(QMessageBox::Retry | QMessageBox::Cancel | QMessageBox::Help);
+        infoBox->setDefaultButton(QMessageBox::Retry);
+        infoBox->setModal(true);
 
-void QmlProfilerTool::populateFileFinder(QString projectDirectory, QString activeSysroot)
-{
-    // Initialize filefinder with some sensible default
-    QStringList sourceFiles;
-    QList<Project *> projects = SessionManager::projects();
-    if (Project *startupProject = SessionManager::startupProject()) {
-        // startup project first
-        projects.removeOne(startupProject);
-        projects.insert(0, startupProject);
-    }
-    foreach (Project *project, projects)
-        sourceFiles << project->files(Project::SourceFiles);
+        connect(infoBox, &QDialog::finished, runWorker, [this, runWorker](int result) {
+            switch (result) {
+            case QMessageBox::Retry:
+                d->m_profilerConnections->retryConnect();
+                break;
+            case QMessageBox::Help:
+                HelpManager::handleHelpRequest(
+                            "qthelp://org.qt-project.qtcreator/doc/creator-debugging-qml.html");
+                Q_FALLTHROUGH();
+            case QMessageBox::Cancel:
+                // The actual error message has already been logged.
+                QmlProfilerTool::logState(QmlProfilerTool::tr("Failed to connect."));
+                runWorker->cancelProcess();
+                break;
+            }
+        });
 
-    if (!projects.isEmpty()) {
-        if (projectDirectory.isEmpty())
-            projectDirectory = projects.first()->projectDirectory().toString();
+        infoBox->show();
+    }, Qt::QueuedConnection); // Queue any connection failures after reportStarted()
 
-        if (activeSysroot.isEmpty()) {
-            if (Target *target = projects.first()->activeTarget())
-                if (RunConfiguration *rc = target->activeRunConfiguration())
-                    activeSysroot = sysroot(rc);
-        }
-    }
-
-    d->m_projectFinder.setProjectDirectory(projectDirectory);
-    d->m_projectFinder.setProjectFiles(sourceFiles);
-    d->m_projectFinder.setSysroot(activeSysroot);
+    d->m_profilerConnections->connectToServer(runWorker->serverUrl());
+    d->m_profilerState->setCurrentState(QmlProfilerStateManager::AppRunning);
 }
 
 void QmlProfilerTool::recordingButtonChanged(bool recording)
@@ -412,9 +439,9 @@ void QmlProfilerTool::recordingButtonChanged(bool recording)
 
     if (recording && d->m_profilerState->currentState() == QmlProfilerStateManager::AppRunning) {
         if (checkForUnsavedNotes()) {
-            if (!d->m_profilerConnections->aggregateTraces() ||
+            if (!d->m_profilerModelManager->aggregateTraces() ||
                     d->m_profilerModelManager->state() == QmlProfilerModelManager::Done)
-                clearData(); // clear before the recording starts, unless we aggregate recordings
+                clearEvents(); // clear before the recording starts, unless we aggregate recordings
             if (d->m_profilerState->clientRecording())
                 d->m_profilerState->setClientRecording(false);
             d->m_profilerState->setClientRecording(true);
@@ -428,34 +455,12 @@ void QmlProfilerTool::recordingButtonChanged(bool recording)
     }
 }
 
-void QmlProfilerTool::setRecording(bool recording)
-{
-    const static QIcon recordOn = Debugger::Icons::RECORD_ON.icon();
-    const static QIcon recordOff = Debugger::Icons::RECORD_OFF.icon();
-
-    // update display
-    d->m_recordButton->setToolTip( recording ? tr("Disable Profiling") : tr("Enable Profiling"));
-    d->m_recordButton->setIcon(recording ? recordOn : recordOff);
-
-    d->m_recordButton->setChecked(recording);
-
-    // manage timer
-    if (d->m_profilerState->currentState() == QmlProfilerStateManager::AppRunning) {
-        if (recording) {
-            d->m_recordingTimer.start();
-            d->m_recordingElapsedTime.start();
-        } else {
-            d->m_recordingTimer.stop();
-        }
-    }
-}
-
 void QmlProfilerTool::gotoSourceLocation(const QString &fileUrl, int lineNumber, int columnNumber)
 {
     if (lineNumber < 0 || fileUrl.isEmpty())
         return;
 
-    const QString projectFileName = d->m_projectFinder.findFile(fileUrl);
+    const QString projectFileName = d->m_profilerModelManager->findLocalFile(fileUrl);
 
     QFileInfo fileInfo(projectFileName);
     if (!fileInfo.exists() || !fileInfo.isReadable())
@@ -463,8 +468,9 @@ void QmlProfilerTool::gotoSourceLocation(const QString &fileUrl, int lineNumber,
 
     // The text editors count columns starting with 0, but the ASTs store the
     // location starting with 1, therefore the -1.
-    EditorManager::openEditorAt(projectFileName, lineNumber, columnNumber - 1, Id(),
-                                EditorManager::DoNotSwitchToDesignMode);
+    EditorManager::openEditorAt(
+                projectFileName, lineNumber == 0 ? 1 : lineNumber, columnNumber - 1, Id(),
+                EditorManager::DoNotSwitchToDesignMode | EditorManager::DoNotSwitchToEditMode);
 }
 
 void QmlProfilerTool::updateTimeDisplay()
@@ -478,7 +484,8 @@ void QmlProfilerTool::updateTimeDisplay()
         if (d->m_profilerState->serverRecording()) {
             seconds = d->m_recordingElapsedTime.elapsed() / 1000.0;
             break;
-        } // else fall through
+        }
+        Q_FALLTHROUGH();
     case QmlProfilerStateManager::Idle:
         if (d->m_profilerModelManager->state() != QmlProfilerModelManager::Empty &&
                d->m_profilerModelManager->state() != QmlProfilerModelManager::ClearingData)
@@ -492,10 +499,20 @@ void QmlProfilerTool::updateTimeDisplay()
 
 void QmlProfilerTool::showTimeLineSearch()
 {
-    d->m_viewContainer->raiseTimeline();
+    QmlProfilerTraceView *traceView = d->m_viewContainer->traceView();
+    QTC_ASSERT(qobject_cast<QDockWidget *>(traceView->parentWidget()), return);
+    traceView->parentWidget()->raise();
+    traceView->setFocus();
     Core::Find::openFindToolBar(Core::Find::FindForwardDirection);
 }
 
+void QmlProfilerTool::clearEvents()
+{
+    d->m_profilerModelManager->clearEvents();
+    d->m_profilerConnections->clearEvents();
+    setRecordedFeatures(0);
+}
+
 void QmlProfilerTool::clearData()
 {
     d->m_profilerModelManager->clear();
@@ -514,13 +531,25 @@ void QmlProfilerTool::setButtonsEnabled(bool enable)
 {
     d->m_clearButton->setEnabled(enable);
     d->m_displayFeaturesButton->setEnabled(enable);
-    d->m_searchButton->setEnabled(enable);
+    d->m_searchButton->setEnabled(d->m_viewContainer->traceView()->isUsable() && enable);
     d->m_recordFeaturesMenu->setEnabled(enable);
 }
 
+void QmlProfilerTool::createTextMarks()
+{
+    QmlProfilerTextMarkModel *model = d->m_profilerModelManager->textMarkModel();
+    foreach (IDocument *document, DocumentModel::openedDocuments())
+        model->createMarks(d->m_viewContainer, document->filePath().toString());
+}
+
+void QmlProfilerTool::clearTextMarks()
+{
+    d->m_profilerModelManager->textMarkModel()->clear();
+}
+
 bool QmlProfilerTool::prepareTool()
 {
-    if (d->m_recordButton->isChecked()) {
+    if (d->m_profilerState->clientRecording()) {
         if (checkForUnsavedNotes()) {
             clearData(); // clear right away to suppress second warning on server recording change
             return true;
@@ -531,17 +560,20 @@ bool QmlProfilerTool::prepareTool()
     return true;
 }
 
-void QmlProfilerTool::startRemoteTool(ProjectExplorer::RunConfiguration *rc)
+ProjectExplorer::RunControl *QmlProfilerTool::attachToWaitingApplication()
 {
+    if (!prepareTool())
+        return nullptr;
+
     Id kitId;
-    quint16 port;
+    int port;
     Kit *kit = 0;
 
     {
         QSettings *settings = ICore::settings();
 
         kitId = Id::fromSetting(settings->value(QLatin1String("AnalyzerQmlAttachDialog/kitId")));
-        port = settings->value(QLatin1String("AnalyzerQmlAttachDialog/port"), 3768).toUInt();
+        port = settings->value(QLatin1String("AnalyzerQmlAttachDialog/port"), 3768).toInt();
 
         QmlProfilerAttachDialog dialog;
 
@@ -549,30 +581,38 @@ void QmlProfilerTool::startRemoteTool(ProjectExplorer::RunConfiguration *rc)
         dialog.setPort(port);
 
         if (dialog.exec() != QDialog::Accepted)
-            return;
+            return nullptr;
 
         kit = dialog.kit();
         port = dialog.port();
+        QTC_ASSERT(port >= 0, return nullptr);
+        QTC_ASSERT(port <= std::numeric_limits<quint16>::max(), return nullptr);
 
         settings->setValue(QLatin1String("AnalyzerQmlAttachDialog/kitId"), kit->id().toSetting());
         settings->setValue(QLatin1String("AnalyzerQmlAttachDialog/port"), port);
     }
 
-    AnalyzerConnection connection;
+    QUrl serverUrl;
 
     IDevice::ConstPtr device = DeviceKitInformation::device(kit);
-    if (device) {
-        Connection toolControl = device->toolControlChannel(IDevice::QmlControlChannel);
-        QTC_ASSERT(toolControl.is<HostName>(), return);
-        connection.analyzerHost = toolControl.as<HostName>().host();
-        connection.connParams = device->sshParameters();
-    }
-    connection.analyzerPort = Utils::Port(port);
+    QTC_ASSERT(device, return nullptr);
+    QUrl toolControl = device->toolControlChannel(IDevice::QmlControlChannel);
+    serverUrl.setScheme(Utils::urlTcpScheme());
+    serverUrl.setHost(toolControl.host());
+    serverUrl.setPort(port);
 
-    auto runControl = qobject_cast<QmlProfilerRunControl *>(createRunControl(rc));
-    runControl->setConnection(connection);
+    Debugger::selectPerspective(Constants::QmlProfilerPerspectiveId);
 
-    ProjectExplorerPlugin::startRunControl(runControl, ProjectExplorer::Constants::QML_PROFILER_RUN_MODE);
+    auto runConfig = RunConfiguration::startupRunConfiguration();
+    auto runControl = new RunControl(runConfig, ProjectExplorer::Constants::QML_PROFILER_RUN_MODE);
+    auto profiler = new QmlProfilerRunner(runControl);
+    profiler->setServerUrl(serverUrl);
+    connect(profiler, &QmlProfilerRunner::starting, this, &QmlProfilerTool::finalizeRunControl);
+
+    connect(d->m_profilerConnections, &QmlProfilerClientManager::connectionClosed,
+            runControl, &RunControl::initiateStop);
+    ProjectExplorerPlugin::startRunControl(runControl);
+    return runControl;
 }
 
 void QmlProfilerTool::logState(const QString &msg)
@@ -652,7 +692,7 @@ void QmlProfilerTool::showLoadDialog()
         Debugger::enableMainWindow(false);
         connect(d->m_profilerModelManager, &QmlProfilerModelManager::recordedFeaturesChanged,
                 this, &QmlProfilerTool::setRecordedFeatures);
-        populateFileFinder();
+        d->m_profilerModelManager->populateFileFinder();
         d->m_profilerModelManager->load(filename);
     }
 }
@@ -695,7 +735,7 @@ void QmlProfilerTool::restoreFeatureVisibility()
 void QmlProfilerTool::clientsDisconnected()
 {
     if (d->m_profilerModelManager->state() == QmlProfilerModelManager::AcquiringData) {
-        if (d->m_profilerConnections->aggregateTraces()) {
+        if (d->m_profilerModelManager->aggregateTraces()) {
             d->m_profilerModelManager->acquiringDone();
         } else {
             // If the application stopped by itself, check if we have all the data
@@ -762,30 +802,25 @@ void QmlProfilerTool::profilerDataModelStateChanged()
 {
     switch (d->m_profilerModelManager->state()) {
     case QmlProfilerModelManager::Empty :
-        d->m_recordButton->setEnabled(true);
         setButtonsEnabled(true);
         break;
     case QmlProfilerModelManager::ClearingData :
-        d->m_recordButton->setEnabled(false);
+        clearTextMarks();
         setButtonsEnabled(false);
         clearDisplay();
         break;
     case QmlProfilerModelManager::AcquiringData :
         restoreFeatureVisibility();
-        d->m_recordButton->setEnabled(true); // Press recording button to stop recording
         setButtonsEnabled(false);            // Other buttons disabled
         break;
     case QmlProfilerModelManager::ProcessingData :
-        d->m_recordButton->setEnabled(false);
         setButtonsEnabled(false);
         break;
     case QmlProfilerModelManager::Done :
         showSaveOption();
         updateTimeDisplay();
-        d->m_recordButton->setEnabled(true);
         setButtonsEnabled(true);
-    break;
-    default:
+        createTextMarks();
         break;
     }
 }
@@ -817,14 +852,19 @@ void QmlProfilerTool::showNonmodalWarning(const QString &warningMsg)
     noExecWarning->show();
 }
 
-QMessageBox *QmlProfilerTool::requestMessageBox()
+QmlProfilerClientManager *QmlProfilerTool::clientManager()
 {
-    return new QMessageBox(ICore::mainWindow());
+    return d->m_profilerConnections;
 }
 
-void QmlProfilerTool::handleHelpRequest(const QString &link)
+QmlProfilerModelManager *QmlProfilerTool::modelManager()
 {
-    HelpManager::handleHelpRequest(link);
+    return d->m_profilerModelManager;
+}
+
+QmlProfilerStateManager *QmlProfilerTool::stateManager()
+{
+    return d->m_profilerState;
 }
 
 void QmlProfilerTool::profilerStateChanged()
@@ -837,13 +877,16 @@ void QmlProfilerTool::profilerStateChanged()
         break;
     }
     case QmlProfilerStateManager::Idle :
-        // when the app finishes, set recording display to client status
-        setRecording(d->m_profilerState->clientRecording());
         break;
     case QmlProfilerStateManager::AppStopRequested:
         // Don't allow toggling the recording while data is loaded when application quits
-        if (d->m_profilerState->serverRecording())
-            d->m_recordButton->setEnabled(false);
+        if (d->m_profilerState->serverRecording()) {
+            // Turn off recording and wait for remaining data
+            d->m_profilerConnections->stopRecording();
+        } else {
+            // Directly transition to idle
+            d->m_profilerState->setCurrentState(QmlProfilerStateManager::Idle);
+        }
         break;
     default:
         // no special action needed for other states
@@ -851,14 +894,6 @@ void QmlProfilerTool::profilerStateChanged()
     }
 }
 
-void QmlProfilerTool::clientRecordingChanged()
-{
-    // if application is running, display server record changes
-    // if application is stopped, display client record changes
-    if (d->m_profilerState->currentState() != QmlProfilerStateManager::AppRunning)
-        setRecording(d->m_profilerState->clientRecording());
-}
-
 void QmlProfilerTool::serverRecordingChanged()
 {
     showLoadOption();
@@ -876,18 +911,20 @@ void QmlProfilerTool::serverRecordingChanged()
                     QMessageBox::Save)
                 showSaveDialog();
 
-            setRecording(true);
-            if (!d->m_profilerConnections->aggregateTraces() ||
+            d->m_recordingTimer.start();
+            d->m_recordingElapsedTime.start();
+            if (!d->m_profilerModelManager->aggregateTraces() ||
                     d->m_profilerModelManager->state() == QmlProfilerModelManager::Done)
-                clearData();
+                clearEvents();
             d->m_profilerModelManager->startAcquiring();
         } else {
-            setRecording(false);
-
-            // changes back once loading is finished, see profilerDataModelStateChanged()
-            if (!d->m_profilerConnections->aggregateTraces())
-                d->m_recordButton->setEnabled(false);
+            d->m_recordingTimer.stop();
+            if (!d->m_profilerModelManager->aggregateTraces())
+                d->m_profilerModelManager->acquiringDone();
         }
+    } else if (d->m_profilerState->currentState() == QmlProfilerStateManager::AppStopRequested) {
+        d->m_profilerModelManager->acquiringDone();
+        d->m_profilerState->setCurrentState(QmlProfilerStateManager::Idle);
     }
 }
 
