@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,22 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -35,7 +30,9 @@
 
 #include <projectexplorer/buildtargetinfo.h>
 #include <projectexplorer/deploymentdata.h>
+#include <projectexplorer/kitinformation.h>
 #include <projectexplorer/project.h>
+#include <projectexplorer/runnables.h>
 #include <projectexplorer/target.h>
 #include <qtsupport/qtoutputformatter.h>
 #include <utils/qtcprocess.h>
@@ -54,26 +51,12 @@ const char WorkingDirectoryKey[] = "RemoteLinux.RunConfig.WorkingDirectory";
 
 } // anonymous namespace
 
-class RemoteLinuxRunConfigurationPrivate {
+class RemoteLinuxRunConfigurationPrivate
+{
 public:
-    RemoteLinuxRunConfigurationPrivate(const QString &targetName)
-        : targetName(targetName),
-          useAlternateRemoteExecutable(false)
-    {
-    }
-
-    RemoteLinuxRunConfigurationPrivate(const RemoteLinuxRunConfigurationPrivate *other)
-        : targetName(other->targetName),
-          arguments(other->arguments),
-          useAlternateRemoteExecutable(other->useAlternateRemoteExecutable),
-          alternateRemoteExecutable(other->alternateRemoteExecutable),
-          workingDirectory(other->workingDirectory)
-    {
-    }
-
     QString targetName;
-    QStringList arguments;
-    bool useAlternateRemoteExecutable;
+    QString arguments;
+    bool useAlternateRemoteExecutable = false;
     QString alternateRemoteExecutable;
     QString workingDirectory;
 };
@@ -82,20 +65,28 @@ public:
 
 using namespace Internal;
 
-RemoteLinuxRunConfiguration::RemoteLinuxRunConfiguration(Target *parent, Core::Id id,
-        const QString &targetName)
-    : AbstractRemoteLinuxRunConfiguration(parent, id),
-      d(new RemoteLinuxRunConfigurationPrivate(targetName))
+RemoteLinuxRunConfiguration::RemoteLinuxRunConfiguration(Target *target)
+    : RemoteLinuxRunConfiguration(target, IdPrefix)
 {
-    init();
 }
 
-RemoteLinuxRunConfiguration::RemoteLinuxRunConfiguration(ProjectExplorer::Target *parent,
-        RemoteLinuxRunConfiguration *source)
-    : AbstractRemoteLinuxRunConfiguration(parent, source),
-      d(new RemoteLinuxRunConfigurationPrivate(source->d))
+RemoteLinuxRunConfiguration::RemoteLinuxRunConfiguration(Target *target, Core::Id id)
+    : RunConfiguration(target, id), d(new RemoteLinuxRunConfigurationPrivate)
 {
-    init();
+    addExtraAspect(new RemoteLinuxEnvironmentAspect(this));
+
+    connect(target, &Target::deploymentDataChanged,
+            this, &RemoteLinuxRunConfiguration::handleBuildSystemDataUpdated);
+    connect(target, &Target::applicationTargetsChanged,
+            this, &RemoteLinuxRunConfiguration::handleBuildSystemDataUpdated);
+    // Handles device changes, etc.
+    connect(target, &Target::kitChanged,
+            this, &RemoteLinuxRunConfiguration::handleBuildSystemDataUpdated);
+}
+
+QString RemoteLinuxRunConfiguration::extraId() const
+{
+    return d->targetName;
 }
 
 RemoteLinuxRunConfiguration::~RemoteLinuxRunConfiguration()
@@ -103,23 +94,6 @@ RemoteLinuxRunConfiguration::~RemoteLinuxRunConfiguration()
     delete d;
 }
 
-void RemoteLinuxRunConfiguration::init()
-{
-    setDefaultDisplayName(defaultDisplayName());
-
-    addExtraAspect(new RemoteLinuxEnvironmentAspect(this));
-
-    connect(target(), SIGNAL(deploymentDataChanged()), SLOT(handleBuildSystemDataUpdated()));
-    connect(target(), SIGNAL(applicationTargetsChanged()), SLOT(handleBuildSystemDataUpdated()));
-    connect(target(), SIGNAL(kitChanged()),
-            this, SLOT(handleBuildSystemDataUpdated())); // Handles device changes, etc.
-}
-
-bool RemoteLinuxRunConfiguration::isEnabled() const
-{
-    return true;
-}
-
 QWidget *RemoteLinuxRunConfiguration::createConfigurationWidget()
 {
     return new RemoteLinuxRunConfigurationWidget(this);
@@ -130,10 +104,19 @@ OutputFormatter *RemoteLinuxRunConfiguration::createOutputFormatter() const
     return new QtSupport::QtOutputFormatter(target()->project());
 }
 
+Runnable RemoteLinuxRunConfiguration::runnable() const
+{
+    StandardRunnable r;
+    r.environment = extraAspect<RemoteLinuxEnvironmentAspect>()->environment();
+    r.executable = remoteExecutableFilePath();
+    r.commandLineArguments = arguments();
+    r.workingDirectory = workingDirectory();
+    return r;
+}
 
 QVariantMap RemoteLinuxRunConfiguration::toMap() const
 {
-    QVariantMap map(RunConfiguration::toMap());
+    QVariantMap map = RunConfiguration::toMap();
     map.insert(QLatin1String(ArgumentsKey), d->arguments);
     map.insert(QLatin1String(TargetNameKey), d->targetName);
     map.insert(QLatin1String(UseAlternateExeKey), d->useAlternateRemoteExecutable);
@@ -142,19 +125,33 @@ QVariantMap RemoteLinuxRunConfiguration::toMap() const
     return map;
 }
 
+QString RemoteLinuxRunConfiguration::buildSystemTarget() const
+{
+    return d->targetName;
+}
+
 bool RemoteLinuxRunConfiguration::fromMap(const QVariantMap &map)
 {
     if (!RunConfiguration::fromMap(map))
         return false;
 
-    d->arguments = map.value(QLatin1String(ArgumentsKey)).toStringList();
+    QVariant args = map.value(QLatin1String(ArgumentsKey));
+    if (args.type() == QVariant::StringList) // Until 3.7 a QStringList was stored.
+        d->arguments = QtcProcess::joinArgs(args.toStringList(), OsTypeLinux);
+    else
+        d->arguments = args.toString();
     d->targetName = map.value(QLatin1String(TargetNameKey)).toString();
     d->useAlternateRemoteExecutable = map.value(QLatin1String(UseAlternateExeKey), false).toBool();
     d->alternateRemoteExecutable = map.value(QLatin1String(AlternateExeKey)).toString();
     d->workingDirectory = map.value(QLatin1String(WorkingDirectoryKey)).toString();
 
-    setDefaultDisplayName(defaultDisplayName());
+    // Hack for old-style mangled ids. FIXME: Remove.
+    if (d->targetName.isEmpty()) {
+        QString extra = ProjectExplorer::idFromMap(map).suffixAfter(id());
+        d->targetName = extra;
+    }
 
+    setDefaultDisplayName(defaultDisplayName());
     return true;
 }
 
@@ -167,18 +164,11 @@ QString RemoteLinuxRunConfiguration::defaultDisplayName()
     return tr("Run on Remote Device");
 }
 
-QStringList RemoteLinuxRunConfiguration::arguments() const
+QString RemoteLinuxRunConfiguration::arguments() const
 {
     return d->arguments;
 }
 
-Environment RemoteLinuxRunConfiguration::environment() const
-{
-    RemoteLinuxEnvironmentAspect *aspect = extraAspect<RemoteLinuxEnvironmentAspect>();
-    QTC_ASSERT(aspect, return Environment());
-    return aspect->environment();
-}
-
 QString RemoteLinuxRunConfiguration::localExecutableFilePath() const
 {
     return target()->applicationTargets().targetFilePath(d->targetName).toString();
@@ -198,7 +188,7 @@ QString RemoteLinuxRunConfiguration::remoteExecutableFilePath() const
 
 void RemoteLinuxRunConfiguration::setArguments(const QString &args)
 {
-    d->arguments = QtcProcess::splitArgs(args, OsTypeLinux); // TODO: Widget should be list-based.
+    d->arguments = args;
 }
 
 QString RemoteLinuxRunConfiguration::workingDirectory() const
@@ -235,7 +225,7 @@ void RemoteLinuxRunConfiguration::handleBuildSystemDataUpdated()
 {
     emit deploySpecsChanged();
     emit targetInformationChanged();
-    updateEnabledState();
+    emit enabledChanged();
 }
 
 const char *RemoteLinuxRunConfiguration::IdPrefix = "RemoteLinuxRunConfiguration:";
