@@ -25,7 +25,6 @@
 
 #include "textdocument.h"
 
-#include "convenience.h"
 #include "extraencodingsettings.h"
 #include "fontsettings.h"
 #include "indenter.h"
@@ -37,11 +36,15 @@
 #include "texteditorconstants.h"
 #include "typingsettings.h"
 #include <texteditor/generichighlighter/highlighter.h>
+#include <coreplugin/diffservice.h>
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/documentmodel.h>
+#include <extensionsystem/pluginmanager.h>
+#include <utils/textutils.h>
 #include <utils/guard.h>
 #include <utils/mimetypes/mimedatabase.h>
 
+#include <QAction>
 #include <QApplication>
 #include <QDir>
 #include <QFileInfo>
@@ -128,7 +131,7 @@ QTextCursor TextDocumentPrivate::indentOrUnindent(const QTextCursor &textCursor,
     bool modified = true;
 
     QTextBlock startBlock = m_document.findBlock(start);
-    QTextBlock endBlock = m_document.findBlock(blockSelection ? end : end - 1).next();
+    QTextBlock endBlock = m_document.findBlock(blockSelection ? end : qMax(end - 1, 0)).next();
     const bool cursorAtBlockStart = (textCursor.position() == startBlock.position());
     const bool anchorAtBlockStart = (textCursor.anchor() == startBlock.position());
     const bool oneLinePartial = (startBlock.next() == endBlock)
@@ -291,6 +294,11 @@ QMap<QString, QTextCodec *> TextDocument::openedTextDocumentEncodings()
     return workingCopy;
 }
 
+TextDocument *TextDocument::currentTextDocument()
+{
+    return qobject_cast<TextDocument *>(EditorManager::currentDocument());
+}
+
 QString TextDocument::plainText() const
 {
     return document()->toPlainText();
@@ -298,7 +306,7 @@ QString TextDocument::plainText() const
 
 QString TextDocument::textAt(int pos, int length) const
 {
-    return Convenience::textAt(QTextCursor(document()), pos, length);
+    return Utils::Text::textAt(QTextCursor(document()), pos, length);
 }
 
 QChar TextDocument::characterAt(int pos) const
@@ -352,6 +360,22 @@ void TextDocument::setFontSettings(const FontSettings &fontSettings)
     emit fontSettingsChanged();
 }
 
+QAction *TextDocument::createDiffAgainstCurrentFileAction(
+    QObject *parent, const std::function<Utils::FileName()> &filePath)
+{
+    const auto diffAgainstCurrentFile = [filePath]() {
+        auto diffService = DiffService::instance();
+        auto textDocument = TextEditor::TextDocument::currentTextDocument();
+        const QString leftFilePath = textDocument ? textDocument->filePath().toString() : QString();
+        const QString rightFilePath = filePath().toString();
+        if (diffService && !leftFilePath.isEmpty() && !rightFilePath.isEmpty())
+            diffService->diffFiles(leftFilePath, rightFilePath);
+    };
+    auto diffAction = new QAction(tr("Diff Against Current File"), parent);
+    QObject::connect(diffAction, &QAction::triggered, parent, diffAgainstCurrentFile);
+    return diffAction;
+}
+
 void TextDocument::triggerPendingUpdates()
 {
     if (d->m_fontSettingsNeedsApply)
@@ -368,9 +392,9 @@ CompletionAssistProvider *TextDocument::completionAssistProvider() const
     return d->m_completionAssistProvider;
 }
 
-QuickFixAssistProvider *TextDocument::quickFixAssistProvider() const
+IAssistProvider *TextDocument::quickFixAssistProvider() const
 {
-    return 0;
+    return nullptr;
 }
 
 void TextDocument::applyFontSettings()
@@ -614,8 +638,7 @@ Core::IDocument::OpenResult TextDocument::open(QString *errorString, const QStri
     emit aboutToOpen(fileName, realFileName);
     OpenResult success = openImpl(errorString, fileName, realFileName, /*reload =*/ false);
     if (success == OpenResult::Success) {
-        Utils::MimeDatabase mdb;
-        setMimeType(mdb.mimeTypeForFile(fileName).name());
+        setMimeType(Utils::mimeTypeForFile(fileName).name());
         emit openFinishedSuccessfully();
     }
     return success;
@@ -841,6 +864,13 @@ void TextDocument::modificationChanged(bool modified)
     emit changed();
 }
 
+void TextDocument::updateLayout() const
+{
+    auto documentLayout = qobject_cast<TextDocumentLayout*>(d->m_document.documentLayout());
+    QTC_ASSERT(documentLayout, return);
+    documentLayout->requestUpdate();
+}
+
 TextMarks TextDocument::marks() const
 {
     return d->m_marksCache;
@@ -948,15 +978,21 @@ void TextDocument::removeMark(TextMark *mark)
     }
 
     removeMarkFromMarksCache(mark);
+    emit markRemoved(mark);
     mark->setBaseTextDocument(0);
+    updateLayout();
 }
 
 void TextDocument::updateMark(TextMark *mark)
 {
-    Q_UNUSED(mark)
-    auto documentLayout = qobject_cast<TextDocumentLayout*>(d->m_document.documentLayout());
-    QTC_ASSERT(documentLayout, return);
-    documentLayout->requestUpdate();
+    QTextBlock block = d->m_document.findBlockByNumber(mark->lineNumber() - 1);
+    if (block.isValid()) {
+        TextBlockUserData *userData = TextDocumentLayout::userData(block);
+        // re-evaluate priority
+        userData->removeMark(mark);
+        userData->addMark(mark);
+    }
+    updateLayout();
 }
 
 void TextDocument::moveMark(TextMark *mark, int previousLine)
