@@ -40,13 +40,14 @@
 
 #include "projectdata_p.h"
 #include "propertymap_p.h"
+#include <language/language.h>
 #include <language/propertymapinternal.h>
 #include <tools/fileinfo.h>
 #include <tools/jsliterals.h>
-#include <tools/propertyfinder.h>
 #include <tools/qbsassert.h>
+#include <tools/qttools.h>
 
-#include <QDir>
+#include <QtCore/qdir.h>
 
 #include <algorithm>
 
@@ -169,8 +170,8 @@ QStringList GroupData::allFilePaths() const
 {
     const QList<ArtifactData> &artifacts = allSourceArtifacts();
     QStringList paths;
-    paths.reserve(artifacts.count());
-    std::transform(artifacts.constBegin(), artifacts.constEnd(), std::back_inserter(paths),
+    paths.reserve(artifacts.size());
+    std::transform(artifacts.cbegin(), artifacts.cend(), std::back_inserter(paths),
                           [](const ArtifactData &sa) { return sa.filePath(); });
     return paths;
 }
@@ -260,8 +261,10 @@ bool ArtifactData::isGenerated() const
  */
 bool ArtifactData::isExecutable() const
 {
-    return d->fileTags.contains(QLatin1String("application"))
-            || d->fileTags.contains(QLatin1String("msi"));
+    const bool isBundle = d->properties.getModuleProperty(
+                QStringLiteral("bundle"), QStringLiteral("isBundle")).toBool();
+    return Internal::isRunnableArtifact(
+                Internal::FileTags::fromStringList(d->fileTags), isBundle);
 }
 
 /*!
@@ -450,6 +453,16 @@ QString ProductData::name() const
     return d->name;
 }
 
+/*!
+  The name of the product as given in the qbs source file, plus information
+  about which properties it was multiplexed on and the values of these properties.
+  If the product was not multiplexed, the returned value is the same as \c name().
+ */
+QString ProductData::fullDisplayName() const
+{
+    return Internal::ResolvedProduct::fullDisplayName(name(), multiplexConfigurationId());
+}
+
 /*!
  * \brief The base name of the product's target file as given in the qbs source file.
  */
@@ -474,6 +487,11 @@ QString ProductData::profile() const
     return d->profile;
 }
 
+QString ProductData::multiplexConfigurationId() const
+{
+   return d->multiplexConfigurationId;
+}
+
 /*!
  * \brief The location at which the product is defined in the source file.
  */
@@ -505,7 +523,7 @@ QList<ArtifactData> ProductData::generatedArtifacts() const
 QList<ArtifactData> ProductData::targetArtifacts() const
 {
     QList<ArtifactData> list;
-    std::copy_if(d->generatedArtifacts.constBegin(), d->generatedArtifacts.constEnd(),
+    std::copy_if(d->generatedArtifacts.cbegin(), d->generatedArtifacts.cend(),
                  std::back_inserter(list),
                  [](const ArtifactData &a) { return a.isTargetArtifact(); });
     return list;
@@ -517,13 +535,13 @@ QList<ArtifactData> ProductData::targetArtifacts() const
 QList<ArtifactData> ProductData::installableArtifacts() const
 {
     QList<ArtifactData> artifacts;
-    foreach (const GroupData &g, groups()) {
-        foreach (const ArtifactData &a, g.allSourceArtifacts()) {
+    for (const GroupData &g : groups()) {
+        for (const ArtifactData &a : g.allSourceArtifacts()) {
             if (a.installData().isInstallable())
                 artifacts << a;
         }
     }
-    foreach (const ArtifactData &a, targetArtifacts()) {
+    for (const ArtifactData &a : targetArtifacts()) {
         if (a.installData().isInstallable())
             artifacts << a;
     }
@@ -537,7 +555,17 @@ QList<ArtifactData> ProductData::installableArtifacts() const
 QString ProductData::targetExecutable() const
 {
     QBS_ASSERT(isValid(), return QString());
-    foreach (const ArtifactData &ta, targetArtifacts()) {
+    if (d->moduleProperties.getModuleProperty(QLatin1String("bundle"),
+                                              QLatin1String("isBundle")).toBool()) {
+        for (const ArtifactData &ta : targetArtifacts()) {
+            if (ta.fileTags().contains(QLatin1String("bundle.application-executable"))) {
+                if (ta.installData().isInstallable())
+                    return ta.installData().localInstallFilePath();
+                return ta.filePath();
+            }
+        }
+    }
+    for (const ArtifactData &ta : targetArtifacts()) {
         if (ta.isExecutable()) {
             if (ta.installData().isInstallable())
                 return ta.installData().localInstallFilePath();
@@ -589,6 +617,12 @@ bool ProductData::isRunnable() const
     return d->isRunnable;
 }
 
+bool ProductData::isMultiplexed() const
+{
+    QBS_ASSERT(isValid(), return false);
+    return d->isMultiplexed;
+}
+
 bool operator==(const ProductData &lhs, const ProductData &rhs)
 {
     if (!lhs.isValid() && !rhs.isValid())
@@ -601,12 +635,14 @@ bool operator==(const ProductData &lhs, const ProductData &rhs)
             && lhs.version() == rhs.version()
             && lhs.dependencies() == rhs.dependencies()
             && lhs.profile() == rhs.profile()
+            && lhs.multiplexConfigurationId() == rhs.multiplexConfigurationId()
             && lhs.location() == rhs.location()
             && lhs.groups() == rhs.groups()
             && lhs.generatedArtifacts() == rhs.generatedArtifacts()
             && lhs.properties() == rhs.properties()
             && lhs.moduleProperties() == rhs.moduleProperties()
-            && lhs.isEnabled() == rhs.isEnabled();
+            && lhs.isEnabled() == rhs.isEnabled()
+            && lhs.isMultiplexed() == rhs.isMultiplexed();
 }
 
 bool operator!=(const ProductData &lhs, const ProductData &rhs)
@@ -621,7 +657,8 @@ bool operator<(const ProductData &lhs, const ProductData &rhs)
         return true;
     if (nameCmp > 0)
         return false;
-    return lhs.profile() < rhs.profile();
+    return lhs.profile() < rhs.profile()
+            && lhs.multiplexConfigurationId() < rhs.multiplexConfigurationId();
 }
 
 /*!
@@ -718,7 +755,7 @@ QList<ProjectData> ProjectData::subProjects() const
 QList<ProductData> ProjectData::allProducts() const
 {
     QList<ProductData> productList = products();
-    foreach (const ProjectData &pd, subProjects())
+    for (const ProjectData &pd : subProjects())
         productList << pd.allProducts();
     return productList;
 }
@@ -729,7 +766,7 @@ QList<ProductData> ProjectData::allProducts() const
 QList<ArtifactData> ProjectData::installableArtifacts() const
 {
     QList<ArtifactData> artifacts;
-    foreach (const ProductData &p, allProducts())
+    for (const ProductData &p : allProducts())
         artifacts << p.installableArtifacts();
     return artifacts;
 }
@@ -818,10 +855,9 @@ QVariant PropertyMap::getProperty(const QString &name) const
 QStringList PropertyMap::getModulePropertiesAsStringList(const QString &moduleName,
                                                           const QString &propertyName) const
 {
-    const QVariantList &vl = Internal::PropertyFinder().propertyValue(d->m_map->value(), moduleName,
-                                                                      propertyName).toList();
+    const QVariantList &vl = d->m_map->moduleProperty(moduleName, propertyName).toList();
     QStringList sl;
-    foreach (const QVariant &v, vl) {
+    for (const QVariant &v : vl) {
         QBS_ASSERT(v.canConvert<QString>(), continue);
         sl << v.toString();
     }
@@ -834,7 +870,7 @@ QStringList PropertyMap::getModulePropertiesAsStringList(const QString &moduleNa
 QVariant PropertyMap::getModuleProperty(const QString &moduleName,
                                         const QString &propertyName) const
 {
-    return Internal::PropertyFinder().propertyValue(d->m_map->value(), moduleName, propertyName);
+    return d->m_map->moduleProperty(moduleName, propertyName);
 }
 
 static QString mapToString(const QVariantMap &map, const QString &prefix)
@@ -842,7 +878,7 @@ static QString mapToString(const QVariantMap &map, const QString &prefix)
     QStringList keys(map.keys());
     qSort(keys);
     QString stringRep;
-    foreach (const QString &key, keys) {
+    for (const QString &key : qAsConst(keys)) {
         const QVariant &val = map.value(key);
         if (val.type() == QVariant::Map) {
             stringRep += mapToString(val.value<QVariantMap>(), prefix + key + QLatin1Char('.'));
@@ -861,12 +897,12 @@ QString PropertyMap::toString() const
 
 bool operator==(const PropertyMap &pm1, const PropertyMap &pm2)
 {
-    return pm1.d->m_map->value() == pm2.d->m_map->value();
+    return *pm1.d->m_map == *pm2.d->m_map;
 }
 
 bool operator!=(const PropertyMap &pm1, const PropertyMap &pm2)
 {
-    return !(pm1.d->m_map->value() == pm2.d->m_map->value());
+    return !(*pm1.d->m_map == *pm2.d->m_map);
 }
 
 } // namespace qbs
