@@ -35,7 +35,9 @@ def main():
     analyzerTargets = Targets.desktopTargetClasses()
     checkedTargets, projectName = createNewQtQuickApplication(workingDir, targets=analyzerTargets)
     editor = waitForObject(":Qt Creator_QmlJSEditor::QmlJSTextEditorWidget")
-    if placeCursorToLine(editor, "MouseArea.*", True):
+    if placeCursorToLine(editor, "}"):
+        type(editor, '<Left>')
+        type(editor, '<Return>')
         type(editor, '<Up>')
         type(editor, '<Return>')
         typeLines(editor, ['Timer {',
@@ -105,13 +107,13 @@ def performTest(workingDir, projectName, targetCount, availableConfigs):
             (colPercent, colTotal, colSelfPercent, colSelf, colCalls,
              colMean, colMedian, colLongest, colShortest) = range(2, 11)
             model = waitForObject(":Events.QmlProfilerEventsTable_QmlProfiler::"
-                                  "Internal::QmlProfilerEventsMainView").model()
+                                  "Internal::QmlProfilerStatisticsMainView").model()
             compareEventsTab(model, "events_qt5.tsv")
             test.compare(dumpItems(model, column=colPercent)[0], '100.00 %')
             # cannot run following test on colShortest (unstable)
             for i in [colTotal, colMean, colMedian, colLongest]:
-                for item in dumpItems(model, column=i)[1:5]:
-                    test.verify(item.endswith(' ms'), "Verify that '%s' ends with ' ms'" % item)
+                for item in dumpItems(model, column=i)[2:5]:
+                    test.verify(item.endswith('ms'), "Verify that '%s' ends with 'ms'" % item)
             for i in [colTotal, colMean, colMedian, colLongest, colShortest]:
                 for item in dumpItems(model, column=i):
                     test.verify(not item.startswith('0.000 '),
@@ -158,9 +160,8 @@ def containsOnce(tuple, items):
     return True
 
 def safeClickTab(tab):
-    for bar in [":*Qt Creator.JavaScript_QTabBar",
-                ":*Qt Creator.Events_QTabBar",
-                ":*Qt Creator.Timeline_QTabBar"]:
+    for bar in [":Qt Creator.Events_QTabBar",
+                ":Qt Creator.Timeline_QTabBar"]:
         try:
             clickOnTab(bar, tab, 1000)
             return True
@@ -168,9 +169,3 @@ def safeClickTab(tab):
             pass
     test.fatal("Tab %s is not being shown." % tab)
     return False
-
-def init():
-    removeQmlDebugFolderIfExists()
-
-def cleanup():
-    removeQmlDebugFolderIfExists()
